//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// DownloadListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "DownloadListCtrl.h"
#include "otherfunctions.h" 
#include "updownclient.h"
#include "opcodes.h"
#ifdef _USE_NEW_DETAILS_
#include ".\\Details\\ClientDetails.h"
#include ".\\Details\\FileDetails.h"
#else
#include "ClientDetailDialog.h"
#include "FileDetailDialog.h"
#endif
#include "commentdialoglst.h"
#include "memcpy_amd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

bool IsAskedForAnotherFile(ST_CTRLITEM *pItem)
{
	if(pItem == NULL)
	{
		return false;	// hmm....
	}

	if(pItem->eType == AVAILABLE_SOURCE || pItem->eType == UNAVAILABLE_SOURCE)
	{
		CUpDownClient* pClient = (CUpDownClient*)pItem->pValue;
		CPartFile* pParent = (CPartFile*)pItem->pOwner;
		
		return (pClient->reqfile != pParent);
	}

	return false;
}

CPartFile* GetFile(ST_CTRLITEM *pItem)
{
	if(pItem == NULL)
	{
		return NULL;
	}
	if(pItem->eType == FILE_TYPE)
	{
		return (CPartFile*)pItem->pValue;
	}
	return (CPartFile*)pItem->pParent->pValue;
}

// CDownloadListCtrl

#define DLC_DT_TEXT (DT_LEFT|DT_SINGLELINE|DT_VCENTER|DT_NOPREFIX|DT_END_ELLIPSIS)
#define DLC_BARUPDATE 512


IMPLEMENT_DYNAMIC(CDownloadListCtrl, CListBox)
CDownloadListCtrl::CDownloadListCtrl() 
{
	m_CurrentSortItem = 0;
	m_SortAscending = true;
	m_ShowSrc=false;  // Cax2
	m_lastSort=0;
	m_sortTime=0;
	m_dblclicked=false; // DonGato
}

CDownloadListCtrl::~CDownloadListCtrl()
{
	EMULE_TRY

	if(m_PrioMenu)
	{ 
		VERIFY( m_PrioMenu.DestroyMenu() );
	}
	if(m_FileMenu) 
	{
		VERIFY( m_FileMenu.DestroyMenu() );
	}
	
	while(m_ListItems.empty() == false)
	{
		delete m_ListItems.begin()->second; // second = ST_CTRLITEM*
		m_ListItems.erase(m_ListItems.begin());
	}	

	m_ImageList.DeleteImageList();	//eklmn: bugfix(01): resource cleanup due to ImageLists recreation

	EMULE_CATCH
}

void CDownloadListCtrl::Init()
{
	EMULE_TRY

	CImageList ilDummyImageList;
	ilDummyImageList.Create(1, 17, ILC_COLOR32 | ILC_MASK, 1, 1); 
	SetImageList(&ilDummyImageList, LVSIL_SMALL);
	ASSERT( (GetStyle() & LVS_SHAREIMAGELISTS) == 0 );
	ilDummyImageList.Detach();
	
	SetStyle();
	SetColors();

	ModifyStyle(LVS_SINGLESEL,0);

	InsertColumn(0,GetResString(IDS_DL_FILENAME),LVCFMT_LEFT, 260);
	InsertColumn(1,GetResString(IDS_DL_SIZE),LVCFMT_RIGHT, 60);
	InsertColumn(2,GetResString(IDS_DL_TRANSF),LVCFMT_RIGHT, 65);
	InsertColumn(3,GetResString(IDS_SF_COMPLETED),LVCFMT_RIGHT, 65);
	InsertColumn(4,GetResString(IDS_DL_SPEED),LVCFMT_RIGHT, 65);
	InsertColumn(5,GetResString(IDS_DL_PROGRESS),LVCFMT_LEFT, 170);
	InsertColumn(6,GetResString(IDS_DL_SOURCES),LVCFMT_RIGHT, 50);
	InsertColumn(7,GetResString(IDS_PRIORITY),LVCFMT_LEFT, 55);
	InsertColumn(8,GetResString(IDS_STATUS),LVCFMT_LEFT, 70);
	InsertColumn(9,GetResString(IDS_DL_REMAINS),LVCFMT_LEFT, 110);
	
// PLUS -----------------------------------------------------------------------
	InsertColumn(10,GetResString(IDS_TIME),LVCFMT_LEFT, 110); 		// should be est. time or ETA
	InsertColumn(11,GetResString(IDS_DL_ULDL),LVCFMT_RIGHT,60);		//netwolf 05.05.03 (right justified)
	InsertColumn(12,GetResString(IDS_QL_RATING),LVCFMT_RIGHT,60);	//netwolf 05.05.03 (right justified)
//-----------------------------------------------------------------------------
	CString lsctitle = GetResString(IDS_LASTSEENCOMPL);
	lsctitle.Remove(':');
	InsertColumn(13, lsctitle,LVCFMT_LEFT, 110);
	lsctitle = GetResString(IDS_FD_LASTCHANGE);
	lsctitle.Remove(':');
	InsertColumn(14, lsctitle, LVCFMT_LEFT, 220);
	InsertColumn(15, GetResString(IDS_CAT), LVCFMT_LEFT, 100);

	m_ImageList.Create(16,16,ILC_COLOR32|ILC_MASK,0,10);
	m_ImageList.SetBkColor(CLR_NONE);
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_NORMAL),16,16));					// 0
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_COMPROT),16,16));				// 1
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_MLDONKEY),16,16));				// 2
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_EDONKEYHYBRID),16,16));  		// 3
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_CREDIT_DOWNESTIMATED),16,16));	// 4
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_FRIEND_ONLY),16,16));      		// 5
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_BANNED_ONLY),16,16));      		// 6
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTPAUSED),16,16));  			// 7
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS1),16,16));         			// 8
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS2),16,16));         			// 9
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS3),16,16));         			// 10
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS4),16,16));         			// 11
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_DCS5),16,16));         			// 12
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTMINUS),16,16));	  			// 13
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTNONE),16,16));	  			// 14
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTPLUS),16,16));	  			// 15
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_NO),16,16));				// 16
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_EXCELLENT),16,16));		// 17
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_GOOD),16,16));			// 18
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_FAIR),16,16));	  		// 19
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_POOR),16,16));			// 20
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_RATING_FAKE),16,16));	  		// 21
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_UNKNOWN),16,16));  				// 22
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_CDONKEY),16,16));  				// 23
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LMULE),16,16));  				// 24
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_SHAREAZA),16,16));  				// 25
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTCOMPLETE),16,16));  			// 26
    m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_LISTSTOPPED),16,16));  			// 27
	m_ImageList.Add(LoadImgLstIcon(MAKEINTRESOURCE(IDI_SECUREHASH),16,16));  			// 28
	LoadImgLstIcon(NULL,0,0); //Destroy last icon

	LoadSettings(CPreferences::tableDownload);
	m_nCurTab = 0;

	//sortdlg Cax2
	if(theApp.glob_prefs->DoUseSort())
	{	
		SortInit(99);
	}
	else 
	{
		// Barry - Use preferred sort order from preferences
		int sortItem = theApp.glob_prefs->GetColumnSortItem(CPreferences::tableDownload);
		bool sortAscending = theApp.glob_prefs->GetColumnSortAscending(CPreferences::tableDownload);
		SetSortArrow(sortItem, sortAscending);
		SortItems(SortProc, sortItem + (sortAscending ? 0:100));	//Cax2 - more than 10 columns
	}

	EMULE_CATCH
}

void CDownloadListCtrl::Localize() 
{
	EMULE_TRY

	if(GetSafeHwnd())
	{
		CHeaderCtrl* pHeaderCtrl = GetHeaderCtrl();
		HDITEM hdi;
		hdi.mask = HDI_TEXT;
		CString strRes;

		strRes = GetResString(IDS_DL_FILENAME);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(0, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_DL_SIZE);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(1, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_DL_TRANSF);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(2, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_SF_COMPLETED);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(3, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_DL_SPEED);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(4, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_DL_PROGRESS);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(5, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_DL_SOURCES);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(6, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_PRIORITY);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(7, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_STATUS);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(8, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_DL_REMAINS);	// netwolf 14.05.03
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(9, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_TIME); // should be est. time
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(10, &hdi);
		strRes.ReleaseBuffer();

		// Maverick
		strRes = GetResString(IDS_DL_ULDL);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(11, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_QL_RATING);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(12, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_LASTSEENCOMPL);
		strRes.Remove(':');
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(13, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_FD_LASTCHANGE);
		strRes.Remove(':');
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(14, &hdi);
		strRes.ReleaseBuffer();

		strRes = GetResString(IDS_CAT);
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(15, &hdi);
		strRes.ReleaseBuffer();

		ShowFilesCount();
	}

	EMULE_CATCH
}

void CDownloadListCtrl::AddFile(CPartFile* toadd)
{
	EMULE_TRY

	if(!toadd)
	{
		return;
	}

	// Create new Item
        ST_CTRLITEM* newitem = new ST_CTRLITEM;
        uint16 itemnr = GetItemCount();
        newitem->pOwner = NULL;
        newitem->eType = FILE_TYPE;
        newitem->pValue = toadd;
        newitem->pParent = NULL;
		newitem->dwUpdated = 0; 

		// The same file shall be added only once
		ASSERT(m_ListItems.find(toadd) == m_ListItems.end());
		m_ListItems.insert(ListItemsPair(toadd, newitem));

		if (CheckShowItemInGivenCat(toadd, m_nCurTab ))
			InsertItem(LVIF_PARAM|LVIF_TEXT,itemnr,LPSTR_TEXTCALLBACK,0,0,0,(LPARAM)newitem);

		ShowFilesCount();

	EMULE_CATCH
}

void CDownloadListCtrl::AddSource(CPartFile* pOwner,CUpDownClient* source,bool notavailable)
{
	EMULE_TRY

	if(!pOwner || !source)
	{
		return;
	}

		// Create new Item
        ST_CTRLITEM* newitem = new ST_CTRLITEM;
        newitem->pOwner = pOwner;
        newitem->eType = (notavailable) ? UNAVAILABLE_SOURCE : AVAILABLE_SOURCE;
        newitem->pValue = source;
		newitem->dwUpdated = 0; 

		// Update cross link to the owner
		ListItems::const_iterator ownerIt = m_ListItems.find(pOwner);
		ASSERT(ownerIt != m_ListItems.end());
		ST_CTRLITEM* ownerItem = ownerIt->second;
		ASSERT(ownerItem->pValue == pOwner);
		newitem->pParent = ownerItem;

		// The same source could be added a few time but only one time per file 
		{
			// Update the other instances of this source
			bool bFound = false;
			std::pair<ListItems::const_iterator, ListItems::const_iterator> rangeIt = m_ListItems.equal_range(source);
			for(ListItems::const_iterator it = rangeIt.first; it != rangeIt.second; it++){
				ST_CTRLITEM* cur_item = it->second;

				// Check if this source has been already added to this file => to be sure
				if(cur_item->pOwner == pOwner){
					// Update this instance with its new setting
					cur_item->eType = newitem->eType;
					cur_item->dwUpdated = 0;
					bFound = true;
				}
				else if(notavailable == false){
					// The state 'Available' is exclusive
					cur_item->eType = UNAVAILABLE_SOURCE;
					cur_item->dwUpdated = 0;
				}
			}

			if(bFound == true){
				delete newitem; 
				return;
			}
		}
		m_ListItems.insert(ListItemsPair(source, newitem));

	EMULE_CATCH
}


void CDownloadListCtrl::RemoveSource(CUpDownClient* source,CPartFile* pOwner)
{
	EMULE_TRY

	if(!theApp.emuledlg->IsRunning()) 
	{
		return;
	}
	if(!source)
	{
		return;
	}

	// Retrieve all entries matching the source
	std::pair<ListItems::iterator, ListItems::iterator> rangeIt = m_ListItems.equal_range(source);
	for(ListItems::iterator it = rangeIt.first; it != rangeIt.second; )
	{
		ST_CTRLITEM* delItem  = it->second;
		if(pOwner == NULL || pOwner == delItem->pOwner)
		{
			// Remove it from the m_ListItems			
			it = m_ListItems.erase(it);

			// Remove it from the CListCtrl
 			LVFINDINFO find;
			find.flags = LVFI_PARAM;
			find.lParam = (LPARAM)delItem;
			sint16 result = FindItem(&find);
			if(result != -1)
			{
				DeleteItem(result);
			}

			// finally it could be delete
			delete delItem;
		}
		else
		{
			it++;
		}
	}

	EMULE_CATCH
}

void CDownloadListCtrl::RemoveFile(CPartFile* toremove)
{
	EMULE_TRY

	if (!theApp.emuledlg->IsRunning()) 
	{
		return;
	}
	if(!toremove)
	{
		return;
	}
	// Retrieve all entries matching the File or linked to the file
	// Remark: The 'asked another files' clients must be removed from here
	ASSERT(toremove != NULL);
	for(ListItems::iterator it = m_ListItems.begin(); it != m_ListItems.end();)
	{
		ST_CTRLITEM* delItem = it->second;
		if(delItem->pOwner == toremove || delItem->pValue == toremove)
		{
			// Remove it from the m_ListItems
			it = m_ListItems.erase(it);

			// Remove it from the CListCtrl
			LVFINDINFO find;
			find.flags = LVFI_PARAM;
			find.lParam = (LPARAM)delItem;
			sint16 result = FindItem(&find);
			if(result != -1)
			{
				DeleteItem(result);
			}

			// finally it could be delete
			delete delItem;
		}
		else 
		{
			it++;
		}
	}
	ShowFilesCount();
	
	EMULE_CATCH
}

void CDownloadListCtrl::UpdateItem(void* toupdate)
{
	EMULE_TRY
	
	if(!::IsWindow(GetSafeHwnd()))
	{
		return;
	}

	if (theApp.emuledlg->IsRunning()) 
	{

		// Retrieve all entries matching the source
		std::pair<ListItems::const_iterator, ListItems::const_iterator> rangeIt = m_ListItems.equal_range(toupdate);
		for(ListItems::const_iterator it = rangeIt.first; it != rangeIt.second; it++)
		{
			ST_CTRLITEM* updateItem  = it->second;

			// Find entry in CListCtrl and update object
 			LVFINDINFO find;
			find.flags = LVFI_PARAM;
			find.lParam = (LPARAM)updateItem;
			sint16 result = FindItem(&find);
			if(result != -1)
			{
				updateItem->dwUpdated = 0;
				Update(result);
			}
		}
	}
	
	EMULE_CATCH
}

#define WARN_PERIOD_OF_NO_PROGRESS 7

void CDownloadListCtrl::DrawFileItem(CDC *dc, int nColumn, LPRECT lpRect, ST_CTRLITEM *lpCtrlItem) 
{
	EMULE_TRY

	if(lpCtrlItem->eType != FILE_TYPE)
		return;

	if(lpRect->left < lpRect->right) 
	{

		CString buffer;
		CPartFile *lpPartFile = (CPartFile*)lpCtrlItem->pValue;

		switch(nColumn)
		{
			case 0:		// file name
				{
					POINT ipoint = { lpRect->left, lpRect->top + 1 };
					int iIcon = 14;
					if(lpPartFile->GetStatus()==PS_COMPLETE)
						iIcon = 26;
					else if(lpPartFile->GetStatus()==PS_STOPPED)
						iIcon = 27;
					else if(lpPartFile->GetStatus()==PS_PAUSED)
						iIcon = 7;
					else
						iIcon = (lpPartFile->GetSourceCount() > 0 || lpPartFile->GetSrcA4AFCount() > 0) ? (lpPartFile->srcarevisible ? 13 : 15) : 14;
					m_ImageList.Draw(dc, iIcon, ipoint, ILD_NORMAL);
					lpRect->left += 18;

					// File Type
					if(theApp.glob_prefs->ShowFileTypeIcon())
					{
						int iImage = theApp.GetFileTypeSystemImageIdx(lpPartFile->GetFileName());
						if (theApp.GetSystemImageList() != NULL)
							::ImageList_Draw(theApp.GetSystemImageList(), iImage, dc->GetSafeHdc(), lpRect->left, lpRect->top, ILD_TRANSPARENT);
						lpRect->left += 19;
					}

                    // now comments/ratings
					if (!theApp.glob_prefs->GetDisableRatingIcons()) {
						if ( lpPartFile->HasComment() || lpPartFile->HasRating() )
						{
							int image=16;
							if (lpPartFile->HasRating())
							{
				        		switch(lpPartFile->GetRating())
				        		{
									case 0: 
										image=16; 
										break; 
									case 1: 
										image=21;
										break; 
									case 2: 
										image=20; 
										break; 
									case 3: 
										image=18; 
										break; 
									case 4: 
										image=19; 
										break; 
									case 5: 
										image=17; 
										break;
									default:
										image=16;
								}
							}
							POINT point= {lpRect->left-4,lpRect->top+3};
							m_ImageList.Draw(dc, image, point, ILD_NORMAL);

						}
						lpRect->left+=10;
					}

                    // obaldin: warn if file had no progress
                    // for the last week
                    COLORREF crOldTxtColor;
                    bool warn=false;
                    if(lpPartFile->GetStatus()==PS_READY)
					{
                        CTimeSpan ts = CTime::GetCurrentTime() - lpPartFile->GetLastDownTransfer();
                        if(ts.GetTotalHours()>24*WARN_PERIOD_OF_NO_PROGRESS)
                            warn = true;
                    }
                    if(warn)
                        crOldTxtColor = dc->SetTextColor((COLORREF)RGB(190,60,60));
					else if(lpPartFile->GetStatus()==PS_PAUSED || lpPartFile->GetStatus()==PS_STOPPED)
						crOldTxtColor = dc->SetTextColor(::GetSysColor(COLOR_GRAYTEXT));
					else 
						crOldTxtColor = dc->SetTextColor( (theApp.glob_prefs->GetCatColor(lpPartFile->GetCategory())>0)?theApp.glob_prefs->GetCatColor(lpPartFile->GetCategory()):(::GetSysColor(COLOR_WINDOWTEXT)) );
				    
					dc->DrawText(lpPartFile->GetFileName(), (int)_tcslen(lpPartFile->GetFileName()),lpRect, DLC_DT_TEXT);
                    
					
					if(warn)
                        dc->SetTextColor(crOldTxtColor);

					if(theApp.glob_prefs->ShowFileTypeIcon())
						lpRect->left -= 19;
					if (!theApp.glob_prefs->GetDisableRatingIcons())
						lpRect->left -= 10;
					lpRect->left -= 18;

					break;
				}

			case 1:		// size
				buffer = CastItoXBytes(lpPartFile->GetFileSize());
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
				break;

			case 2:		// transfered
				buffer = CastItoXBytes(lpPartFile->GetTransfered());
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
				break;

		    case 3:		// transfered complete
			    buffer = CastItoXBytes(lpPartFile->GetCompletedSize());
			    dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
			    break;

			case 4:		// speed
		       	{
					buffer.Empty();
					if (lpPartFile->GetStatus()!=PS_COMPLETING && lpPartFile->GetStatus()!=PS_COMPLETE)
					{
						uint32 dRate = lpPartFile->GetDatarate();
			       		if (dRate > 10)
							buffer.AppendFormat(_T("%.1f "), dRate/1024.0f);
						//bond006 (smoothed "remaining time" calculation) //SyruS
			       		if(theApp.glob_prefs->GetShowAverageDatarate())
			       		{
							dRate = lpPartFile->GetAvgDatarate(true);
							if (dRate > 10)
								buffer.AppendFormat(_T("(%.2f) "), dRate/1024.0f);
						}
					}
					else	//SyruS show average sessionrate
						buffer.AppendFormat(_T("[%.2f] "), lpPartFile->GetAvgDatarate(false)/1024.0f);
					if (!buffer.IsEmpty())
						buffer.Append(GetResString(IDS_KBYTESEC));
					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
				}
				break;

			case 5:		// progress
				{
					lpRect->bottom --;
					lpRect->top ++;
					int  iWidth = lpRect->right - lpRect->left; 
					int iHeight = lpRect->bottom - lpRect->top;
					if (lpCtrlItem->bmpStatus == (HBITMAP)NULL)
						VERIFY(lpCtrlItem->bmpStatus.CreateBitmap(1, 1, 1, 8, NULL)); 
					CDC cdcStatus; 
					HGDIOBJ hOldBitmap;
					cdcStatus.CreateCompatibleDC(dc); 
					int cx = lpCtrlItem->bmpStatus.GetBitmapDimension().cx; 
					DWORD dwTicks = GetTickCount(); 
					if(lpCtrlItem->dwUpdated + DLC_BARUPDATE < dwTicks || cx !=  iWidth || !lpCtrlItem->dwUpdated)
					{ 
						lpCtrlItem->bmpStatus.DeleteObject(); 
						lpCtrlItem->bmpStatus.CreateCompatibleBitmap(dc,  iWidth, iHeight); 
						lpCtrlItem->bmpStatus.SetBitmapDimension(iWidth,  iHeight); 
						hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->bmpStatus); 

						RECT rec_status; 
						rec_status.left = 0; 
						rec_status.top = 0; 
						rec_status.bottom = iHeight; 
						rec_status.right = iWidth; 
						lpPartFile->DrawStatusBar(&cdcStatus,  &rec_status, theApp.glob_prefs->UseFlatBar()); 

						lpCtrlItem->dwUpdated = dwTicks + (rand() % 128); 
					} else 
						hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->bmpStatus); 

					dc->BitBlt(lpRect->left, lpRect->top, iWidth, iHeight,  &cdcStatus, 0, 0, SRCCOPY); 
					cdcStatus.SelectObject(hOldBitmap);

					if (theApp.glob_prefs->GetUseDwlPercentage())
					{
						// HoaX_69: BEGIN Display percent in progress bar
						COLORREF oldclr = dc->SetTextColor(RGB(255,255,255));
						int iOMode = dc->SetBkMode(TRANSPARENT);
						buffer.Format("%.1f%%", lpPartFile->GetPercentCompleted());

						int iOLeft = lpRect->left;
						lpRect->left += iWidth / 2 - 10; // Close enough
						if(lpPartFile->GetStatus()!=PS_COMPLETE && lpPartFile->GetStatus()!=PS_COMPLETING)
							lpRect->top += 3; // Just a little down
						dc->DrawText(buffer, buffer.GetLength(), lpRect, DLC_DT_TEXT);

						lpRect->left = iOLeft;
						dc->SetBkMode(iOMode);
						dc->SetTextColor(oldclr);
						// HoaX_69: END
					}

					lpRect->bottom ++;
					lpRect->top --;
				}
				break;

			case 6:		// sources
				{
					CString buffer2;
					uint16 ncsc=lpPartFile->GetNotCurrentSourcesCount();
					if(ncsc>0)
						buffer2.Format(_T("%i/"),lpPartFile->GetSourceCount() -ncsc);
					else
						buffer2 = "";
					buffer.Format(_T("%s%i (%i)"),buffer2, lpPartFile->GetSourceCount(), lpPartFile->GetTransferingSrcCount());
					if (((CKnownFile*)lpPartFile)->IsAutoPrioritized())
						((CKnownFile*)lpPartFile)->UpdateUploadAutoPriority();
					// theApp.sharedfiles->UpdateItem(((CKnownFile*)lpPartFile)); // obaldin: what the heck was it doing here? moved to UpdateItem
					dc->DrawText(buffer,(int)_tcslen(buffer),lpRect, DLC_DT_TEXT | DT_RIGHT);
				}
				break;

			case 7:		// prio
				// Tarod (auto priority)
				if (lpPartFile->IsAutoPrioritized())
				{
					lpPartFile->UpdateDownloadAutoPriority();
	
					switch(lpPartFile->GetPriority())
					{
						case PR_LOW:
							dc->DrawText(GetResString(IDS_PRIOAUTOLOW),GetResString(IDS_PRIOAUTOLOW).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_NORMAL:
							dc->DrawText(GetResString(IDS_PRIOAUTONORMAL),GetResString(IDS_PRIOAUTONORMAL).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_HIGH:
							dc->DrawText(GetResString(IDS_PRIOAUTOHIGH),GetResString(IDS_PRIOAUTOHIGH).GetLength(),lpRect, DLC_DT_TEXT);
							break;
					}
				} else {
					switch(lpPartFile->GetPriority())
					{
						case PR_LOW:
							dc->DrawText(GetResString(IDS_PRIOLOW),GetResString(IDS_PRIOLOW).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_NORMAL:
							dc->DrawText(GetResString(IDS_PRIONORMAL),GetResString(IDS_PRIONORMAL).GetLength(),lpRect, DLC_DT_TEXT);
							break;
						case PR_HIGH:
							dc->DrawText(GetResString(IDS_PRIOHIGH),GetResString(IDS_PRIOHIGH).GetLength(),lpRect, DLC_DT_TEXT);
							break;
					}
				}
				break;

			case 8:		// status
				buffer = lpPartFile->GetPartfileStatus();
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				break;

			case 9:	// remaining size and %
				if (!IsColumnHidden(9)) 
				{
					// size 
					uint32 remains;
					remains=lpPartFile->GetFileSize()-lpPartFile->GetCompletedSize();

					if(remains < 0) 
						remains = 0;
					buffer = CastItoXBytes(remains);

					if (lpPartFile->GetStatus()==PS_COMPLETING || lpPartFile->GetStatus()==PS_COMPLETE)
						buffer="";
					else
					{
						float saved = 100.0f - lpPartFile->GetPercentCompleted(); 
						CString str1 = buffer;
						buffer.Format( _T("%s [%.1f%%]"), str1, saved);
					}
					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				}
			    break;

			case 10:	//remaining time
				if (!IsColumnHidden(10)) 
				{
					sint32 restTime;
					buffer.Empty();
					if (lpPartFile->GetStatus()!=PS_COMPLETING || lpPartFile->GetStatus()!=PS_COMPLETE)
					{
						restTime = lpPartFile->GetTimeRemaining();
						buffer.AppendFormat(_T("%s"), CastSecondsToHM(restTime));
						//bond006 (smoothed "remaining time" calculation) //SyruS
						if(theApp.glob_prefs->GetShowAverageDatarate())
						{
							restTime = lpPartFile->GetTimeRemaining(true);
							buffer.AppendFormat(_T(" (%s)"), CastSecondsToHM(restTime));
						}
					}
					else
					{	//SyruS show total sessiontransfer and sessiontime
						restTime = lpPartFile->GetFlushTimeSpan().GetTotalSeconds();
						buffer.AppendFormat(_T("[%s / %s]"), CastItoXBytes(lpPartFile->GetSessionTransfered()), CastSecondsToHM(restTime));
					}
					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				}
				break;

			case 13:	// last seen complete
				if (!IsColumnHidden(13)) 
				{
					if (lpPartFile->lastseencomplete==NULL)
						buffer=GetResString(IDS_NEVER);
					else
					buffer = lpPartFile->localelastseencomplete();

					dc->DrawText(buffer,(int)_tcslen(buffer),lpRect, DLC_DT_TEXT);
				}
				break;

			case 14: // last receive
				if (!IsColumnHidden(14)) 
				{
					if(lpPartFile->GetFileDate() != NULL)
					{					
						CTime ctTime(lpPartFile->GetFileDate());
						SYSTEMTIME st;
						ctTime.GetAsSystemTime(st);
						COleDateTime odtTime(st);
						dc->DrawText(odtTime.Format(), lpRect, DLC_DT_TEXT);
					}
				}
				break;

			case 15: // cat
				if (!IsColumnHidden(15)) 
				{

					if(lpPartFile->GetCategory() == 0)
					{
						buffer = GetResString(IDS_ALLOTHERS);
					}
					else
					{
						Category_Struct *pCatStruct = theApp.glob_prefs->GetCategory(lpPartFile->GetCategory());
						if(pCatStruct != NULL)
						{
							buffer = pCatStruct->title;
						}
						else
						{
							lpPartFile->SetCategory(0);
							buffer = GetResString(IDS_ALLOTHERS);
						}
					}

					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				}
				break;
				
			default:
					break;
		}
	}

	EMULE_CATCH
}

void CDownloadListCtrl::DrawSourceItem(CDC *dc, int nColumn, LPRECT lpRect, ST_CTRLITEM *lpCtrlItem) 
{
	EMULE_TRY

	if(lpCtrlItem == NULL)
		return;
	if(lpCtrlItem->eType != AVAILABLE_SOURCE && lpCtrlItem->eType != UNAVAILABLE_SOURCE)
		return;

	if(lpRect->left < lpRect->right) 
	{
	CString buffer;
	CUpDownClient *lpClient = (CUpDownClient*)lpCtrlItem->pValue;
	CString sDownloadState = _T("");
	try{ sDownloadState = lpClient->GetDownloadStateAsString(); }catch(...){}
	uint8 nDownloadState = lpClient->GetDownloadState();
	eClientTypes nClientSoft = lpClient->GetClientSoft();
	uint32 nVersion;	//Cax2 - version nr bugfix
	switch(nClientSoft)
	{
		case SO_EMULE:
		case SO_LMULE:
		case SO_OLDEMULE:
		case SO_CDONKEY:
		case SO_SHAREAZA:
			nVersion= lpClient->GetMuleVersion();
			break;
		default:
			nVersion= lpClient->GetVersion();
	}
	bool bIsBanned = lpClient->IsBanned(); //katsyonak 07/07/03
	bool bIsFriend = lpClient->IsFriend();
	float fRemoteBaseModifier = lpClient->GetRemoteBaseModifier();
	CString sClientName = _T("");
	if(lpClient->GetUserName())
		try{ sClientName = CString(lpClient->GetUserName()); }catch(...){}
	uint32 nTransferredDown = lpClient->GetTransferedDown();
	uint32 nDownloadDataRate = lpClient->GetDownloadDatarate();
	bool bIsEmuleClient = lpClient->IsEmuleClient();
	uint16 nRemoteQueueRank = lpClient->GetRemoteQueueRank();
	uint16 nDifference = lpClient->GetDifference(); //cendre-eau
	bool bNeverAskedBefore = lpClient->GetLastSrcReqTime() == 0;
	bool bRemoteQueueFull = lpClient->IsRemoteQueueFull();
	bool bIsRemoteInfoAvail = lpClient->IsRemoteInfoAvaiable();
	bool bCredits = lpClient->credits != (CClientCredits*)NULL && lpClient->credits != (CClientCredits*)-1;
	uint16 nRemoteRatio = lpClient->GetRemoteRatio();
	bool a4af=IsAskedForAnotherFile(lpCtrlItem);

	// Lord KiRon - just make compiler happy :)
 	POINT point;
 	CString status;
	
	switch(nColumn) 
	{
		case 0:  	// icon, name
			// Lord Kiron
  			{
				
				RECT cur_rec;
				#ifndef AMD
				memcpy(&cur_rec, lpRect, sizeof(RECT));
				#else
				memcpy_amd(&cur_rec, lpRect, sizeof(RECT));
				#endif
				cur_rec.left += 18;
				point.x = cur_rec.left;
				point.y = cur_rec.top+1;
   				status = _T(""); 
				if(!a4af)
				{
					switch (nDownloadState)
					{
						case DS_CONNECTING:
						case DS_CONNECTED:
						case DS_WAITCALLBACK:
						case DS_TOOMANYCONNS:
							m_ImageList.Draw(dc, 10, point, ILD_NORMAL);
							break;
						case DS_ONQUEUE:
							m_ImageList.Draw(dc, 9, point, ILD_NORMAL);
							break;
						case DS_DOWNLOADING:
						case DS_REQHASHSET:
							m_ImageList.Draw(dc, 8, point, ILD_NORMAL);
							break;
						case DS_NONEEDEDPARTS:
						case DS_LOWTOLOWIP:
							m_ImageList.Draw(dc, 11, point, ILD_NORMAL);
							break;
						default:
							m_ImageList.Draw(dc, 12, point, ILD_NORMAL);
					}
				} else {
               		m_ImageList.Draw(dc, 11, point, ILD_NORMAL); 
					if(theApp.glob_prefs->IsA4AFStringEnabled()) {
						//Cax2 - get local file name
						try{ status.Format(_T("(%s)"),lpClient->reqfile->GetFileName()); }catch(...){}
					} else
						status = _T("(") + GetResString(IDS_ASKED4ANOTHERFILE) + _T(")");
  				}
	
				cur_rec.left += 20;

                POINT point2= {cur_rec.left,cur_rec.top+1};

				// Display client icon
				switch(nClientSoft) {
					case SO_CDONKEY:
						m_ImageList.Draw(dc, 23, point2, ILD_NORMAL);
						break;
					case SO_LMULE:
						m_ImageList.Draw(dc, 24, point2, ILD_NORMAL);
						break;
					case SO_SHAREAZA:
						m_ImageList.Draw(dc, 25, point2, ILD_NORMAL);
						break;
					case SO_EDONKEYHYBRID:
						m_ImageList.Draw(dc, 3, point2, ILD_NORMAL);
						break;
					case SO_MLDONKEY:
						m_ImageList.Draw(dc, 2, point2, ILD_NORMAL);
						break;
					case SO_EMULE:
					case SO_OLDEMULE:
						if (lpClient->credits->GetCurrentIdentState(lpClient->GetIP()) != IS_IDENTIFIED)
							m_ImageList.Draw(dc, 28, point2, ILD_NORMAL);
						else
							m_ImageList.Draw(dc, 1, point2, ILD_NORMAL);
						break;
					case SO_EDONKEY:
						m_ImageList.Draw(dc, 0, point2, ILD_NORMAL);
						break;
					default:
						m_ImageList.Draw(dc, 22, point2, ILD_NORMAL);
						break;
				}

				// Display any Extra Overlay icons
				if (bIsBanned) //katsyonak 07/07/03
					m_ImageList.Draw(dc, 6, point2, ILD_TRANSPARENT);
				else if (bIsFriend)
					m_ImageList.Draw(dc, 5, point2, ILD_TRANSPARENT);
				else if (fRemoteBaseModifier >= 1.1)
					m_ImageList.Draw(dc, 4, point2, ILD_TRANSPARENT);

				cur_rec.left += 20;
				
				//CString sName;
				CString buffer1;
				if (sClientName.IsEmpty())
				{
					buffer1.Format(_T("?  (%s)"),GetResString(IDS_UNKNOWN));
				} else {
					buffer1.Format(_T("%s %s"), sClientName, status);
				}

                // DonGato: different color for A4AF sources
				COLORREF crOldTxtColor;
				if(a4af)
					crOldTxtColor = dc->SetTextColor(GetSysColor(COLOR_GRAYTEXT));
				dc->DrawText(buffer1,buffer1.GetLength(),&cur_rec, DLC_DT_TEXT);
                if(a4af)
                	dc->SetTextColor(crOldTxtColor);
			}
			break;
			// Lord KiRon - end

		case 1:		// size
			break;

		case 2:		// transfered
			if (!a4af && nTransferredDown && !theApp.glob_prefs->IsTransferredOnCompleted()) {
				buffer = CastItoXBytes(nTransferredDown);
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
			}
				break;

		case 3:		// completed
			if (!a4af && nTransferredDown && theApp.glob_prefs->IsTransferredOnCompleted()) {
				buffer = CastItoXBytes(nTransferredDown);
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
			}
			break;

		case 4:		// speed
			if (!a4af)
			{
				if (nDownloadDataRate==0)
					buffer="";
				else
					buffer.Format(_T("%.1f %s"), nDownloadDataRate/1024.0f,GetResString(IDS_KBYTESEC));
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
			}
			break;

		case 5:		// file info
			{
				lpRect->bottom--; 
				lpRect->top++; 

				int iWidth = lpRect->right - lpRect->left; 
				int iHeight = lpRect->bottom - lpRect->top; 
				if (lpCtrlItem->bmpStatus == (HBITMAP)NULL)
					VERIFY(lpCtrlItem->bmpStatus.CreateBitmap(1, 1, 1, 8, NULL)); 
				CDC cdcStatus; 
				HGDIOBJ hOldBitmap;
				cdcStatus.CreateCompatibleDC(dc); 
				int cx = lpCtrlItem->bmpStatus.GetBitmapDimension().cx; 
				DWORD dwTicks = GetTickCount(); 
				if(lpCtrlItem->dwUpdated + DLC_BARUPDATE < dwTicks || cx !=  iWidth  || !lpCtrlItem->dwUpdated)
				{ 
					lpCtrlItem->bmpStatus.DeleteObject(); 
					lpCtrlItem->bmpStatus.CreateCompatibleBitmap(dc,  iWidth, iHeight); 
					lpCtrlItem->bmpStatus.SetBitmapDimension(iWidth,  iHeight); 
					hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->bmpStatus); 

					RECT rec_status; 
					rec_status.left = 0; 
					rec_status.top = 0; 
					rec_status.bottom = iHeight; 
					rec_status.right = iWidth; 
					try{
					lpClient->DrawStatusBar(&cdcStatus,  &rec_status,(a4af), theApp.glob_prefs->UseFlatBar()); 
					}catch(...){} // in case client has been deleted meanwhile

					lpCtrlItem->dwUpdated = dwTicks + (rand() % 128); 
				} else 
					hOldBitmap = cdcStatus.SelectObject(lpCtrlItem->bmpStatus); 

				dc->BitBlt(lpRect->left, lpRect->top, iWidth, iHeight,  &cdcStatus, 0, 0, SRCCOPY); 
				cdcStatus.SelectObject(hOldBitmap);

				lpRect->bottom++; 
				lpRect->top--; 
			}
			break;

		case 6:		// sources
			{
				CString clientVerString = GetClientNameAndVersionString(lpClient);
				// [/Lord KiRon]
				dc->DrawText(clientVerString,clientVerString.GetLength(),lpRect, DLC_DT_TEXT);
				break;
			}

		case 7:		// prio
			if ((bIsEmuleClient || nRemoteQueueRank)&& nDownloadState==DS_ONQUEUE && !a4af) //now emule v ? display the QR too!
			{
				if(bRemoteQueueFull)
				{
					buffer=GetResString(IDS_QUEUEFULL);
					dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
				}
				else
				{
					if (nRemoteQueueRank)
					{
						//cendre-eau
						COLORREF crOldTxtColor;
						if (nDifference==nRemoteQueueRank)
						{
							
							//if(bNeverAskedBefore)
							//{
								crOldTxtColor = dc->SetTextColor((COLORREF)RGB(0,0,0));
								buffer.Format("QR: %u (%u)",nRemoteQueueRank,(nRemoteQueueRank-nDifference));
							/*} else {
								crOldTxtColor = dc->SetTextColor((COLORREF)RGB(0,0,255));
								buffer.Format("QR: %u (%u)",nRemoteQueueRank,(nRemoteQueueRank-nDifference));
							}*/
						}      
						else if (nDifference<nRemoteQueueRank)
						{  
							crOldTxtColor = dc->SetTextColor((COLORREF)RGB(190,60,60));
							buffer.Format("QR: %u (+%u)",nRemoteQueueRank,(nDifference));
							if (nDifference==0)
							{
								crOldTxtColor = dc->SetTextColor((COLORREF)RGB(5,65,195));
								buffer.Format("QR: %u (%u)",nRemoteQueueRank,(nDifference));
							}
						} else {
							crOldTxtColor = dc->SetTextColor((COLORREF)RGB(10,160,70));
							buffer.Format("QR: %u (-%u)",nRemoteQueueRank,(65536-nDifference));
						}
						dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
						crOldTxtColor = dc->SetTextColor((COLORREF)RGB(0,0,0));
						//cendre-eau
					}
					else
					{
						//buffer="";
						dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
					}
				}
			} else {
				//	buffer="";
				dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT);
			}
			break;

		case 8:		// status
			// Lord KiRon
			RECT cur_rec;
			#ifndef AMD
			memcpy(&cur_rec, lpRect, sizeof(RECT));
			#else
			memcpy_amd(&cur_rec, lpRect, sizeof(RECT));
			#endif
			cur_rec.left += 6;
			point.x = cur_rec.left;
			point.y = cur_rec.top+1;

			dc->DrawText(sDownloadState,sDownloadState.GetLength(),&cur_rec, DLC_DT_TEXT);
			// Lord KiRon - end
			
		case 9:		// remaining size
			break;
		case 10:	// remaining time
			break;
		case 11: // UL/DL Ratio Maverick
			if (bIsRemoteInfoAvail || bCredits)
				buffer.Format(_T("%0.1f"), fRemoteBaseModifier);
			else buffer="";
			dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
			break;
		case 12: // Ratio total Maverick
			if (bIsRemoteInfoAvail || bCredits)
				buffer.Format(_T("%ld"), nRemoteRatio);
			else buffer="";
			dc->DrawText(buffer,buffer.GetLength(),lpRect, DLC_DT_TEXT | DT_RIGHT);
			break;
		}
	}
	EMULE_CATCH
}

void CDownloadListCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	EMULE_TRY

	if (!theApp.emuledlg->IsRunning())
		return;
	if (!lpDrawItemStruct->itemData)
		return;

	if(theApp.glob_prefs->DoUseSrcSortCol2() && theApp.glob_prefs->DoUseSort() && m_ShowSrc && m_sortTime+1000<::GetTickCount()) 
	{	
		SortInit(false); // Cax2 sortdlg
	}

	CDC* odc = CDC::FromHandle(lpDrawItemStruct->hDC);
	ST_CTRLITEM* content = (ST_CTRLITEM*)lpDrawItemStruct->itemData;
	BOOL bCtrlFocused = ((GetFocus() == this) || (GetStyle() & LVS_SHOWSELALWAYS));

	if ((content->eType == FILE_TYPE) && (lpDrawItemStruct->itemAction | ODA_SELECT) && (lpDrawItemStruct->itemState & ODS_SELECTED))
	{
		if(bCtrlFocused)
			odc->SetBkColor(m_crHighlight);
		else
			odc->SetBkColor(m_crNoHighlight);
	} else
		odc->SetBkColor(GetBkColor());

	CMemDC dc(odc,&CRect(lpDrawItemStruct->rcItem));
	CFont *pOldFont = dc->SelectObject(GetFont());
	COLORREF crOldTextColor = dc->SetTextColor(m_crWindowText);

	BOOL notLast = lpDrawItemStruct->itemID + 1 != GetItemCount();
	BOOL notFirst = lpDrawItemStruct->itemID != 0;
	int tree_start=0;
	int tree_end=0;

	RECT cur_rec;
	#ifndef AMD
	memcpy(&cur_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
	#else
	memcpy_amd(&cur_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
	#endif

	//offset was 4, now it's the standard 2 spaces
	int iOffset = dc->GetTextExtent(_T(" "), 1 ).cx * 2;
	CHeaderCtrl *pHeaderCtrl = GetHeaderCtrl();
	int iCount = pHeaderCtrl->GetItemCount();
	cur_rec.right = cur_rec.left;
	cur_rec.right -= iOffset;
	cur_rec.left += iOffset/2;

	if (content->eType == FILE_TYPE)
	{		
		for(int iCurrent = 0; iCurrent < iCount; iCurrent++) 
		{
			int iColumn = pHeaderCtrl->OrderToIndex(iCurrent);
			int cx = CListCtrl::GetColumnWidth(iColumn);
			if(iColumn == 5)
			{
				int iNextLeft = cur_rec.left + cx;
				//set up tree vars
				cur_rec.left = cur_rec.right + iOffset;
				cur_rec.right = cur_rec.left + min(8, cx);
				tree_start = cur_rec.left + 1;
				tree_end = cur_rec.right;
				//normal column stuff
				cur_rec.left = cur_rec.right + 1;
				cur_rec.right = tree_start + cx - iOffset;
				DrawFileItem(dc, 5, &cur_rec, content);
				cur_rec.left = iNextLeft;
			} 
			else 
			{
				cur_rec.right += cx;
				DrawFileItem(dc, iColumn, &cur_rec, content);
				cur_rec.left += cx;
			}

		}
	} 
	else if (content->eType == AVAILABLE_SOURCE || content->eType == UNAVAILABLE_SOURCE)
	{
		for(int iCurrent = 0; iCurrent < iCount; iCurrent++)
		{
			int iColumn = pHeaderCtrl->OrderToIndex(iCurrent);
			int cx = CListCtrl::GetColumnWidth(iColumn);

			if(iColumn == 5)
			{
				int iNextLeft = cur_rec.left + cx;
				//set up tree vars
				cur_rec.left = cur_rec.right + iOffset;
				cur_rec.right = cur_rec.left + min(8, cx);
				tree_start = cur_rec.left + 1;
				tree_end = cur_rec.right;
				//normal column stuff
				cur_rec.left = cur_rec.right + 1;
				cur_rec.right = tree_start + cx - iOffset;
				DrawSourceItem(dc, 5, &cur_rec, content);
				cur_rec.left = iNextLeft;
			} 
			else 
			{
				cur_rec.right += cx;
				DrawSourceItem(dc, iColumn, &cur_rec, content);
				cur_rec.left += cx;
			}
		}
	}

	//draw rectangle around selected item(s)
	if ((lpDrawItemStruct->itemAction | ODA_SELECT) &&
		(lpDrawItemStruct->itemState & ODS_SELECTED) && (content->eType == FILE_TYPE))
	{
		RECT outline_rec;
		#ifndef AMD
		memcpy(&outline_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
		#else
		memcpy_amd(&outline_rec,&lpDrawItemStruct->rcItem,sizeof(RECT));
		#endif

		outline_rec.top--;
		outline_rec.bottom++;
		dc->FrameRect(&outline_rec, &CBrush(GetBkColor()));
		outline_rec.top++;
		outline_rec.bottom--;
		outline_rec.left++;
		outline_rec.right--;

		if(notFirst && (GetItemState(lpDrawItemStruct->itemID - 1, LVIS_SELECTED)))
		{
			ST_CTRLITEM* prev = (ST_CTRLITEM*)GetItemData(lpDrawItemStruct->itemID - 1);
			if(prev->eType == FILE_TYPE)
				outline_rec.top--;
		} 

		if(notLast && (GetItemState(lpDrawItemStruct->itemID + 1, LVIS_SELECTED)))
		{
			ST_CTRLITEM* next = (ST_CTRLITEM*)GetItemData(lpDrawItemStruct->itemID + 1);
			if(next->eType == FILE_TYPE)
				outline_rec.bottom++;
		} 

		if(bCtrlFocused)
			dc->FrameRect(&outline_rec, &CBrush(m_crFocusLine));
		else
			dc->FrameRect(&outline_rec, &CBrush(m_crNoFocusLine));
	}
	//draw focus rectangle around non-highlightable items when they have the focus
	else if (((lpDrawItemStruct->itemState & ODS_FOCUS) == ODS_FOCUS) &&
		(GetFocus() == this))
	{
		RECT focus_rec;
		focus_rec.top    = lpDrawItemStruct->rcItem.top;
		focus_rec.bottom = lpDrawItemStruct->rcItem.bottom;
		focus_rec.left   = lpDrawItemStruct->rcItem.left + 1;
		focus_rec.right  = lpDrawItemStruct->rcItem.right - 1;
		dc->FrameRect(&focus_rec, &CBrush(m_crNoFocusLine));
	}

	//draw tree last so it draws over selected and focus (looks better)
	if(tree_start < tree_end)
	{
		//set new bounds
		RECT tree_rect;
		tree_rect.top    = lpDrawItemStruct->rcItem.top;
		tree_rect.bottom = lpDrawItemStruct->rcItem.bottom;
		tree_rect.left   = tree_start;
		tree_rect.right  = tree_end;
		dc->SetBoundsRect(&tree_rect, DCB_DISABLE);

		//gather some information
		BOOL hasNext = notLast &&
			((ST_CTRLITEM*)GetItemData(lpDrawItemStruct->itemID + 1))->eType != FILE_TYPE;
		BOOL isOpenRoot = hasNext && content->eType == FILE_TYPE;
		BOOL isChild = content->eType != FILE_TYPE;
		//BOOL isExpandable = !isChild && ((CPartFile*)content->pValue)->GetSourceCount() > 0;
		//might as well calculate these now
		int treeCenter = tree_start + 3;
		int middle = (cur_rec.top + cur_rec.bottom + 1) / 2;

		//set up a new pen for drawing the tree
		CPen pn, *oldpn;
		pn.CreatePen(PS_SOLID, 1, dc->GetTextColor());
		oldpn = dc->SelectObject(&pn);

		if(isChild)
		{
			//draw the line to the status bar
			dc->MoveTo(tree_end, middle);
			dc->LineTo(tree_start + 3, middle);

			//draw the line to the child node
			if(hasNext)
			{
				dc->MoveTo(treeCenter, middle);
				dc->LineTo(treeCenter, cur_rec.bottom + 1);
			}
		} else if(isOpenRoot) {
			//draw circle
			RECT circle_rec;
			COLORREF crBk = dc->GetBkColor();
			circle_rec.top    = middle - 2;
			circle_rec.bottom = middle + 3;
			circle_rec.left   = treeCenter - 2;
			circle_rec.right  = treeCenter + 3;
			dc->FrameRect(&circle_rec, &CBrush(dc->GetTextColor()));
			dc->SetPixelV(circle_rec.left,      circle_rec.top,    crBk);
			dc->SetPixelV(circle_rec.right - 1, circle_rec.top,    crBk);
			dc->SetPixelV(circle_rec.left,      circle_rec.bottom - 1, crBk);
			dc->SetPixelV(circle_rec.right - 1, circle_rec.bottom - 1, crBk);
			//draw the line to the child node
			if(hasNext)
			{
				dc->MoveTo(treeCenter, middle + 3);
				dc->LineTo(treeCenter, cur_rec.bottom + 1);
			}
		}
		//draw the line back up to parent node
		if(notFirst && isChild)
		{
			dc->MoveTo(treeCenter, middle);
			dc->LineTo(treeCenter, cur_rec.top - 1);
		}

		//put the old pen back
		dc->SelectObject(oldpn);
		pn.DeleteObject();
	}

	//put the original objects back
	dc->SelectObject(pOldFont);
	dc->SetTextColor(crOldTextColor);
	EMULE_CATCH
}

// modifier-keys -view filtering [Ese Juani+xrmb]
void CDownloadListCtrl::HideSources(CPartFile* toCollapse,bool isShift,bool isCtrl,bool isAlt) 
{
	EMULE_TRY

	SetRedraw(false);
	int pre,post;
	pre = post = 0;
	for(int i = 0; i < GetItemCount(); i++) 
	{
		ST_CTRLITEM* item = (ST_CTRLITEM*)this->GetItemData(i);
		if(item->pOwner == toCollapse) 
		{
			pre++;
			if(isShift || isCtrl || isAlt)
			{
				eDLQState ds=((CUpDownClient*)item->pValue)->GetDownloadState();
				if((isShift && ds==DS_DOWNLOADING) ||
					(isCtrl && ((CUpDownClient*)item->pValue)->GetRemoteQueueRank()> 0) ||
					(isAlt && ds!=DS_NONEEDEDPARTS))
				{
					continue;
				}
			}
			item->dwUpdated = 0;
			item->bmpStatus.DeleteObject();
			DeleteItem(i--);
			post++;
		}
	}

	if (m_ShowSrc)			//Cax2 check if we don't show sources anymore...
	{
		m_ShowSrc = false; 
		for(int i = 0; i < GetItemCount(); i++)
		{	
			ST_CTRLITEM* item = (ST_CTRLITEM*)this->GetItemData(i);
			if(item)
			{
				if(item->eType == AVAILABLE_SOURCE || item->eType == UNAVAILABLE_SOURCE)
				{
					m_ShowSrc = true;
					break;
				}
			}
		}
		
		if (theApp.glob_prefs->DoUseSort() && !m_ShowSrc)	//Cax2 ...& restore the sort index that was there before expanding the files
		{
			m_lastSort = FilesSortIndex;
			m_CurrentSortItem = m_lastSort < 100 ? m_lastSort : m_lastSort - 100;
			SetSortArrow(m_CurrentSortItem, (m_lastSort < 100));
		}
	}

	if (pre-post==0)
	{ 
		toCollapse->srcarevisible = false;
	}
	SetRedraw(true);

	EMULE_CATCH
}

BEGIN_MESSAGE_MAP(CDownloadListCtrl, CMuleListCtrl)
	ON_NOTIFY_REFLECT(LVN_ITEMACTIVATE, OnItemActivate)
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED, OnListModified)
	ON_NOTIFY_REFLECT(LVN_INSERTITEM, OnListModified)
	ON_NOTIFY_REFLECT(LVN_DELETEITEM, OnListModified)
	ON_WM_CONTEXTMENU()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnNMDblclkDownloadlist)
	ON_NOTIFY_REFLECT(NM_CLICK, OnNMClick)	
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetDispInfo)
	ON_WM_SIZE()
	ON_WM_KEYUP()
END_MESSAGE_MAP()

void CDownloadListCtrl::ExpandCollapseItem(int item,uint8 expand,bool collapsesource)
{
	EMULE_TRY

	if (item == -1) 
	{
		return;
	}

	ST_CTRLITEM* content = (ST_CTRLITEM*)this->GetItemData(item);

	// modifier-keys -view filtering [Ese Juani+xrmb]
	bool isShift=GetAsyncKeyState(VK_SHIFT) < 0;
	bool isCtrl=GetAsyncKeyState(VK_CONTROL) < 0;
	bool isAlt=GetAsyncKeyState(VK_MENU) < 0;

	if (collapsesource && content->pParent!=NULL) 
	{// to collapse/expand files when one of its source is selected
		content=content->pParent;
		
 		LVFINDINFO find;
		find.flags = LVFI_PARAM;
		find.lParam = (LPARAM)content;
		item = FindItem(&find);
		if (item==-1) return;
	}

	if (!content || content->eType != FILE_TYPE) 
	{
		return;
	}
	
	if(theApp.glob_prefs->DoUseSort() && !m_ShowSrc)		//Cax2 - restore the sort index that was there before opening the files if advanced sort
	{		
		FilesSortIndex = m_lastSort; 
	}
	m_ShowSrc=true;  //newsort added by Cax2 03/11/02 
	
	CPartFile* partfile = reinterpret_cast<CPartFile*>(content->pValue);
	if (!partfile) 
	{
		return;
	}

	if((partfile->GetStatus() == PS_PAUSED || partfile->GetStatus() == PS_STOPPED) && !m_dblclicked)
		partfile->ResumeFile();
	else if (partfile->GetStatus() == PS_COMPLETE && !m_dblclicked) 
		ClearCompleted(partfile->GetFileHash());
	// Check if the source branch is disable
	else if(partfile->srcarevisible == false && partfile->GetStatus() != PS_COMPLETING) 
	{
		if (expand > COLLAPSE_ONLY)
		{
			SetRedraw(false);
			
			// Go throught the whole list to find out the sources for this file
			// Remark: don't use GetSourceCount() => UNAVAILABLE_SOURCE
			for(ListItems::const_iterator it = m_ListItems.begin(); it != m_ListItems.end(); it++)
			{
				const ST_CTRLITEM* cur_item = it->second;
				if(cur_item->pOwner == partfile)
				{
					if(isShift || isCtrl || isAlt) 
					{
						ASSERT(cur_item->eType != FILE_TYPE);
						eDLQState ds=((CUpDownClient*)cur_item->pValue)->GetDownloadState();
						if(!(isShift && ds==DS_DOWNLOADING ||
							isCtrl && ((CUpDownClient*)cur_item->pValue)->GetRemoteQueueRank()>0 ||
							isAlt && ds!=DS_NONEEDEDPARTS))
						{
							continue; // skip this source
						}
					}
					partfile->srcarevisible = true;
					InsertItem(LVIF_PARAM|LVIF_TEXT,item+1,LPSTR_TEXTCALLBACK,0,0,0,(LPARAM)cur_item);
				}
			}

			SetRedraw(true);
		}
	}
	else 
	{
		if(expand == EXPAND_COLLAPSE || expand == COLLAPSE_ONLY) 
		{
			HideSources(partfile,isShift,isCtrl,isAlt);
		}
	}

	SortInit(partfile->srcarevisible&&theApp.glob_prefs->DoUseSort()); //override if useSort & showing sources...

	m_dblclicked = false;

	EMULE_CATCH
}

void CDownloadListCtrl::OnItemActivate(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	ExpandCollapseItem(pNMIA->iItem,2);
	*pResult = 0;
	
	EMULE_CATCH
}

void CDownloadListCtrl::OnContextMenu(CWnd* pWnd, CPoint point)
{
	EMULE_TRY

	CTitleMenu		m_ClientMenu;
	CTitleMenu		m_FileMenu;
	CMenu			m_PrioMenu;
	CMenu			m_ed2kMenu;
	CMenu			m_Web;

	if (GetSelectionMark() != (-1) && GetSelectedCount()>0)
	{
		ST_CTRLITEM* content = (ST_CTRLITEM*)GetItemData(GetSelectionMark());
		const CPartFile* cFile=NULL; //Cax2 bugfix for Dr Slump? start
		if (content->eType == FILE_TYPE) 
		{
			cFile = (CPartFile*)content->pValue;
		}
		if (cFile!=NULL)
		{
			CPartFile* file=(CPartFile*)cFile; //Cax2 bugfix
			bool filedone=(file->GetStatus()==PS_COMPLETE || file->GetStatus()==PS_COMPLETING);
			bool justOne=(GetSelectedCount()==1); //Cax2 faster code
			int status=file->GetStatus();

			theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToFile(file); // FoRcHa
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
			
			UINT uFlagIfDone	=	filedone?MF_GRAYED:MF_ENABLED;
			m_FileMenu.CreatePopupMenu();
			m_FileMenu.AddMenuTitle(GetResString(IDS_DOWNLOADMENUTITLE));

			m_PrioMenu.CreateMenu();
			m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOLOW, GetResString(IDS_PRIOLOW));
			m_PrioMenu.EnableMenuItem(MP_PRIOLOW,uFlagIfDone);
			m_PrioMenu.AppendMenu(MF_STRING,MP_PRIONORMAL,GetResString(IDS_PRIONORMAL));
			m_PrioMenu.EnableMenuItem(MP_PRIONORMAL,uFlagIfDone);
			m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOHIGH, GetResString(IDS_PRIOHIGH));
			m_PrioMenu.EnableMenuItem(MP_PRIOHIGH,uFlagIfDone);
			m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOAUTO, GetResString(IDS_PRIOAUTO));
			m_PrioMenu.EnableMenuItem(MP_PRIOAUTO,uFlagIfDone);
		
			m_ed2kMenu.CreateMenu();
			m_ed2kMenu.AppendMenu(MF_STRING,MP_GETED2KLINK, GetResString(IDS_DL_LINK1));
			m_ed2kMenu.AppendMenu(MF_STRING,MP_GETHTMLED2KLINK, GetResString(IDS_DL_LINK2));
			m_ed2kMenu.AppendMenu(MF_STRING,MP_GETSOURCEED2KLINK, GetResString(IDS_CREATESOURCELINK)); //<<-- enkeyDEV(Ottavio84) -Create source ed2k from downloads-
		
			int counter;
			m_Web.CreateMenu();
			UpdateURLMenu(m_Web,counter);
			UINT flag;
			flag=(counter==0) ? MF_GRAYED:MF_STRING;

			m_FileMenu.AppendMenu(MF_STRING,MP_CLEARCOMPLETED, GetResString(IDS_DL_CLEAR));
			m_FileMenu.AppendMenu(MF_SEPARATOR);

			m_FileMenu.AppendMenu(MF_STRING|MF_POPUP,(UINT_PTR)m_PrioMenu.m_hMenu, GetResString(IDS_PRIORITY));
			m_FileMenu.EnableMenuItem((UINT_PTR)m_PrioMenu.m_hMenu, MF_ENABLED);
			m_PrioMenu.CheckMenuItem(MP_PRIOLOW,(justOne && !file->IsAutoPrioritized() && file->GetPriority() == PR_LOW)? MF_CHECKED:MF_UNCHECKED);
			m_PrioMenu.CheckMenuItem(MP_PRIONORMAL,(justOne && !file->IsAutoPrioritized() && file->GetPriority() == PR_NORMAL)? MF_CHECKED:MF_UNCHECKED);
			m_PrioMenu.CheckMenuItem(MP_PRIOHIGH,(justOne && !file->IsAutoPrioritized() && file->GetPriority() == PR_HIGH)? MF_CHECKED:MF_UNCHECKED);
			m_PrioMenu.CheckMenuItem(MP_PRIOAUTO, (justOne && file->IsAutoPrioritized())? MF_CHECKED:MF_UNCHECKED);
			
			m_FileMenu.AppendMenu(MF_STRING,MP_CANCEL, GetResString(IDS_MAIN_BTN_CANCEL));
			m_FileMenu.EnableMenuItem(MP_CANCEL,(!filedone) ? MF_ENABLED:MF_GRAYED);

			m_FileMenu.AppendMenu(MF_STRING,MP_STOP, GetResString(IDS_DL_STOP));
			m_FileMenu.EnableMenuItem(MP_STOP,((status != PS_STOPPED && status != PS_ERROR && !filedone ) ? MF_ENABLED:MF_GRAYED));

			m_FileMenu.AppendMenu(MF_STRING,MP_PAUSE, GetResString(IDS_DL_PAUSE));
			m_FileMenu.EnableMenuItem(MP_PAUSE,((status != PS_PAUSED && status != PS_STOPPED && status != PS_ERROR && !filedone) ? MF_ENABLED:MF_GRAYED));

			m_FileMenu.AppendMenu(MF_STRING,MP_RESUME, GetResString(IDS_DL_RESUME));
			m_FileMenu.EnableMenuItem(MP_RESUME,((status == PS_PAUSED || status == PS_STOPPED) ? MF_ENABLED:MF_GRAYED));

			m_FileMenu.AppendMenu(MF_SEPARATOR);

			m_FileMenu.AppendMenu(MF_STRING,MP_OPEN, GetResString(IDS_DL_OPEN));
			m_FileMenu.EnableMenuItem(MP_OPEN,((justOne && status == PS_COMPLETE) ? MF_ENABLED:MF_GRAYED)); //<<--9/21/02

			m_FileMenu.AppendMenu(MF_STRING,MP_MOVIE, GetResString(IDS_MOVIE));
			if(file->IsMovie() && justOne && !file->FirstLastLoaded())	//Cax2 - greyed out if we have those chunks
			{
				//m_FileMenu.EnableMenuItem(MP_MOVIE,(file->IsArchive())?MF_GRAYED:MF_ENABLED);
				m_FileMenu.CheckMenuItem(MP_MOVIE,(file->GetMovieMode() != 0)?MF_CHECKED:MF_UNCHECKED);
			} else
				m_FileMenu.EnableMenuItem(MP_MOVIE, MF_GRAYED);

			m_FileMenu.AppendMenu(MF_STRING,MP_PREVIEW, GetResString(IDS_DL_PREVIEW));
			m_FileMenu.EnableMenuItem(MP_PREVIEW,((justOne && file->PreviewAvailable()) ? MF_ENABLED:MF_GRAYED));

			m_FileMenu.AppendMenu(MF_STRING,MP_METINFO, GetResString(IDS_DL_INFO));
			m_FileMenu.EnableMenuItem(MP_METINFO,(justOne)?MF_ENABLED:MF_GRAYED); // Cax2

			m_FileMenu.AppendMenu(MF_STRING,MP_VIEWFILECOMMENTS, GetResString(IDS_CMT_SHOWALL));
			m_FileMenu.EnableMenuItem(MP_VIEWFILECOMMENTS,(justOne && (file->HasComment() || file->HasRating())) /*&& !filedone*/ ?MF_ENABLED:MF_GRAYED); // Cax2  //netwolf: no comments on complete files	// netwolf 06.05.03 only enabled if there is a comment/rating	// #zegzav:pastcomment; allow to see past comments for completed files

			m_FileMenu.AppendMenu(MF_STRING,MP_CHANGEDIR, GetResString(IDS_CHANGEDIR)); //<<-- enkeyDEV(Ottavio84) -ChangeDir-
			m_FileMenu.EnableMenuItem(MP_CHANGEDIR, !filedone ? MF_ENABLED:MF_GRAYED); //<<-- enkeyDEV(Ottavio84) -ChangeDir-

			//SyruS: cleanup
			m_FileMenu.AppendMenu(MF_STRING,MP_DOCLEANUP, GetResString(IDS_DOCLEANUP));
			m_FileMenu.EnableMenuItem(MP_DOCLEANUP, !filedone ? MF_ENABLED:MF_GRAYED);

			m_FileMenu.AppendMenu(MF_SEPARATOR);

			m_FileMenu.AppendMenu(MF_STRING,MP_DROPNONEEDEDSRCS, GetResString(IDS_DROPNONEEDEDSRCS)); // Tarod
			m_FileMenu.EnableMenuItem(MP_DROPNONEEDEDSRCS,((status != PS_COMPLETE && status != PS_COMPLETING)?MF_ENABLED:MF_GRAYED)); // Added by Tarod	// netwolf 07.05.03

            m_FileMenu.AppendMenu(MF_STRING,MP_DROP_SUPERCOMPRESSED, GetResString(IDS_DROPSUPERCOMPRESSED));
			m_FileMenu.CheckMenuItem(MP_DROP_SUPERCOMPRESSED, file->GetDiscardSuperCompressed() ? MF_CHECKED:MF_UNCHECKED);
			m_FileMenu.EnableMenuItem(MP_DROP_SUPERCOMPRESSED, ((status != PS_COMPLETE && status != PS_COMPLETING)?MF_ENABLED:MF_GRAYED));	//netwolf: no dropSCB on complete/ing) files

			m_FileMenu.AppendMenu(MF_STRING,MP_ALL_A4AF_TO_HERE, GetResString(IDS_ALL_A4AF_TO_HERE)); // Tarod
			m_FileMenu.EnableMenuItem(MP_ALL_A4AF_TO_HERE,(justOne && status != PS_COMPLETE && status != PS_COMPLETING)?MF_ENABLED:MF_GRAYED); // Tarod(+Cax2 Patch 21/12/02)	//netwolf 07.05.03

			CString buffer;
			buffer.Format(_T("%s (%s)"), GetResString(IDS_ALL_A4AF_TO_HERE), GetResString(IDS_PRIOAUTO));
			m_FileMenu.AppendMenu(MF_STRING,MP_ALL_A4AF_AUTO,buffer); // Tarod
			if (justOne && status != PS_COMPLETE && status != PS_COMPLETING) // Cax2	// netwolf 07.05.03
			{
				m_FileMenu.CheckMenuItem(MP_ALL_A4AF_AUTO, (file== theApp.downloadqueue->GetA4AFAutoFile()) ? MF_CHECKED:MF_UNCHECKED); // Tarod
				m_FileMenu.EnableMenuItem(MP_ALL_A4AF_AUTO, MF_ENABLED);
			} else {
				m_FileMenu.CheckMenuItem(MP_ALL_A4AF_AUTO, MF_UNCHECKED);
				m_FileMenu.EnableMenuItem(MP_ALL_A4AF_AUTO, MF_GRAYED);
			}

			m_FileMenu.AppendMenu(MF_SEPARATOR);

			m_FileMenu.AppendMenu(MF_STRING|MF_POPUP,(UINT_PTR)m_ed2kMenu.m_hMenu, GetResString(IDS_ED2KLINKFIX));

			m_FileMenu.AppendMenu(flag|MF_POPUP,(UINT_PTR)m_Web.m_hMenu, GetResString(IDS_WEBSERVICES));

			m_FileMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
//			m_FileMenu.RemoveMenu(28,MF_BYPOSITION);
			m_PrioMenu.DestroyMenu();
			m_ed2kMenu.DestroyMenu();
			m_Web.DestroyMenu();
			m_FileMenu.DestroyMenu();
		}
		else
		{
			const CUpDownClient* client = (CUpDownClient*)content->pValue;
			if (client !=NULL) 
				theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToUser((CUpDownClient*)client);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();

			m_ClientMenu.CreatePopupMenu();
			m_ClientMenu.AddMenuTitle(GetResString(IDS_CLIENTS));
			
			m_ClientMenu.AppendMenu(MF_STRING,MP_DETAIL, GetResString(IDS_SHOWDETAILS));
			if(client->IsFriend())
				m_ClientMenu.AppendMenu(MF_STRING,MP_REMOVEFRIEND, GetResString(IDS_REMOVEFRIEND));
			else
				m_ClientMenu.AppendMenu(MF_STRING,MP_ADDFRIEND, GetResString(IDS_ADDFRIEND));
			m_ClientMenu.AppendMenu(MF_STRING,MP_MESSAGE, GetResString(IDS_SEND_MSG));
			m_ClientMenu.AppendMenu(MF_STRING,MP_SHOWLIST, GetResString(IDS_VIEWFILES));
			if(IsAskedForAnotherFile(content))
				m_ClientMenu.AppendMenu(MF_STRING,MP_DOWNNOW, GetResString(IDS_DOWNLOAD_A4AF)); // Ambdribant

			m_ClientMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
			m_ClientMenu.DestroyMenu();
		}
	} else {
		m_FileMenu.CreatePopupMenu();
		m_FileMenu.AddMenuTitle(GetResString(IDS_DOWNLOADMENUTITLE));
		m_FileMenu.AppendMenu(MF_STRING,MP_CLEARCOMPLETED, GetResString(IDS_DL_CLEAR));
		m_FileMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
		m_FileMenu.DestroyMenu();
	}

	EMULE_CATCH
}

BOOL CDownloadListCtrl::OnCommand(WPARAM wParam,LPARAM lParam )
{
	EMULE_TRY

	if (GetSelectionMark() != (-1))
	{
		ST_CTRLITEM* content = (ST_CTRLITEM*)GetItemData(GetSelectionMark());
				
		//for multiple selections 
		UINT selectedCount = GetSelectedCount(); 
		CTypedPtrList<CPtrList, CPartFile*> selectedList; 
		int index = -1; 
		POSITION pos = GetFirstSelectedItemPosition(); 
		while(pos != NULL) 
		{ 
			index = GetNextSelectedItem(pos); 
			if(index > -1) 
			{
				if (((ST_CTRLITEM*)GetItemData(index))->eType == FILE_TYPE) 
				{
					selectedList.AddTail((CPartFile*)((ST_CTRLITEM*)GetItemData(index))->pValue); 
				}
			} 
		} 

		if (content->eType == FILE_TYPE)
		{
			CPartFile* file = (CPartFile*)content->pValue;

			if (wParam>=MP_WEBURL && wParam<=MP_WEBURL+256) 
			{
				RunURL(file, theApp.webservices.GetAt(wParam-MP_WEBURL) );
			}
			if (wParam>=MP_ASSIGNCAT && wParam<=MP_ASSIGNCAT+99) 
			{
				while(!selectedList.IsEmpty()) 
				{ 
					CPartFile *selected = selectedList.GetHead();
					selected->SetCategory(wParam-MP_ASSIGNCAT);
					selectedList.RemoveHead(); 
				}				
				ChangeCategory(m_nCurTab);
			}

			switch (wParam)
			{
				case MPG_DELETE:
				case MP_CANCEL:
				{
					//for multiple selections 
					if(selectedCount > 0)
					{
						SetRedraw(false);
						CString fileList="";
						bool validdelete = false;
						
						for (pos = selectedList.GetHeadPosition() ; pos != 0 ; selectedList.GetNext(pos))
						{
							if(selectedList.GetAt(pos)->GetStatus() != PS_COMPLETING && selectedList.GetAt(pos)->GetStatus() != PS_COMPLETE)
							{
								validdelete = true;
								if (selectedCount<50) fileList.Append("\n"+CString(selectedList.GetAt(pos)->GetFileName())); 
							} 
						}

						CString quest;
						if (selectedCount==1)
						{	
							quest=GetResString(IDS_Q_CANCELDL2);
						}
						else
						{	
							quest=GetResString(IDS_Q_CANCELDL);
						}
						if (validdelete && AfxMessageBox(quest + fileList,MB_ICONQUESTION|MB_YESNO) == IDYES)
						{ 
							while(!selectedList.IsEmpty())
							{
								HideSources(selectedList.GetHead());
								switch(selectedList.GetHead()->GetStatus()) 
								{ 
									case PS_WAITINGFORHASH:
									case PS_HASHING:
									case PS_COMPLETING:
									case PS_COMPLETE:
										selectedList.RemoveHead(); 
										break;
									case PS_STOPPED:
									case PS_PAUSED:
										selectedList.GetHead()->DeleteFile(); 
										selectedList.RemoveHead();
										break;
									default:
										selectedList.GetHead()->DeleteFile(); 
										selectedList.RemoveHead(); 
								}
							}  
						}
						SetRedraw(true);
					}
					break;
				}
				case MP_PRIOHIGH:
					if(selectedCount > 1)
					{
						SetRedraw(false);
						while(!selectedList.IsEmpty()) 
						{ 
							selectedList.GetHead()->SetAutoPriority(false); 
							selectedList.GetHead()->SetPriority(PR_HIGH); 
							selectedList.RemoveHead(); 
						} 
						SetRedraw(true);
						break; 
					} 
					file->SetAutoPriority(false);
					file->SetPriority(PR_HIGH);
					break;
				case MP_PRIOLOW:
					if(selectedCount > 1)
					{
						SetRedraw(false);
						while(!selectedList.IsEmpty()) 
						{ 
							selectedList.GetHead()->SetAutoPriority(false); 
							selectedList.GetHead()->SetPriority(PR_LOW); 
							selectedList.RemoveHead(); 
						}
						SetRedraw(true);
						break; 
					} 
					file->SetAutoPriority(false);
					file->SetPriority(PR_LOW);
					break;
				case MP_PRIONORMAL:
					if(selectedCount > 1)
					{
						SetRedraw(false);
						while(!selectedList.IsEmpty()) 
						{
							selectedList.GetHead()->SetAutoPriority(false);
							selectedList.GetHead()->SetPriority(PR_NORMAL);
							selectedList.RemoveHead();
						}
						SetRedraw(true);
						break;
					} 
					file->SetAutoPriority(false);
					file->SetPriority(PR_NORMAL);
					break;
				case MP_PRIOAUTO:
					if(selectedCount > 1)
					{
						SetRedraw(false);
						while(!selectedList.IsEmpty()) 
						{
							selectedList.GetHead()->SetAutoPriority(true);
							selectedList.GetHead()->SetPriority(PR_HIGH);
							selectedList.RemoveHead();
						}
						SetRedraw(true);
						break;
					} 
					file->SetAutoPriority(true);
					file->SetPriority(PR_HIGH);
					break;
				case MP_PAUSE:
					if(selectedCount > 1)
					{
						SetRedraw(false);
						while(!selectedList.IsEmpty()) 
						{ 
							selectedList.GetHead()->PauseFile(); 
							selectedList.RemoveHead(); 
						} 
						SetRedraw(true);
						break; 
					} 
					file->PauseFile();
					break;
				case MP_RESUME:
					if(selectedCount > 1)
					{
						SetRedraw(false);
						while(!selectedList.IsEmpty()) 
						{ 
							selectedList.GetHead()->ResumeFile(); 
							selectedList.RemoveHead(); 
						}
						SetRedraw(true);
						break; 
					} 
					file->ResumeFile();
					break;
				case MP_STOP:
					if(selectedCount > 1)
					{
						SetRedraw(false);
						while(!selectedList.IsEmpty()) 
						{ 
							CPartFile *selected = selectedList.GetHead();
							HideSources(selected);
							selected->StopFile(); 
							selectedList.RemoveHead(); 
						}
						SetRedraw(true);
						break; 
					} 
					HideSources(file);
					file->StopFile();
					break;
				case MP_CLEARCOMPLETED:
					SetRedraw(false);
					ClearCompleted();
					SetRedraw(true);
					break;
				case MP_METINFO:
					{
					#ifdef _USE_NEW_DETAILS_	
						CFileDetails dialog("File Details", file, this, 0);
					#else
						CFileDetailDialog dialog(file);
					#endif
						dialog.DoModal();
						break;
					}
				case MP_GETED2KLINK:
					if(selectedCount > 1)
					{
						CString str;
						while(!selectedList.IsEmpty()) 
						{ 
							str += selectedList.GetHead()->CreateED2kLink() + "\n"; 
							selectedList.RemoveHead(); 
						}
						theApp.CopyTextToClipboard(str);
						//AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
						break; 
					} 
					theApp.CopyTextToClipboard(file->CreateED2kLink());
					break;
				case MP_GETHTMLED2KLINK:
					if(selectedCount > 1)
					{
						CString str;
						while(!selectedList.IsEmpty()) 
						{ 
							str += selectedList.GetHead()->CreateHTMLED2kLink() + "\n"; 
							selectedList.RemoveHead(); 
						}
						theApp.CopyTextToClipboard(str);
						//AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
						break; 
					} 
					theApp.CopyTextToClipboard(file->CreateHTMLED2kLink());
					break;
				case MP_OPEN:
				{
					if(selectedCount > 1)
					{	
						break;
					}
					ShellOpenFile(file->GetFullName());
					break; 
				}
				case MP_PREVIEW:
				{
					if(selectedCount > 1)
					{	
						break;
					}
					file->PreviewFile();
					break;
				}
				case MP_VIEWFILECOMMENTS:
				{
					CCommentDialogLst dialog(file); 
					dialog.DoModal(); 
					break;
				}
				case MP_CHANGEDIR: 
				{
					CString dir = file->GetOutputDir();
					if(dir.GetLength() == 2) // DonGato: fix for root directories
						dir.Format("%s\\", dir);
					CString newpath = BrowseFolder(theApp.emuledlg->m_hWnd,GetResString(IDS_SELECTOUTPUTDIR),dir);
					if(newpath!=dir)
					{
						if(selectedCount > 1)
						{
							while(!selectedList.IsEmpty())
							{
								selectedList.GetHead()->SetAlternativeOutputDir(&newpath);
								selectedList.GetHead()->SavePartFile();
								selectedList.RemoveHead();
							}			
						}
						file->SetAlternativeOutputDir(&newpath);
						file->SavePartFile();
						file->GetOutputDir(); //<<-- to add new path to shared list
					}
					break;
				}
				case MP_DOCLEANUP:
					while(!selectedList.IsEmpty())
					{ 
						CPartFile* selFile = selectedList.GetHead();	//subst for shorter code, debugging & intellisense
						selFile->SetFileName(CleanupFilename(selFile->GetFileName()).GetBuffer());
						selFile->SavePartFile();
						theApp.sharedfiles->Reload(false);
						selFile->UpdateDisplayedInfo(true);		//better update (saves last updatetime)
						//UpdateItem(selectedList.GetHead());	//update display at once, not in 1 sec
						selectedList.RemoveHead();
					}
					break;
				case MP_ALL_A4AF_TO_HERE: // Tarod
					{
						file->DownloadAllA4AF();
					}
					break;
				case MP_ALL_A4AF_AUTO: // Tarod
					{
						// v6:- eklmn: A4AF mangement
						if (theApp.downloadqueue->GetA4AFAutoFile() == file)
							//curent file is A4AF auto => User want to switch it off
							theApp.downloadqueue->SetA4AFAutoFile(NULL);
						else
							//another file is A4AF auto => switch to new file
							theApp.downloadqueue->SetA4AFAutoFile(file);
					}
					break;
				case MP_DROPNONEEDEDSRCS:
					while(!selectedList.IsEmpty()) // Tarod
					{
						selectedList.GetHead()->RemoveNoNeededSources();
						selectedList.RemoveHead();
					} 
					break;
				// DonGato - Moview Preview Mode
				case MP_MOVIE:
					if (!(file->GetStatus()==PS_COMPLETE) && !(file->GetStatus()==PS_COMPLETING))
					{
						if (file->GetMovieMode())
						{
							file->SetMovieMode(0);
							file->SaveMovieMode();
						} else {
							if (file->IsAviMovie())
							{
								if (selectedList.GetHead()->GetFileSize() < 209715200)
									file->SetMovieMode(1);
								else
									file->SetMovieMode(2);
							}	
							else if (file->IsMpgMovie())
							{
								if (selectedList.GetHead()->GetFileSize() < 209715200)
									file->SetMovieMode(4);
								else
									file->SetMovieMode(5);
							}
							else
								file->SetMovieMode(0);
							file->SaveMovieMode();
						}
					}						
					break;
				// DonGato - Moview Preview Mode end

                // obaldin - make dropping supercompressed a per-file setting
                case MP_DROP_SUPERCOMPRESSED:
					while(!selectedList.IsEmpty())
					{ 
						CPartFile* selFile = selectedList.GetHead();
						if (!(selFile->GetStatus()==PS_COMPLETE) && !(selFile->GetStatus()==PS_COMPLETING))
						{
							selFile->SetDiscardSuperCompressed(!selFile->GetDiscardSuperCompressed());
							selFile->SaveMovieMode();
						}
						selectedList.RemoveHead();
					} 
					break;
					/*
					if (!(file->GetStatus()==PS_COMPLETE) && !(file->GetStatus()==PS_COMPLETING))
					{
                        file->SetDiscardSuperCompressed( !file->GetDiscardSuperCompressed() );
                        file->SaveMovieMode();
                    }
                    break;
                    */

				// START - enkeyDEV(Ottavio84) -ThniCreate source ed2k from downloads-
				case MP_GETSOURCEED2KLINK:
					if(selectedCount > 1)
					{
						CString str;
						while(!selectedList.IsEmpty())
						{ 
							str += selectedList.GetHead()->CreateED2KSourceLink(7, 10); 
							selectedList.RemoveHead(); 
							if(!selectedList.IsEmpty())
								str += "\n";
						}
						theApp.CopyTextToClipboard(str);
						AfxMessageBox(GetResString(IDS_COPIED2CB) + str);
						break; 
					} 
					theApp.CopyTextToClipboard(file->CreateED2KSourceLink(7, 10));
					break;
			}
		}
		else
		{
			CUpDownClient* client = (CUpDownClient*)content->pValue;
			switch (wParam)
			{
				case MP_SHOWLIST:
					client->RequestSharedFileList();
					break;
				case MP_MESSAGE:
					theApp.emuledlg->chatwnd.StartSession(client);
					break;
				case MP_ADDFRIEND:
					theApp.friendlist->AddFriend(client);
					break;
				case MP_REMOVEFRIEND:
					theApp.friendlist->RemoveFriend(client);
					break;
				case MP_DETAIL:
				{
				#ifdef _USE_NEW_DETAILS_
					CClientDetails dialog("Client Details", client, this, 0);
				#else
					CClientDetailDialog dialog(client);
				#endif
					dialog.DoModal();
					break;
				}
				case MP_DOWNNOW: 
				{
					CPartFile *pFile = (CPartFile*)content->pOwner;
					client->SwapToAnotherFile(pFile);	//eklmn: bugfix(08): A4AF switching
					break; 
				}				
			}
		}
		// cleanup multiselection
		selectedList.RemoveAll(); 
	}
	else /*nothing selected*/
	{
		switch (wParam)
		{
			case MP_CLEARCOMPLETED:
				ClearCompleted();
				break;
		}

	}

	return true;
	
	EMULE_CATCH

	return false;
}

void CDownloadListCtrl::OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult)
{
	EMULE_TRY

	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	m_SortAscending = (m_CurrentSortItem != pNMListView->iSubItem) ? true : !m_SortAscending;
	m_CurrentSortItem = pNMListView->iSubItem;
	SetSortArrow(m_CurrentSortItem, m_SortAscending);
	m_lastSort=m_CurrentSortItem + (m_SortAscending ? 0:100);	//Cax2 - more than 10 columns
	SortItems(SortProc, m_CurrentSortItem + (m_SortAscending ? 0:100)+(m_ShowSrc?1000:0)); // Cax2

	EMULE_CATCH

	*pResult = 0;
}

int CDownloadListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	EMULE_TRY

	ST_CTRLITEM* item1 = (ST_CTRLITEM*)lParam1;
	ST_CTRLITEM* item2 = (ST_CTRLITEM*)lParam2;

	bool showSrc=false;  
	if (lParamSort>=1000)
	{ 
		showSrc=true; 
		lParamSort-=1000; 
	} 
    int sortMod = 1;
    if(lParamSort >= 100)	//Cax2 - more than 10 columns
    {
		sortMod = -1;
		lParamSort -= 100;
	}

	int comp;
	if ((item1->eType == FILE_TYPE && item2->eType != FILE_TYPE) ||
		(item2->eType == FILE_TYPE && item1->eType != FILE_TYPE))
	{ // files vs source?
		comp = 0; // leave it as it is...
	} 
	else if (item1->eType == FILE_TYPE) 
	{ //both files? (!)
		if (showSrc)
		{	
			comp = 0;
		}
		else
		{ 
			CPartFile* file1 = (CPartFile*)item1->pValue;
			CPartFile* file2 = (CPartFile*)item2->pValue;
			comp = Compare(file1, file2, lParamSort);
		}
    } 
	else 
	{ //both sources...
		comp = Compare(GetFile(item1), GetFile(item2),0);
        if(comp != 0)
		{ 
			return 0;
		}
		//Cax2:a4af clients kept separate, unless sorting for name
		if ((IsAskedForAnotherFile(item1) || IsAskedForAnotherFile(item2)) && lParamSort != 0)
		{
			int n = 0;
			if(IsAskedForAnotherFile(item1) && !IsAskedForAnotherFile(item2))
			{	
				n = -1;
			}
			if(IsAskedForAnotherFile(item2) && !IsAskedForAnotherFile(item1))
			{	
				n = 1;
			}
			if (n != 0)			//always at the end, unless sort by source - then a4af always at the top
			{	
				return (lParamSort == 5)? n : -n;
			}
		}

		comp = Compare((CUpDownClient*)item1->pValue, (CUpDownClient*)item2->pValue, lParamSort, sortMod);
    } 
   	return sortMod * comp;

	EMULE_CATCH
	return 0;
}

void CDownloadListCtrl::ClearCompleted(bool ignorecats)
{
	// Search for completed file(s)
	for(ListItems::iterator it = m_ListItems.begin(); it != m_ListItems.end();)
	{
		ST_CTRLITEM* cur_item = it->second;
		it++; // Already point to the next iterator. 
		if(cur_item->eType == FILE_TYPE)
		{
			CPartFile* file = reinterpret_cast<CPartFile*>(cur_item->pValue);
			if(file->IsPartFile() == false && (CheckShowItemInGivenCat(file,m_nCurTab) || ignorecats) )
			{	
				RemoveFile(file);
			}
		}
	}
}

//SyruS clear single completed files
void CDownloadListCtrl::ClearCompleted(const uchar* FileHash /*=NULL*/)
{
	EMULE_TRY

	//eklmn: init temp array
	CArray<CPartFile*, CPartFile*> CompletedFiles;
	for(ListItems::iterator it = m_ListItems.begin(); it != m_ListItems.end();)
	{
		ST_CTRLITEM* cur_item = it->second;
		it++; // Already point to the next iterator. 
		if(cur_item->eType == FILE_TYPE)
		{
			CPartFile* pFile = reinterpret_cast<CPartFile*>(cur_item->pValue);
			if(!pFile->IsPartFile())
			{	
				if (FileHash == NULL)
				{	
					CompletedFiles.Add(pFile); 
				}
				else if (md4cmp(FileHash, pFile->GetFileHash()) == 0) 
				{
					RemoveFile(pFile);
					break;	//we can stop searching if match is found
				}
			}
		}
	}
	// eklmn: clear all file correct way
	if (FileHash == NULL)
	{
		for(int i = 0; i < CompletedFiles.GetCount(); i++)
		{	
			RemoveFile(CompletedFiles[i]);	//continue clearing all
		}
		CompletedFiles.RemoveAll();
	}
	
	EMULE_CATCH
}

void CDownloadListCtrl::SetStyle() 
{
	EMULE_TRY

	SetExtendedStyle(LVS_EX_FULLROWSELECT);

	EMULE_CATCH
}

void CDownloadListCtrl::SortInit(int override) // Cax2 sortdlg
{
	EMULE_TRY
	m_sortTime=::GetTickCount();
	if (override) m_lastSort-=(m_lastSort>=0)?10000:0;	//simple override, keep the old sorting column, restore it later...
	if (override==99) //major override	wipe old sorting order...
	{
		m_lastSort=theApp.glob_prefs->GetDownloadSortCol();
		m_CurrentSortItem=(m_lastSort<100)?m_lastSort:m_lastSort-100;
		SetSortArrow(m_CurrentSortItem,(m_lastSort<100));
	}
	if(m_ShowSrc)
	{
		if(theApp.glob_prefs->DoUseSort() && m_lastSort<0)
		{
			SortItems(&SortProc,1000 + theApp.glob_prefs->GetSrcSortCol1()); //+ show sources(clients)
			if (theApp.glob_prefs->DoUseSrcSortCol2())
			{
				if (theApp.glob_prefs->GetSrcSortCol2()==255)
				{
					SortItems(&SortProc,1107); // kiron's status desc, show sources(clients) 05/01/03
					SortItems(&SortProc,1103); // speed desc, show sources(clients)
				} else 
					SortItems(&SortProc,1000 + theApp.glob_prefs->GetSrcSortCol2());
			}
		}
	} else {
		if (m_lastSort<0)
		{
			SortItems(&SortProc,theApp.glob_prefs->GetDownloadSortCol());
		} else 
			SortItems(&SortProc, m_lastSort);
	}
	EMULE_CATCH
}
void CDownloadListCtrl::OnListModified(NMHDR *pNMHDR, LRESULT *pResult) 
{
	EMULE_TRY

	NM_LISTVIEW *pNMListView = (NM_LISTVIEW*)pNMHDR;

	//this works because true is equal to 1 and false equal to 0
	BOOL notLast = pNMListView->iItem + 1 != GetItemCount();
	BOOL notFirst = pNMListView->iItem != 0;

	// Lord KiRon
	CRect rcViewRect;
	GetViewRect(&rcViewRect);
	CRect rcItemRect;
	GetItemRect(pNMListView->iItem,&rcItemRect,LVIR_BOUNDS);
	CRect rcIntersection;
	rcIntersection.IntersectRect(rcViewRect,rcItemRect);
	if (!rcIntersection.IsRectEmpty())
	{
		RedrawItems(pNMListView->iItem - notFirst, pNMListView->iItem + notLast);
	}
	// Lord KiRon - end
	//RedrawItems(pNMListView->iItem - notFirst, pNMListView->iItem + notLast);

	EMULE_CATCH
}

int CDownloadListCtrl::Compare(CPartFile* file1, CPartFile* file2, LPARAM lParamSort) 
{
	EMULE_TRY
	if (file1 == NULL || file2 == NULL)
	{
		return 0;
	}
	switch(lParamSort)
	{
		case 0: //filename asc
            // obaldin: funny, but we might not have a filename yet..
            {
                CString fname1=file1->GetFileName();
                CString fname2=file2->GetFileName();
                if  (fname1.IsEmpty() || fname2.IsEmpty())
                {   
					return 0;
				}
                else
			    {   
					return _tcsicmp(fname1,fname2);
				}
            }
		case 1: //size asc
			return (file1->GetFileSize()== file2->GetFileSize())?0:((file1->GetFileSize() < file2->GetFileSize())?-1:1);		//sort for >2gb
		case 2: //transfered asc
			return CompareUnsigned(file1->GetTransfered(), file2->GetTransfered());
	    case 3: //completed asc
			return CompareUnsigned(file1->GetCompletedSize(), file2->GetCompletedSize());
		case 4: //speed asc
			return CompareUnsigned(file1->GetDatarate(), file2->GetDatarate());
		case 5: //progress asc
			{
				float comp = file1->GetPercentCompleted() - file2->GetPercentCompleted();
				return (comp > 0) ? 1 : ((comp < 0) ? -1 : 0);
			}
		case 6: //sources asc
			return file1->GetSourceCount() - file2->GetSourceCount();
		case 7: //priority asc
			return file1->GetPriority() - file2->GetPriority();
		case 8: //Status asc 
			{
				int comp = file1->getPartfileStatusRang() - file2->getPartfileStatusRang();

				// Second sort order on filename
				if(comp == 0) 
				{
					comp = _tcsicmp(file1->GetFileName(), file2->GetFileName());
				}

				return comp;
			}
		case 9: //Remaining size asc
			{
				int tmp1=file1->GetFileSize()-file1->GetCompletedSize();
				int tmp2=file2->GetFileSize()-file2->GetCompletedSize();
				return (tmp1==tmp2)?0:((tmp1<tmp2)?-1:1); 
			}
		case 10: //Remaining Time asc
			{
				int tmp1=file1->GetTimeRemaining();
				int tmp2=file2->GetTimeRemaining();
				if (tmp1 > 0 && tmp2 > 0)
				{ 
					return tmp1-tmp2 ;
				}
				if (tmp1 > 0 || tmp2 > 0)
				{
					return tmp2-tmp1; 
				}
				//if no remaining time sort by remaining size asc. Leave completed files last...
				tmp1=(file1->GetStatus()==PS_COMPLETING || file1->GetStatus()==PS_COMPLETE);
				tmp2=(file2->GetStatus()==PS_COMPLETING || file2->GetStatus()==PS_COMPLETE);
				if ((tmp1) && (tmp2))
				{	
					return 0;
				}
				if (tmp2)
				{	
					return -1;
				}
				if (tmp1)
				{	
					return 1;
				}
			}
		case 13: //Last seen complete asc 
		    if (file1->lastseencomplete > file2->lastseencomplete)
			{    
				return 1;
			}
		    else if(file1->lastseencomplete < file2->lastseencomplete)
			{    
				return -1;
			}
		    else
			{    
				return 0;
			}
		case 14: //last received Time asc 
			if (file1->GetFileDate() > file2->GetFileDate())
			{
				return 1;
			}
			else if(file1->GetFileDate() < file2->GetFileDate())
			{	
				return -1;
			}
			else
			{
				return 0;
			}
		case 15:
			return _tcsicmp(
			(file1->GetCategory()!=0)?theApp.glob_prefs->GetCategory(file1->GetCategory())->title:GetResString(IDS_ALL),
			(file2->GetCategory()!=0)?theApp.glob_prefs->GetCategory(file2->GetCategory())->title:GetResString(IDS_ALL));
		default:
			return 0;
	}
	
	EMULE_CATCH

	return 0;
}

int CDownloadListCtrl::Compare(CUpDownClient* client1,CUpDownClient* client2, LPARAM lParamSort, int sortMod) 
{
	EMULE_TRY
	if (client1==NULL||client2==NULL)
		return 0;

	switch(lParamSort)
	{
		case 0: //name asc
			{
				CString sName1, sName2;
				try{ sName1 = CString(client1->GetUserName()); }
				catch(...){ sName1 = _T(""); }
				try{ sName2 = CString(client2->GetUserName()); }
				catch(...){ sName2 = _T(""); }
				if  (sName1.IsEmpty() || sName2.IsEmpty())
					return (sName1.IsEmpty()) ? ((sName2.IsEmpty()) ? 0:sortMod):-1*sortMod;	//Cax2 append all unknown ones at the end
				return _tcsicmp(sName1, sName2);
			}
		case 2: //transfered asc
			{
				if(!theApp.glob_prefs->IsTransferredOnCompleted())
			        return CompareUnsigned(client1->GetTransferedDown(), client2->GetTransferedDown());
				else
	                return 0;
			}
		case 3://completed asc
			{
				if(theApp.glob_prefs->IsTransferredOnCompleted())
			        return CompareUnsigned(client1->GetTransferedDown(), client2->GetTransferedDown());
				else
					return 0;
			}
		case 4: //speed asc
				return CompareUnsigned(client1->GetDownloadDatarate(), client2->GetDownloadDatarate());
		case 5: //progress asc
				return client1->GetAvailablePartCount() - client2->GetAvailablePartCount();
		case 6: //sources column
			{
				if(client1->GetClientSoft() == SO_UNKNOWN || client2->GetClientSoft() == SO_UNKNOWN )
					return (client1->GetClientSoft()-client2->GetClientSoft())*sortMod;		//Cax2 append all unknown ones at the end
				if(client1->GetClientSoft() == client2->GetClientSoft())
					if(client1->IsEmuleClient())
					{
						uint32 plus1=(client1->PlusVersion()<0x2600)?client1->PlusVersion():0;
						uint32 plus2=(client2->PlusVersion()<0x2600)?client2->PlusVersion():0;
						if(plus1 && plus2) 
							return plus2-plus1;
						if(plus1 || plus2)
							return (plus1)?-1:1;
						if (client1->GetMuleVersion() == client2->GetMuleVersion() )
						{
							if (!client1->GetModVersion().IsEmpty() && !client2->GetModVersion().IsEmpty())
								return _tcsicmp(client1->GetModVersion(), client2->GetModVersion());
							if (!client1->GetModVersion().IsEmpty() || !client2->GetModVersion().IsEmpty())
								return (client1->GetModVersion().IsEmpty()) ? 1 : -1;
						}
						return client2->GetMuleVersion() - client1->GetMuleVersion();
					}
					else
						return client2->GetVersion() - client1->GetVersion();
				return client1->GetClientSoft() - client2->GetClientSoft();
			}
		case 7: //priority column - QR
			{
				if (client1->GetDownloadState()!=DS_ONQUEUE || client2->GetDownloadState()!=DS_ONQUEUE)	//Cax2 QR gaps bugfix
				{
					if(client1->GetClientSoft() == SO_UNKNOWN || client2->GetClientSoft() == SO_UNKNOWN )		//Cax2 append all unknown ones at the end
						return (client1->GetClientSoft()-client2->GetClientSoft())*sortMod;							//Cax2 append all unknown ones at the end
					if (client1->GetDownloadState()==DS_DOWNLOADING || client2->GetDownloadState()==DS_DOWNLOADING)		//Cax2 QR gaps bugfix
						return (client1->GetDownloadState()==DS_DOWNLOADING)?((client2->GetDownloadState()==DS_DOWNLOADING)?0:-1):2; //Cax2 QR gaps bugfix
					return (client1->GetDownloadState()!=DS_ONQUEUE)?((client2->GetDownloadState()!=DS_ONQUEUE)?0:sortMod):-1*sortMod;		//Cax2 now should show properly...
				}
				if (sortMod<0 || (client1->GetRemoteQueueRank()>0 && client2->GetRemoteQueueRank()>0))
					return client1->GetRemoteQueueRank() - client2->GetRemoteQueueRank();   
				if(client1->GetRemoteQueueRank()==0 && client2->GetRemoteQueueRank()==0)
					return (client1->IsEmuleClient())?((client2->IsEmuleClient())?0:-1):2;	//"queue full" or " " ?
				return (client1->GetRemoteQueueRank()==0)?((client2->GetRemoteQueueRank()==0)?0:2):-1;
			}
		case 8: //Status column - Lord KiRon
			return client1->GetDownloadStateAsString().Compare(client2->GetDownloadStateAsString()); 
		case 11: // UL/DL column - Maverick
		case 12: // Ratio column - Maverick
			{	//Cax2 - faster this way & less code to go wrong...
				if(client1->GetClientSoft() == SO_UNKNOWN || client2->GetClientSoft() == SO_UNKNOWN )	//Cax2 append all unknown ones at the end
					return (client1->GetClientSoft()-client2->GetClientSoft())*sortMod;					//Cax2 append all unknown ones at the end
				bool isValid1 = (client1->IsRemoteInfoAvaiable()|| client1->credits);
				bool isValid2 = (client2->IsRemoteInfoAvaiable()|| client2->credits);
				if ( lParamSort==10 && isValid1 && isValid2)											//Cax2 - bugfix after reCDVst changes 
					return (client1->GetRemoteBaseModifier() <client2->GetRemoteBaseModifier())?-1:1;	//Cax2 - has to return an integer!
				if (isValid1 && isValid2)																//Cax2 - it's got to be lParamSort==11 after reCDVst changes 
					return client1->GetRemoteRatio() - client2->GetRemoteRatio();
				if (isValid1)				
					return 1;
				if (isValid2)				
					return -1;
				return 0;
			}
		default:
			return 0;
	}
	
	EMULE_CATCH
	return 0;
}

void CDownloadListCtrl::OnNMClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	RefreshInfo();

	DWORD pos = GetMessagePos();
	CPoint pt(LOWORD(pos), HIWORD(pos));
	ScreenToClient(&pt);

	// Get indexes of the first and last visible items in 
	// the listview control.
	int index = GetTopIndex();
	int last_visible_index = index + GetCountPerPage();
	if(last_visible_index > GetItemCount())
		last_visible_index = GetItemCount();

	// Loop until number visible items has been reached.
	while(index <= last_visible_index)
	{
		// Get the bounding rectangle of an item. If the mouse
		// location is within the bounding rectangle of the item,
		// you know you have found the item that was being clicked.
		CRect r;
		GetItemRect(index, &r, LVIR_BOUNDS);
		if(r.PtInRect(pt))
		{
			ST_CTRLITEM *pListItem = (ST_CTRLITEM*)GetItemData(index);
			if(!pListItem)
				return;
			if(!pListItem->eType == FILE_TYPE)			// if it isn't a file we're done
				return;

		// [+]/[-]	
			CRect rPlusMinus(r);
			rPlusMinus.left += 4 + 4;			// eeek, hardcoded values
			rPlusMinus.top += 1 + 2;						//  --||--
			rPlusMinus.right = rPlusMinus.left + 16 - 4;	//  --||--
			rPlusMinus.bottom = rPlusMinus.top + 16 - 4;	//  --||--
			if(rPlusMinus.PtInRect(pt))
			{	
				CPartFile *pPartFile = (CPartFile*)pListItem->pValue;
				if(!pPartFile)
					return;
				if(pPartFile->GetStatus() == PS_PAUSED)
					pPartFile->ResumeFile();
				else
					OnItemActivate(pNMHDR, pResult);
			}

		// rating
			CRect rRating;
			rRating.left = rPlusMinus.right+2; //EC 30.07.03 +2 for filetype icon gap +16 for size
			if (theApp.glob_prefs->ShowFileTypeIcon())
				rRating.left += 16+2; //EC 30.07.03 +2 for filetype icon gap +16 for size
			rRating.top = rPlusMinus.top;
			rRating.right = rRating.left + 16 - 6;
			rRating.bottom = rRating.top + 16 - 2;
			if(rRating.PtInRect(pt))
			{						
				CPartFile *pPartFile = (CPartFile*)pListItem->pValue;
				if(!pPartFile)
					return;
				if(pPartFile->HasComment()||pPartFile->HasRating())		//Cax2 - bugfix!
				{	
					CCommentDialogLst dlg(pPartFile); 
					dlg.DoModal(); 
				}
            }
			break;
		}

		// Get the next item in listview control.
		index++;
	}
	
	*pResult = 0;

	EMULE_CATCH
}

void CDownloadListCtrl::OnNMDblclkDownloadlist(NMHDR *pNMHDR, LRESULT *pResult)
{
	EMULE_TRY

	int iSel = GetSelectionMark();
	if (iSel != -1)
	{
		ST_CTRLITEM* content = (ST_CTRLITEM*)this->GetItemData(iSel);
		if (content && content->pValue)
		{
			if (content->eType == FILE_TYPE)
			{
				CPartFile* file = (CPartFile*)content->pValue;
				if(file->GetStatus()==PS_COMPLETE)
				{
					CString buffer;
					buffer.Format(_T("%s\\%s"),file->GetPath(),file->GetFileName());
					AddLogLine(false, _T("%s\\%s"),file->GetPath(),file->GetFileName());
					ShellOpenFile(buffer);
				}
			}
			else	//client  options
			{
				CUpDownClient* client = (CUpDownClient*)content->pValue;
				if (theApp.glob_prefs->GetDetailsOnClick())
				{
				#if _USE_NEW_DETAILS_
					CClientDetails dialog("Client Details", client, this, 0);
				#else
					CClientDetailDialog dialog(client);
				#endif
					dialog.DoModal();
				} 
				else 
				{
					theApp.emuledlg->chatwnd.StartSession(client);
				}
			}
		}
	}
	
	*pResult = 0;
	m_dblclicked=true;

	EMULE_CATCH
}

void CDownloadListCtrl::OnSize(UINT nType, int cx, int cy)
{
	EMULE_TRY

	CListCtrl::OnSize(nType, cx, cy);
	Invalidate();

	EMULE_CATCH
}

// Original by deltHF and Rod modified by Tarod
// After that modded by Vorlost too 
void CDownloadListCtrl::RestartWaitingDownloads() 
{ 
	EMULE_TRY

	CTypedPtrList<CPtrList, CPartFile*> selectedList; 
	int count = GetItemCount(); 
	POSITION pos; 

	// Vorlost get initial sources from server if we dont have our MaxSourcesPerFile Limit reached 
	for(int i = 0; i != count; i++)
	{ 
		ST_CTRLITEM *pItem = (ST_CTRLITEM*)GetItemData(i);
		if(pItem)
		{
			if(pItem->eType == FILE_TYPE)
			{
				CPartFile* pFile = (CPartFile*)pItem->pValue;
				if(pFile)
				{
					if(pFile->GetSourceCount() <= theApp.glob_prefs->GetMaxSourcePerFile())
					{
						selectedList.AddTail(pFile);
					}
				}
			}
	   	} 
	}

	count = selectedList.GetCount(); 

	for (pos = selectedList.GetHeadPosition() ; pos != 0 ; selectedList.GetNext(pos)) 
		selectedList.GetAt(pos)->GetSourcesAfterServerConnect(); 
    
	AddLogLine(true,IDS_NEWSERVERCONNECT,theApp.glob_prefs->GetMaxSourcePerFile()); 

	selectedList.RemoveAll(); 

	EMULE_CATCH
}

void CDownloadListCtrl::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	EMULE_TRY

	RefreshInfo();
	CMuleListCtrl::OnKeyUp(nChar, nRepCnt, nFlags);

	EMULE_CATCH
}

void CDownloadListCtrl::RefreshInfo(void)
{
	EMULE_TRY

	POSITION pos = GetFirstSelectedItemPosition();
	if(pos == NULL)
	{			
		theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToFile(NULL);
		theApp.emuledlg->transferwnd.UpdateInfoHeader();
	} 
	else 
	{
		ST_CTRLITEM* content = (ST_CTRLITEM*)GetItemData(GetSelectionMark());
		if(content->eType == FILE_TYPE)
		{
			const CPartFile* file = (CPartFile*)content->pValue;
			theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToFile((CPartFile*)file);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
		} 
		else 
		{
			const CUpDownClient* client = (CUpDownClient*)content->pValue;
			theApp.emuledlg->transferwnd.m_wndInfoList.SwitchToUser((CUpDownClient*)client);
			theApp.emuledlg->transferwnd.UpdateInfoHeader();
		}	
	}	

	EMULE_CATCH
}

void CDownloadListCtrl::ShowFilesCount() 
{
	uint16	count=0;
	CString counter;
	ST_CTRLITEM* cur_item;

	for(ListItems::const_iterator it = m_ListItems.begin(); it != m_ListItems.end(); it++)
	{
		cur_item = it->second;
		if (cur_item->eType == FILE_TYPE)
		{
			CPartFile* file=(CPartFile*)cur_item->pValue;
			if (CheckShowItemInGivenCat(file,m_nCurTab))
			{
				++count;
			}
		}
	}

//	counter.Format("%s (%u)", GetResString(IDS_TW_DOWNLOADS),count);
//	theApp.emuledlg->transferwnd.GetDlgItem(IDC_DOWNLOAD_TEXT)->SetWindowText(counter);
	theApp.emuledlg->transferwnd.UpdateDownloadHeader();
}

CString CDownloadListCtrl::getTextList() 
{
	EMULE_TRY

	CString out=_T("");
	CString temp,temp2;

	// Search for file(s)
	for(ListItems::iterator it = m_ListItems.begin(); it != m_ListItems.end(); it++)
	{ // const is better
		ST_CTRLITEM* cur_item = it->second;
		if(cur_item->eType == FILE_TYPE)
		{
			CPartFile* file = reinterpret_cast<CPartFile*>(cur_item->pValue);

			AddLogLine(false,file->GetFileName());
			temp2 = file->GetFileName(); 
			if (temp2.GetLength()>50) 
			{
				temp2=temp2.Mid(0,50);
			}
			temp.Format(_T("\n%s\t [%.1f%%] %i/%i - %s"),
							temp2,
							file->GetPercentCompleted(),
							file->GetTransferingSrcCount(),
							file->GetSourceCount(), 
							file->GetPartfileStatus());
			out = out + temp;
		}
	}
	AddLogLine(false,out);

	return out;

	EMULE_CATCH

	return _T("");
}

bool CDownloadListCtrl::GetDisplayedFiles(CArray<CPartFile*,CPartFile*> *list)
{
	EMULE_TRY

	for(ListItems::iterator it = m_ListItems.begin(); it != m_ListItems.end();)
	{
		ST_CTRLITEM* cur_item = it->second;
		it++; // Already point to the next iterator. 
		if(cur_item->eType == FILE_TYPE)
		{
			CPartFile* file = reinterpret_cast<CPartFile*>(cur_item->pValue);
			list->Add(file);
		}
	}	
	return true;

	EMULE_CATCH

	return false;
}

void CDownloadListCtrl::ShowSelectedFileDetails() 
{
	EMULE_TRY

	POINT point;
	::GetCursorPos(&point);
	CPoint p = point; 
    ScreenToClient(&p); 
    int it = HitTest(p); 
	SetSelectionMark(it);   // display selection mark correctly! 
    if (it == -1) return;

	ST_CTRLITEM* content = (ST_CTRLITEM*)this->GetItemData(GetSelectionMark());

	if (content->eType == FILE_TYPE)
	{
		CPartFile* file = (CPartFile*)content->pValue;

		if ((file->HasComment() || file->HasRating()) && p.x<13 ) 
		{
			CCommentDialogLst dialog(file);
			dialog.DoModal();
		}
		else 
		{
		#ifdef _USE_NEW_DETAILS_	
			CFileDetails dialog("File Details", file, this, 0);
		#else
			CFileDetailDialog dialog(file);
		#endif
			dialog.DoModal();
			dialog.DoModal();
		}
	}
	else 
	{
		CUpDownClient* client = (CUpDownClient*)content->pValue;
	#if _USE_NEW_DETAILS_
		CClientDetails dialog("Client Details", client, this, 0);
	#else
		CClientDetailDialog dialog(client);
	#endif
		dialog.DoModal();
	}

	EMULE_CATCH
}

void CDownloadListCtrl::ChangeCategory(int newsel)
{
	EMULE_TRY

	SetRedraw(FALSE);

	// remove all displayed files with a different cat and show the correct ones
	for(ListItems::const_iterator it = m_ListItems.begin(); it != m_ListItems.end(); it++)
	{
		const ST_CTRLITEM* cur_item = it->second;
		if (cur_item->eType == FILE_TYPE)
		{
			CPartFile* file = reinterpret_cast<CPartFile*>(cur_item->pValue);
			
			if ( !CheckShowItemInGivenCat(file,newsel))
			{ 
				HideFile(file);
			}
			else 
			{
				ShowFile(file);
			}
		}
	}

	SetRedraw(TRUE);
	m_nCurTab=newsel;
	ShowFilesCount();

	EMULE_CATCH
}

void CDownloadListCtrl::HideFile(CPartFile* tohide)
{
	EMULE_TRY

	HideSources(tohide);

	// Retrieve all entries matching the source
	std::pair<ListItems::const_iterator, ListItems::const_iterator> rangeIt = m_ListItems.equal_range(tohide);
	for(ListItems::const_iterator it = rangeIt.first; it != rangeIt.second; it++)
	{
		ST_CTRLITEM* updateItem  = it->second;

		// Find entry in CListCtrl and update object
 		LVFINDINFO find;
		find.flags = LVFI_PARAM;
		find.lParam = (LPARAM)updateItem;
		sint16 result = FindItem(&find);
		if(result != -1) 
		{
			DeleteItem(result);
			return;
		}
	}

	EMULE_CATCH
}

void CDownloadListCtrl::ShowFile(CPartFile* toshow)
{
	EMULE_TRY

	// Retrieve all entries matching the source
	std::pair<ListItems::const_iterator, ListItems::const_iterator> rangeIt = m_ListItems.equal_range(toshow);
	for(ListItems::const_iterator it = rangeIt.first; it != rangeIt.second; it++)
	{
		ST_CTRLITEM* updateItem  = it->second;

		// Check if entry is already in the List
 		LVFINDINFO find;
		find.flags = LVFI_PARAM;
		find.lParam = (LPARAM)updateItem;
		sint16 result = FindItem(&find);
		if(result == -1)
		{
			InsertItem(LVIF_PARAM|LVIF_TEXT,GetItemCount(),LPSTR_TEXTCALLBACK,0,0,0,(LPARAM)updateItem);
		}

		return;
	}

	EMULE_CATCH
}

void CDownloadListCtrl::MoveCompletedfilesCat(uint8 from, uint8 to)
{
	EMULE_TRY

	uint8 mycat;

	for(ListItems::iterator it = m_ListItems.begin(); it != m_ListItems.end();)
	{
		ST_CTRLITEM* cur_item = it->second;
		it++; // Already point to the next iterator.
		if(cur_item->eType == FILE_TYPE)
		{
			CPartFile* file = reinterpret_cast<CPartFile*>(cur_item->pValue);
			if (!file->IsPartFile())
			{
				mycat=file->GetCategory();
				if ( mycat>=min(from,to) && mycat<=max(from,to)) 
				{
					if (mycat==from) 
					{	
						file->SetCategory(to); 
					}
					else
					{	
						if (from<to)
						{
							file->SetCategory(mycat-1);
						}
						else 
						{
							file->SetCategory(mycat+1);
						}
					}
				}
			}
		}
	}

	EMULE_CATCH
}

void CDownloadListCtrl::OnGetDispInfo(NMHDR* pNMHDR, LRESULT* pResult)
{
    NMLVDISPINFO* pDispInfo = (NMLVDISPINFO*)pNMHDR;
	/*TRACE("CDownloadListCtrl::OnGetDispInfo iItem=%d iSubItem=%d", pDispInfo->item.iItem, pDispInfo->item.iSubItem);
	if (pDispInfo->item.mask & LVIF_TEXT)
		TRACE(" LVIF_TEXT");
	if (pDispInfo->item.mask & LVIF_IMAGE)
		TRACE(" LVIF_IMAGE");
	if (pDispInfo->item.mask & LVIF_STATE)
		TRACE(" LVIF_STATE");
	TRACE("\n");*/

	// Although we have an owner drawn listview control we store the text for the primary item in the listview, to be
	// capable of quick searching those items via the keyboard. Because our listview items may change their contents,
	// we do this via a text callback function. The listview control will send us the LVN_DISPINFO notification if
	// it needs to know the contents of the primary item.
	//
	// But, the listview control sends this notification all the time, even if we do not search for an item. At least
	// this notification is only sent for the visible items and not for all items in the list. Though, because this
	// function is invoked *very* often, no *NOT* put any time consuming code here in.

    if (pDispInfo->item.mask & LVIF_TEXT){
        const ST_CTRLITEM* pItem = reinterpret_cast<ST_CTRLITEM*>(pDispInfo->item.lParam);
        if (pItem != NULL && pItem->pValue != NULL){
			if (pItem->eType == FILE_TYPE){
				switch (pDispInfo->item.iSubItem){
					case 0:
						if (pDispInfo->item.cchTextMax > 0){
							_tcsncpy(pDispInfo->item.pszText, ((const CPartFile*)pItem->pValue)->GetFileName(), pDispInfo->item.cchTextMax);
							pDispInfo->item.pszText[pDispInfo->item.cchTextMax-1] = _T('\0');
						}
						break;
					default:
						// shouldn't happen
						pDispInfo->item.pszText[0] = _T('\0');
						break;
				}
			}
			else if (pItem->eType == UNAVAILABLE_SOURCE || pItem->eType == AVAILABLE_SOURCE){
				switch (pDispInfo->item.iSubItem){
					case 0:
						if (((CUpDownClient*)pItem->pValue)->GetUserName() != "" && pDispInfo->item.cchTextMax > 0){
							_tcsncpy(pDispInfo->item.pszText, ((CUpDownClient*)pItem->pValue)->GetUserName(), pDispInfo->item.cchTextMax);
							pDispInfo->item.pszText[pDispInfo->item.cchTextMax-1] = _T('\0');
						}
						break;
					default:
						// shouldn't happen
						pDispInfo->item.pszText[0] = _T('\0');
						break;
				}
			}
			else
				ASSERT(0);
        }
    }
    *pResult = 0;
}
