//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "downloadqueue.h"
#include "updownclient.h"
#include "partfile.h"
#include "ed2klink.h"
#include "memcpy_amd.h"
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// START enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
CHostnameSourceWnd::CHostnameSourceWnd(){
	m_pOwner = NULL;
}

BEGIN_MESSAGE_MAP(CHostnameSourceWnd, CWnd)
	ON_MESSAGE(TM_SOURCEHOSTNAMERESOLVED, OnSourceHostnameResolved)
	ON_WM_TIMER()
END_MESSAGE_MAP()

LRESULT CHostnameSourceWnd::OnSourceHostnameResolved(WPARAM wParam,LPARAM lParam) {
	if (m_pOwner)
		m_pOwner->SourceHostnameResolved(wParam,lParam);
	return true;
}
// END enkeyDEV(kei-kun) -hostname sources in ED2K superLink-

CDownloadQueue::CDownloadQueue(CPreferences* in_prefs,CSharedFileList* in_sharedfilelist){
	app_prefs = in_prefs;
	sharedfilelist = in_sharedfilelist;
	filesrdy = 0;
	datarate = 0;
	cur_udpserver = 0;
	lastfile = 0;
	lastudpsearchtime = 0;
	lastudpstattime = 0;
	m_nUDPSearchFile = 0;
	m_nUDPSearchServer = 0;
	averageDatarate = 0;
	udcounter = 0;
	m_nDownDataRateMSOverhead = 0;
	m_nDownDatarateOverhead = 0;
	m_nDownDataOverheadSourceExchange = 0;
	m_nDownDataOverheadFileRequest = 0;
	m_nDownDataOverheadOther = 0;
	m_nDownDataOverheadServer = 0;
	m_nDownDataOverheadSourceExchangePackets = 0;
	m_nDownDataOverheadFileRequestPackets = 0;
	m_nDownDataOverheadOtherPackets = 0;
	m_nDownDataOverheadServerPackets = 0;
	m_A4AF_auto_file = NULL;
	m_bIsInitialized = false;
}

void CDownloadQueue::UpdateDisplayedInfo(boolean force) {
	EMULE_TRY
	if(!m_bIsInitialized) return;
    DWORD curTick = GetTickCount();

    if(force || curTick-m_lastRefreshedDLDisplay > MINWAIT_BEFORE_DLDISPLAY_WINDOWUPDATE+(uint32)(rand()*1000/RAND_MAX)) {
        theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
        m_lastRefreshedDLDisplay = curTick;
    }
	EMULE_CATCH
}

void CDownloadQueue::AddPartFilesToShare()
{
	EMULE_TRY
    for (POSITION pos = filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		CPartFile* cur_file = filelist.GetAt(pos);
		if (cur_file->GetStatus(true) == PS_READY)
			sharedfilelist->SafeAddKFile(cur_file,true);
	}
    EMULE_CATCH
}

void CDownloadQueue::CompDownDatarateOverhead(){
    EMULE_TRY
	m_AvarageDDRO_list.AddTail(m_nDownDataRateMSOverhead);
	if (m_AvarageDDRO_list.GetCount() > 150)
		m_AvarageDDRO_list.RemoveAt(m_AvarageDDRO_list.GetHeadPosition());

	m_nDownDatarateOverhead = 0;
	m_nDownDataRateMSOverhead = 0;
	for (POSITION pos = m_AvarageDDRO_list.GetHeadPosition();pos != 0;m_AvarageDDRO_list.GetNext(pos))
		m_nDownDatarateOverhead += m_AvarageDDRO_list.GetAt(pos);

	if(m_AvarageDDRO_list.GetCount() > 10)
		m_nDownDatarateOverhead = 10*m_nDownDatarateOverhead/m_AvarageDDRO_list.GetCount();
	else
		m_nDownDatarateOverhead = 0;
	return;
    EMULE_CATCH
}



void CDownloadQueue::Init()
{
    EMULE_TRY

	// find all part files, read & hash them if needed and store into a list
	// start changed by InterCeptor (multiple tempdirs) 19.11.02
	uint16 count = 0;
	CString sTempDir = app_prefs->GetTempDir();
	bool bRemove = false;
	if(app_prefs->tempdir_list.Find(sTempDir) == NULL && app_prefs->tempdir_list.Find(sTempDir+'\\') == NULL)
	{
		app_prefs->tempdir_list.AddHead(sTempDir);
		bRemove = true;
	}
	for (POSITION pos = app_prefs->tempdir_list.GetHeadPosition();pos != 0;app_prefs->tempdir_list.GetNext(pos))
	{
		CFileFind ff;
		CString searchPath(app_prefs->tempdir_list.GetAt(pos).GetBuffer());
		searchPath += "\\*.part.met";

		//check all part.met files
		bool end = !ff.FindFile(searchPath,0);
		while (!end)
		{
			end = !ff.FindNextFile();
			if (ff.IsDirectory())
				continue;
			CPartFile* toadd = new CPartFile();
			if (toadd->LoadPartFile(app_prefs->tempdir_list.GetAt(pos).GetBuffer(),ff.GetFileName().GetBuffer()))
			{
				count++;
				// Added by Tarod -check if is an A4AF auto file, correct place?
				if (md4cmp(toadd->GetFileHash(), theApp.glob_prefs->GetA4AFHash()) == 0)
					// v1:- eklmn: A4AF mangement
					theApp.downloadqueue->SetA4AFAutoFile(toadd);
				filelist.AddTail(toadd); // to downloadqueue
				if (toadd->GetStatus(true) == PS_READY)
					sharedfilelist->SafeAddKFile(toadd); // part files are always shared files
				theApp.emuledlg->transferwnd.m_wndDownloadList.AddFile(toadd);// show in downloadwindow
			}
			else
				delete toadd;
		}
		ff.Close();

		//try recovering any part.met files
		searchPath += ".backup";
		end = !ff.FindFile(searchPath, 0);
		while (!end){
			end = !ff.FindNextFile();
			if (ff.IsDirectory())
				continue;
			CPartFile* toadd = new CPartFile();
			if (toadd->LoadPartFile(app_prefs->tempdir_list.GetAt(pos).GetBuffer(),ff.GetFileName().GetBuffer()))
			{
				toadd->SavePartFile(); // resave backup
				count++;
				filelist.AddTail(toadd);			// to downloadqueue
				if (toadd->GetStatus(true) == PS_READY)
					sharedfilelist->SafeAddKFile(toadd); // part files are always shared files
				theApp.emuledlg->transferwnd.m_wndDownloadList.AddFile(toadd);// show in downloadwindow

				AddLogLine(false, IDS_RECOVERED_PARTMET, toadd->GetFileName());
			}
			else
			{
				delete toadd;
			}
		}
		ff.Close();
	}
	if(bRemove)
		app_prefs->tempdir_list.RemoveHead();
	// end added by InterCeptor (multiple tempdirs) 19.11.02
	if(count == 0) {
		AddLogLine(false,IDS_NOPARTSFOUND);
	} else {
		AddLogLine(false,IDS_FOUNDPARTS,count);
		SortByPriority();
	}

	//--- xrmb:keepPartFileStats ---
	m_lastPartFileStatsSave=GetTickCount();
	//--- :xrmb ---
	// START enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
	hsCallbackWnd=NULL; 
	sourceHostnameResolveRetry=0;
	isResolving=false;
	// END enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
	m_bIsInitialized=true;
    EMULE_CATCH
}

CDownloadQueue::~CDownloadQueue(){
    EMULE_TRY
	//--- xrmb:keepPartFileStats ---
	SaveAllPartFileStats();
	//--- :xrmb ---

	for (POSITION pos = filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		filelist.GetAt(pos)->m_hpartfile.Flush() ; // Flush before close app [Tarod]
		filelist.GetAt(pos)->SavePartFile();
		delete filelist.GetAt(pos);
	}
    EMULE_CATCH
}

void CDownloadQueue::AddSearchToDownload(CSearchFile* toadd,uint8 cat)
{
	EMULE_TRY
	// Ultras: prevent to donload an empty file
	if(!toadd->GetFileSize()) 
	{
		AddLogLine(true, IDS_SKIPZEROLENGTHFILE, toadd->GetFileName());
		return;
	}
	if (IsFileExisting(toadd->GetFileHash()))
		return;
	CPartFile* newfile = new CPartFile(toadd);
	if (newfile->GetStatus() == PS_ERROR)
	{
		delete newfile;
		return;
	}
	newfile->SetCategory(cat);
	AddDownload(newfile);

	// netwolf 20.05.03 enhanced source finding (bluecow)
	// If the search result is from a client ('View Files') add that client as source
	if (toadd->GetType() == CSearchFile::Client && toadd->GetClientID() && toadd->GetClientPort())
	{
		CMemFile sources;
		uint16 uSources = 1;
		sources.Write(&uSources, sizeof uSources);
		uint32 uIP = toadd->GetClientID();
		sources.Write(&uIP, sizeof uIP);
		uint16 uPort = toadd->GetClientPort();
		sources.Write(&uPort, sizeof uPort);
		uIP = toadd->GetClientServerIP();
		sources.Write(&uIP, sizeof uIP);
		uPort = toadd->GetClientServerPort();
		sources.Write(&uPort, sizeof uPort);
		sources.SeekToBegin();
	if (newfile) newfile->AddClientSources(&sources);
		else
		{
			CPartFile* partfile = GetFileByID(toadd->GetFileHash());
				if (partfile)
					partfile->AddClientSources(&sources);
		}
	}				
	// netwolf 15.05.03 enhanced source finding END

	EMULE_CATCH
}

void CDownloadQueue::AddSearchToDownload(CString link,uint8 cat){
    EMULE_TRY
	CPartFile* newfile = new CPartFile(link);
	if (newfile->GetStatus() == PS_ERROR){
		delete newfile;
		return;
	}
	newfile->SetCategory(cat);
	AddDownload(newfile);
    EMULE_CATCH
}

void CDownloadQueue::AddFileLinkToDownload(CED2KFileLink* pLink,uint8 cat)
{
    EMULE_TRY
	CPartFile* newfile = new CPartFile(pLink);
	if (newfile->GetStatus() == PS_ERROR){
		delete newfile;
		newfile=NULL;
	}
	else {
		newfile->SetCategory(cat);
		AddDownload(newfile);
	}
	// START enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
	if(pLink->HasValidSources())
	{
		if (newfile)
		{ 
			if (pLink->ClientSourcesList)
				newfile->AddClientSources(pLink->ClientSourcesList); //add all IP source and eliminates them from list
		}
		else
		{
			CPartFile* partfile = GetFileByID((uchar*)pLink->GetHashKey());
			if (partfile && pLink->ClientSourcesList) partfile->AddClientSources(pLink->ClientSourcesList); //add all IP source and eliminates them from list
		}
	}
	// END enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
    EMULE_CATCH
}

//in the official they renamed this funcion StartNextFile()
// Added by Tarod
// Starts a paused download (takes in account priority)
void CDownloadQueue::StartPausedDownload(int cat)
{
    EMULE_TRY

	CPartFile* cur_file;
	CPartFile* file_to_start = NULL;
	POSITION pos;

	for (pos = filelist.GetHeadPosition(); pos != (POSITION)NULL; filelist.GetNext(pos))
	{
		cur_file = (CPartFile*)filelist.GetAt(pos);

		if (cur_file->GetStatus() == PS_PAUSED && (cur_file->GetCategory()==cat ||
			(cat==0 && theApp.glob_prefs->GetAllcatType()==0 && cur_file->GetCategory()>0)))
		{
			if (file_to_start ==NULL || file_to_start->GetPriority() < cur_file->GetPriority())
				file_to_start = cur_file;
		}
	}

	if (file_to_start != NULL)
		file_to_start->ResumeFile() ;

    EMULE_CATCH
}

void CDownloadQueue::AddDownload(CPartFile* newfile)
{
    EMULE_TRY

	filelist.AddTail(newfile);
	SortByPriority();
	if (theApp.glob_prefs->StartDownloadPaused()) {
		newfile->StopFile();
		newfile->SetStartTimeReset(true);	//SyruS enable reset on resume
	}

	SetAutoCat(newfile);// HoaX_69 / Slugfiller: AutoCat

	theApp.emuledlg->transferwnd.m_wndDownloadList.AddFile(newfile);
	AddLogLine(true,IDS_NEWDOWNLOAD,newfile->GetFileName());
	// Purity - Sending message when download added
	CString MessageText;
	MessageText.Format(GetResString(IDS_NEWDOWNLOAD)+_T(" "),newfile->GetFileName());
	theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetUseDownloadAddNotifier(), theApp.glob_prefs->IsSMTPInfoEnabled());
	theApp.emuledlg->ShowNotifier(MessageText, TBN_DLOAD_ADD, false, theApp.glob_prefs->GetUseDownloadAddNotifier());
    EMULE_CATCH
}

bool CDownloadQueue::IsFileExisting(const uchar* fileid)
{
    EMULE_TRY
	if (CKnownFile* file = sharedfilelist->GetFileByID((uchar*)fileid)){
		if (file->IsPartFile())
			AddLogLine(true, IDS_ERR_ALREADY_DOWNLOADING, file->GetFileName());
		else
			AddLogLine(true, IDS_ERR_ALREADY_DOWNLOADED, file->GetFileName());
		return true;
	}
	else if ( file = GetFileByID((uchar*)fileid)){
		AddLogLine(true, IDS_ERR_ALREADY_DOWNLOADING, file->GetFileName());
		return true;
	}
    EMULE_CATCH
	return false;
}


void CDownloadQueue::Process()
{
    EMULE_TRY
	if(!m_bIsInitialized) return;
	uint32 downspeed = 0;
	uint16 max_DL_speed = app_prefs->GetMaxDownload();
	uint32 max_DL_speed_kB = max_DL_speed*1024;
	if ( max_DL_speed != UNLIMITED && datarate > 1500 /*&& max_DL_speed_kB>datarate*/) {
		downspeed = (max_DL_speed_kB*100)/(datarate+1); //(uint16)((float)((float)(app_prefs->GetMaxDownload()*1024)/(datarate+1)) * 100);
		if (downspeed < 50)
			downspeed = 50;
		// eklmn: don't remove upper DL limit, otherwise on DL will block all other
		else if (downspeed > 200)
			downspeed = 200;
	}

	//static uint32 iteration=0;
	uint32 tempDatarate = 0;
	udcounter++;
	for (POSITION pos =filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		CPartFile* cur_file =  filelist.GetAt(pos);
		if ((cur_file->GetStatus() == PS_READY || cur_file->GetStatus() == PS_EMPTY) 
			&&(cur_file->GetPriority() == PR_HIGH || cur_file->GetPriority() == PR_AUTO))	//jicxicmic fix
		{
			tempDatarate += cur_file->Process(downspeed, udcounter /*iteration*/);
		}
	}
	for (POSITION pos =filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		CPartFile* cur_file =  filelist.GetAt(pos);
		if ((cur_file->GetStatus() == PS_READY || cur_file->GetStatus() == PS_EMPTY) && cur_file->GetPriority() == PR_NORMAL){
			tempDatarate += cur_file->Process(downspeed, udcounter /*iteration*/);
		}
	}
	for (POSITION pos =filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		CPartFile* cur_file =  filelist.GetAt(pos);
		if ((cur_file->GetStatus() == PS_READY || cur_file->GetStatus() == PS_EMPTY) && cur_file->GetPriority() == PR_LOW){
			tempDatarate += cur_file->Process(downspeed, udcounter /*iteration*/); 
		}
	}

	//if(++iteration>15) iteration=0;

	datarate = tempDatarate;
	DWORD curTick = GetTickCount();
	// Save used bandwidth for speed calculations
	//avarage_dr_list.AddTail(usedBandwidthThisCall);
	if(average_tick_list.empty() || curTick - average_tick_list.front()>= 500)	//Cax2 - update no faster than every .5 sec
	{
		average_dr_list.push_front(theApp.stat_sessionReceivedBytes);

		// Save time beetween each speed snapshot
		average_tick_list.push_front(curTick);

		//Cax2 save 40 secs of data. It seems to give the most accurate result atm...
		while(curTick-average_tick_list.back() > 40000) {
   			average_dr_list.pop_back();
			average_tick_list.pop_back();
		}
	}

	// Calculate average datarate
	if(average_dr_list.size() >1) 
		averageDatarate = (uint32)(((float)(average_dr_list.front()-average_dr_list.back()))*1000 / (average_tick_list.front()-average_tick_list.back()));
	else
		averageDatarate = 0;
	
	curTick = GetTickCount();	//Cax2 - Refresh the variable
	if (udcounter == 5){
		if((!lastudpstattime) || (curTick - lastudpstattime) > UDPSERVERSTATTIME){
			lastudpstattime = curTick;
			theApp.serverlist->ServerStats();
		}
	}
	if (udcounter == 10){
		udcounter = 0;
		if ((!lastudpsearchtime) || (curTick - lastudpsearchtime) > UDPSERVERREASKTIME)
			SendNextUDPPacket();
	}


	//--- xrmb:keepPartFileStats ---
	//--- i hope that is a safe point to save the stats ---
	if(m_lastPartFileStatsSave+300000 < curTick )
	{
		m_lastPartFileStatsSave=curTick;

		SaveAllPartFileStats();
	}
	//--- :xrmb ---
    EMULE_CATCH
}

void	CDownloadQueue::SaveAllPartFileStats()
{
    EMULE_TRY
	for (POSITION pos =filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos))
	{
		CPartFile* cur_file =  filelist.GetAt(pos);
		cur_file->SavePartFileStats();
	}
    EMULE_CATCH
}

CPartFile*	CDownloadQueue::GetFileByIndex(int index){
	POSITION pos = filelist.FindIndex(index);
	if (pos)
		return filelist.GetAt(pos);
	else
		return NULL;
}

CPartFile*	CDownloadQueue::GetFileByID(const uchar* filehash)
{
    EMULE_TRY
	for (POSITION pos = filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		if (!md4cmp(filehash,filelist.GetAt(pos)->GetFileHash()))
			return filelist.GetAt(pos);
	}
    EMULE_CATCH
	return 0;
}

bool CDownloadQueue::IsPartFile(void* totest)
{
	EMULE_TRY

	POSITION pos = filelist.Find(totest);
	if (pos)
		return true;
	else
		return false;

	EMULE_CATCH
	return false;
	
}

void CDownloadQueue::CheckAndAddSource(CPartFile* sender,CUpDownClient* source, bool saveSources)
{
    EMULE_TRY
	// if we block loopbacks at this point it should prevent us from connecting to ourself
	if(!source->HasLowID() && (source->GetUserID() & 0xFF) == 0x7F) {
		delete source;
		return;
	}	

	//(filtering invalid eMule clients)
	if(source->GetMuleVersion()==0 && source->GetVersion()==0 && (source->GetClientSoft()==SO_EMULE || source->GetClientSoft()==SO_OLDEMULE)) {
		delete source;
		return;
	}
	//end (filtering invalid eMule clients) //Mr Faber / netwolf


	// double check... just testing...
	// [Lord KiRon] v2
	if (md4cmp(theApp.glob_prefs->GetUserHash(), source->GetUserHash()) == 0)
	{
		delete source;
		return;
	} 
	// [/Lord KiRon] v2 

	//eklmn: if this part file is stopped or complete/completing no sources needed any more ()
	if (sender->GetStatus() == PS_STOPPED	|| sender->GetStatus() == PS_COMPLETE || sender->GetStatus() == PS_COMPLETING) 
	{
		delete source;
		return;
	}

	// uses this only for temp. clients
	POSITION pos, pos1, pos2;
	for (pos = filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		CPartFile* cur_file = filelist.GetAt(pos);
		for (int sl=0;sl<SOURCESSLOTS;sl++) 
			if (!cur_file->srclists[sl].IsEmpty())
				for (pos1 = cur_file->srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;) 
				{
					cur_file->srclists[sl].GetNext(pos1);
					CUpDownClient* temp_src = cur_file->srclists[sl].GetAt(pos2);
					// check if souce is already exist
					if (temp_src->Compare(source)) 
					{
						// if we are adding another (different from current file) check in OtherFileList
						if (cur_file != sender){ 
							// check in other file list
							if (temp_src->AddRequestForAnotherFile(sender)) 
								theApp.emuledlg->transferwnd.m_wndDownloadList.AddSource(sender,temp_src,true);

						}
						delete source;
						return;
					}
				}
	}
	//our new source is real new but maybe it is already uploading to us?
	//if yes the known client will be attached to the var "source"
	//and the old sourceclient will be deleted
#ifdef OLD_SOCKETS_ENABLED
	if (theApp.clientlist->AttachToAlreadyKnown(&source,0))
		source->reqfile = sender;
	else
#endif //OLD_SOCKETS_ENABLED
		theApp.clientlist->AddClient(source,true);
	
	sender->srclists[source->sourcesslot].AddTail(source);
	sender->RemovePastComment(source);		// #zegzav:pastcomment
	source->StartQueueWaitTimer(); //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
	theApp.emuledlg->transferwnd.m_wndDownloadList.AddSource(sender,source,false);
	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
	UpdateDisplayedInfo();	// eklmn: bugfix(12): refresh whole downloadlist

   EMULE_CATCH
}

BOOL CDownloadQueue::CheckAndAddKnownSource(CPartFile* sender,CUpDownClient* source)
{
    EMULE_TRY
	// SyruS "stop adding sources on stopped, completing and completed files" (Cax2 + SyruS check)
	// eklmn: please never use > or < in state check
	if (sender->GetStatus() == PS_PAUSED 
		|| sender->GetStatus() == PS_STOPPED
		|| sender->GetStatus() == PS_COMPLETE
		|| sender->GetStatus() == PS_COMPLETING) 
		return FALSE;
	// SyruS end
	if(!source->HasLowID() && (source->GetUserID() & 0xFF) == 0x7F)
		return FALSE;
	// [Lord KiRon] v3 
   	if (md4cmp(theApp.glob_prefs->GetUserHash(), source->GetUserHash()) == 0) 
   	{ 
    	// This is most important part of the fix 
        // if we not throw here, CClientReqSocket::ProcessPacket will never diconnect a client 
        /*throw CString(_T("Got myself as a source"));*/ 
        return FALSE; 
	}
	// [/Lord KiRon] v3 
	// v-eklmn: bugfix(09): hang up by start if all files are paused
	for (POSITION pos = filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		CPartFile* cur_file = filelist.GetAt(pos);
		for (int sl=0;sl<SOURCESSLOTS;sl++){ 
			if (!cur_file->srclists[sl].IsEmpty()) {
				if (cur_file->srclists[sl].Find(source)) {
					if (cur_file == sender) return TRUE; // [Lord KiRon] v3
					if (source->AddRequestForAnotherFile(sender))
						theApp.emuledlg->transferwnd.m_wndDownloadList.AddSource(sender,source,true);
					return TRUE;
				}
			}
		}
	}
	// ^-eklmn: bugfix(09): hang up by start if all files are paused
	source->reqfile = sender;
	//if (source->GetFileRate()>0 ||source->GetFileComment().GetLength()>0) sender->UpdateFileRatingCommentAvail();
	sender->srclists[source->sourcesslot].AddTail(source);
	sender->RemovePastComment(source);		// #zegzav:pastcomment
	source->StartQueueWaitTimer(); //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
	theApp.emuledlg->transferwnd.m_wndDownloadList.AddSource(sender,source,false);
	UpdateDisplayedInfo();	// eklmn: bugfix(12): refresh whole downloadlist

	return TRUE;
	EMULE_CATCH
		
	return FALSE;
}

//eklmn: this function will remove  the source complete, from all files
bool CDownloadQueue::RemoveSource(CUpDownClient* toremove, bool updatewindow)
{
	EMULE_TRY

	// v-eklmn: bugfix(09): hang up by start if all files are paused
	bool removed = false;
	CPartFile* cur_file;
	// eklmn: set client state to DS_NONE in order to prevent any action from him
	toremove->SetDownloadState(DS_NONE);
	
	//eklmn: check actual requested file (we have this info in "reqfile")
	cur_file = toremove->reqfile;
	if (cur_file != NULL && !cur_file->srclists[toremove->sourcesslot].IsEmpty())
	{
		POSITION remove_pos = cur_file->srclists[toremove->sourcesslot].Find(toremove);
		if (remove_pos != NULL) 
		{
			cur_file->AddPastComment(toremove);		// #zegzav:pastcomment
			cur_file->srclists[toremove->sourcesslot].RemoveAt(remove_pos);
			removed = true;
		} //if remove_pos
		//eklmn: update source info for the file
		cur_file->NewSrcPartsInfo();	// eklmn: changes(00): NewSrcPartsInfo & UpdateAvailablePartsCount merged (CPU load)
	}

	//eklmn: now remove client in A4AF files. let's use "m_OtherRequests_list" for this goal
	if (!toremove->m_OtherRequests_list.IsEmpty())
	{
		for (POSITION pos = toremove->m_OtherRequests_list.GetHeadPosition();pos != 0; toremove->m_OtherRequests_list.GetNext(pos))
		{
			cur_file = toremove->m_OtherRequests_list.GetAt(pos);
			if(IsPartFile(cur_file))
			{
				POSITION pos_A4AF = cur_file->A4AF_srclist.Find(toremove);
				if (pos_A4AF) 
				{
					cur_file->A4AF_srclist.RemoveAt(pos_A4AF);
				}
			}
		}
	}
	//eklmn: clear client from all graphical lists
	if (updatewindow)
		theApp.emuledlg->transferwnd.m_wndDownloadList.RemoveSource(toremove,NULL);	

	if (toremove->GetFileComment().GetLength()>0 || toremove->GetFileRate()>0 )
		toremove->reqfile->UpdateFileRatingCommentAvail();

	toremove->reqfile = NULL;
	toremove->CleanOtherRequests();	//Cax2 - downloadlistcontrol bugfix
	return removed;
	
	EMULE_CATCH
		
    return false;
}

void CDownloadQueue::RemoveFile(CPartFile* toremove)
{
	EMULE_TRY

	POSITION remove_pos = filelist.Find(toremove);
	if (remove_pos)
		filelist.RemoveAt(remove_pos);

	EMULE_CATCH
}

void CDownloadQueue::ResumeFiles()
{
	for(POSITION pos = filelist.GetHeadPosition(); pos != NULL;filelist.GetNext(pos))
	{ 
		CPartFile* cur_file = filelist.GetAt(pos); 
		if( cur_file->GetStatus() == PS_READY || cur_file->GetStatus() == PS_EMPTY ) //netwolf
		{
			cur_file->ResumeFile();
		}
	}
}

void CDownloadQueue::DeleteAll()
{
    EMULE_TRY
	POSITION pos;
	for (pos = filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos)){
		CPartFile* cur_file = filelist.GetAt(pos);
		for (int sl=0;sl<SOURCESSLOTS;sl++) if (!cur_file->srclists[sl].IsEmpty())
			cur_file->srclists[sl].RemoveAll();
		// Barry - Should also remove all requested blocks
		// Don't worry about deleting the blocks, that gets handled 
		// when CUpDownClient is deleted in CClientList::DeleteAll()
		cur_file->RemoveAllRequestedBlocks();
	}
    EMULE_CATCH
}

bool CDownloadQueue::SendNextUDPPacket()
{
#ifdef OLD_SOCKETS_ENABLED
	EMULE_TRY
	if(!m_bIsInitialized) return false;
	if (filelist.IsEmpty() || !theApp.serverconnect->IsConnected())
		return false;
	if (!cur_udpserver)
		if (!(cur_udpserver = theApp.serverlist->GetNextServer(cur_udpserver))){
			TRACE("ERROR:SendNextUDPPacket() no server found");
			StopUDPRequests();
		};

	// get nextfile
	CPartFile* nextfile = 0;
	while (!(nextfile && (nextfile->GetStatus() == PS_READY ||nextfile->GetStatus() == PS_EMPTY))){
		if (lastfile == 0){
			nextfile = filelist.GetHead();
			m_nUDPSearchFile = 1;
			lastfile = nextfile;
		}
		else{
			POSITION pos = filelist.Find(lastfile);
			if (!pos){
				TRACE("Error: CDownloadQueue::SendNextUDPPacket()");
				nextfile = filelist.GetHead();
				m_nUDPSearchFile = 1;
				lastfile = nextfile;
			}
			else{
				filelist.GetNext(pos);
				if (pos == 0){
					cur_udpserver = theApp.serverlist->GetNextServer(cur_udpserver);
					m_nUDPSearchServer++;
					if (cur_udpserver == 0){
						//TRACE("finished");
						lastudpsearchtime = ::GetTickCount();
						lastfile = 0;
						m_nUDPSearchFile = 0;
						m_nUDPSearchServer = 0;
						return false; // finished (processed all file & all servers)
					}
					nextfile = filelist.GetHead();
					m_nUDPSearchFile = 1;
					lastfile = nextfile;
				}
				else{
					nextfile = filelist.GetAt(pos);
					m_nUDPSearchFile++;
					lastfile = nextfile;
				}
			}
		}
	}
	if( (theApp.glob_prefs->GetMaxSourcePerFileUDP()) < nextfile->GetSourceCount()) //<<--
		return true; 
	Packet packet(OP_GLOBGETSOURCES,16);
	md4cpy(packet.pBuffer,nextfile->GetFileHash());
	if (cur_udpserver != theApp.serverlist->GetServerByAddress(theApp.serverconnect->GetCurrentServer()->GetAddress(),theApp.serverconnect->GetCurrentServer()->GetPort())){
		theApp.uploadqueue->AddUpDataOverheadServer(packet.size);
		theApp.serverconnect->SendUDPPacket(&packet,cur_udpserver,false);
	}
	return true;
	EMULE_CATCH
#endif //OLD_SOCKETS_ENABLED
	return false;
}

void CDownloadQueue::StopUDPRequests()
{
	EMULE_TRY
	cur_udpserver = 0;
	lastudpsearchtime = ::GetTickCount();
	lastfile = 0;
	EMULE_CATCH
}

void CDownloadQueue::SortByPriority()
{
	EMULE_TRY
	POSITION pos1, pos2;
	uint16 i = 0;
	for( pos1 = filelist.GetHeadPosition(); ( pos2 = pos1 ) != NULL; ){
	   filelist.GetNext(pos1);
	   CPartFile* cur_file = filelist.GetAt(pos2);
	   if (cur_file->GetPriority() == PR_HIGH){
		   filelist.AddHead(cur_file);
		   filelist.RemoveAt(pos2);
	   }
	   else if (cur_file->GetPriority() == PR_LOW){
		   filelist.AddTail(cur_file);
		   filelist.RemoveAt(pos2);
	   }
	   i++;
	   if (i == filelist.GetCount())
		   break;
   }
	EMULE_CATCH
}

CUpDownClient* CDownloadQueue::GetDownloadClientByIP(uint32 dwIP)
{
	EMULE_TRY

	POSITION pos,pos1,pos2;
	CUpDownClient* cur_src;
	CPartFile* cur_file;
	
	for (pos = filelist.GetHeadPosition();pos != 0;filelist.GetNext(pos))
	{
		cur_file = filelist.GetAt(pos);
		for (int sl=0;sl<SOURCESSLOTS;sl++) 
			if (!cur_file->srclists[sl].IsEmpty())
				for (pos1 = cur_file->srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;)
				{
					cur_file->srclists[sl].GetNext(pos1);
					cur_src = cur_file->srclists[sl].GetAt(pos2);

					if (dwIP == cur_src->GetIP())
					{
						return cur_src;
					}
				}
	}
	EMULE_CATCH
	return NULL;
}

CUpDownClient* CDownloadQueue::GetDownloadClientByIP_UDP(uint32 dwIP, uint16 nUDPPort)
{
	EMULE_TRY

	CPartFile* cur_file;

	for (POSITION pos = filelist.GetHeadPosition();pos != 0;)
	{
		cur_file = filelist.GetNext(pos);
		for (int sl=0;sl<SOURCESSLOTS;sl++) if (!cur_file->srclists[sl].IsEmpty())
		{
			for (POSITION pos2 = cur_file->srclists[sl].GetHeadPosition();pos2 != 0;)
			{
				CUpDownClient* cur_client = cur_file->srclists[sl].GetNext(pos2);
				if (dwIP == cur_client->GetIP() && nUDPPort == cur_client->GetUDPPort())
				{
					return cur_client;
				}
			}
		}
	}
	EMULE_CATCH
	return NULL;
}

//eklmn: by adding a new type don't forget to change constants in opcodes
void CDownloadQueue::GetDownloadStats(uint16 sources[], uint64  data[]) 
{
	EMULE_TRY

	for (int i=0;i<STATS_DL_SOURCES_TYPE;i++) sources[i] = 0;
	for (int i=0;i<STATS_DL_DATA_INFO;i++) data[i] = 0;

	for (POSITION pos = filelist.GetHeadPosition();pos != 0; filelist.GetNext(pos))
	{
		CPartFile* cur_file =  filelist.GetAt(pos);
		sources[0]+=cur_file->GetSourceCount();
		sources[1]+=cur_file->GetTransferingSrcCount();
		//extra stats
		sources[2]+=cur_file->GetOnQueueSrcCount();
		sources[3]+=cur_file->GetQueueFullSrcCount();
		sources[4]+=cur_file->GetNoNeededPartsSrcCount();
		sources[5]+=cur_file->GetConnectedSrcCount();
		sources[6]+=cur_file->GetRecHashsetSrcCount();
		sources[7]+=cur_file->GetConnectingSrcCount();
		sources[8]+=cur_file->GetConnectingViaServerSrcCount();
		sources[9]+=cur_file->GetTooManyConnsSrcCount();
		sources[10]+=cur_file->GetLow2LowSrcCount();
		sources[11]+=cur_file->GetUnknownSrcCount();
		sources[12]+=cur_file->GetErrorSrcCount();
		sources[13]+=cur_file->GetBannedSrcCount();
		sources[14]+=cur_file->TotalPacketsSavedDueToICH();
		sources[15]+=cur_file->GetSrcA4AFCount();
		sources[16]+=cur_file->GetTimeoutedSrcCount();
		sources[17]+=cur_file->GetHighQRSrcCount();
		sources[18]+=cur_file->GetNoUploadSrcCount();
		// some data
		data[0]+=cur_file->GetLostDueToCorruption();
		data[1]+=cur_file->GetGainDueToCompression();
		uint32 ui32SizeToTransfer=0;
		uint32 ui32NeededSpace=0;
		cur_file->GetSizeToTransferAndNeededSpace (ui32SizeToTransfer,ui32NeededSpace);
		if (cur_file->GetPartfileStatus() != GetResString(IDS_PAUSED) && cur_file->GetPartfileStatus() != GetResString(IDS_STOPPED))
		{
			data[2] += cur_file->GetFileSize();
			data[3] += ui32SizeToTransfer;
			data[4] += ui32NeededSpace;
		}
		data[5] += cur_file->GetFileSize();
		data[6] += ui32NeededSpace;
	}
	
	EMULE_CATCH
}


// added by FoRcHa: ////////////////////////
int CDownloadQueue::GetTransferingFiles()
{
	EMULE_TRY
	int iCount = 0;
	for(POSITION pos = filelist.GetHeadPosition(); pos != 0; filelist.GetNext(pos))
	{
		CPartFile* cur_file = filelist.GetAt(pos);
		if(cur_file->GetTransferingSrcCount() > 0)
			iCount++;
	}
	return iCount;
	EMULE_CATCH
	return 0;
}

int CDownloadQueue::GetPausedFileCount()
{
	EMULE_TRY
	int iCount = 0;
	for (POSITION pos = filelist.GetHeadPosition();pos != 0;){
		CPartFile* cur_file = filelist.GetNext(pos);
		if (cur_file->GetStatus() == PS_PAUSED)
			iCount++;
	}
	return iCount;
	EMULE_CATCH
	return 0;
}

// START enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
void CDownloadQueue::AddClientHostnameToResolve(CTypedPtrList<CPtrList, CClientSource*>* pLink) {
	if (pLink)
		hostnameResolveQueue.AddTail(pLink);

	if (hostnameResolveQueue.IsEmpty())
		return;

	if (isResolving) {
		DEBUG_ONLY(AddLogLine(false,CString("void CDownloadQueue::AddClientHostnameToResolve(CPtrList* pLink): already resolving. Exiting.")));
		return;
	}

	ResolveNextSourceHostname(); 
}

void CDownloadQueue::ResolveNextSourceHostname() {
	isResolving = true;
	CClientSource* source;
	while (1) { 
		if (hostnameResolveQueue.IsEmpty()) { //No more hostnames to resolve
			if (hsCallbackWnd) {
				delete hsCallbackWnd; //free unneccessary window
				hsCallbackWnd=NULL;
			}
			isResolving=false;
			return;
		}
		source = hostnameResolveQueue.GetHead();
		if (source->sourceType != ED2KLINK_SOURCE_HOSTNAME) {
			delete source;
			hostnameResolveQueue.RemoveHead();
		} else
			break;
	}

	memset(hostentBuffer, 0, sizeof(hostentBuffer));

	if (!hsCallbackWnd) {
		hsCallbackWnd=new CHostnameSourceWnd();
		VERIFY( hsCallbackWnd->CreateEx(0, AfxRegisterWndClass(0),_T("Emule DNS Source Socket Wnd"),WS_OVERLAPPED, 0, 0, 0, 0, NULL, NULL));
		hsCallbackWnd->m_pOwner = this;
	}

	AddLogLine(false,CString("Resolving source hostname: %s"),source->sourceHostname);
	HANDLE RetValue;
	USES_CONVERSION;
	RetValue=WSAAsyncGetHostByName(hsCallbackWnd->m_hWnd, TM_SOURCEHOSTNAMERESOLVED, 
		CT2CA(source->sourceHostname), hostentBuffer, MAXGETHOSTSTRUCT);

	if(RetValue==0) {
		if (sourceHostnameResolveRetry > 3) {
			CClientSource* hostsource;  
			hostsource = hostnameResolveQueue.RemoveHead();  
			AddLogLine(false,CString("Error calling WSAAsyncGetHostByName to resolve source "),hostsource->sourceHostname);
			delete hostsource;
			if (hsCallbackWnd) {
				delete hsCallbackWnd;
				hsCallbackWnd=NULL;
			}
			memset(hostentBuffer, 0, sizeof(hostentBuffer));
			isResolving=false;
		} 
		else {
			sourceHostnameResolveRetry +=1;
			memset(hostentBuffer, 0, sizeof(hostentBuffer));
			AddLogLine(false,CString("Source resolve retry numer %i"),sourceHostnameResolveRetry);
			ResolveNextSourceHostname();
		}
	}
}

void CDownloadQueue::SourceHostnameResolved(WPARAM wp, LPARAM lp) {

	ASSERT (hostnameResolveQueue.GetCount() > 0);

	sourceHostnameResolveRetry=0;

	CClientSource* hostsource;
	hostsource = hostnameResolveQueue.RemoveHead();  

	if (WSAGETASYNCERROR(lp) != 0){
		DEBUG_ONLY(AddLogLine(false,CString("Error resolving source : %s"),hostsource->sourceHostname));
		delete hostsource;
		delete hsCallbackWnd;
		hsCallbackWnd=NULL;
		memset(hostentBuffer, 0, sizeof(hostentBuffer));  
		ResolveNextSourceHostname();
		return;
	}

	int iBufLen = WSAGETASYNCBUFLEN(lp);
	LPHOSTENT lphost = (LPHOSTENT)malloc(iBufLen);
	#ifndef AMD
	memcpy(lphost, hostentBuffer, iBufLen);
	#else
	memcpy_amd(lphost, hostentBuffer, iBufLen);
	#endif
	hostsource->sourceID=((LPIN_ADDR)lphost->h_addr_list[0])->s_addr; 
	in_addr resolvedaddr = *(LPIN_ADDR)lphost->h_addr_list[0];
	free(lphost);

	AddLogLine(false,CString("Resolved source hostname: %s - %i:%i:%i:%i"),hostsource->sourceHostname, resolvedaddr.S_un.S_un_b.s_b1, resolvedaddr.S_un.S_un_b.s_b2, resolvedaddr.S_un.S_un_b.s_b3, resolvedaddr.S_un.S_un_b.s_b4);
	CPartFile* partfile=this->GetFileByID(hostsource->filehashkey);
	if (partfile)
		partfile->AddClientSource(hostsource);
	else {
		//TODO: notify user the partfile exists no more?
		DEBUG_ONLY(AddLogLine(false,CString("void CDownloadQueue::SourceHostnameResolved(WPARAM wp, LPARAM lp): impossible to add source because the partfile doesn't exist anymore.")));
	}
	delete hostsource;
	memset(hostentBuffer, 0, sizeof(hostentBuffer));
	ResolveNextSourceHostname();
}
// END enkeyDEV(kei-kun) -hostname sources in ED2K superLink-

CString CDownloadQueue::GetUDPSearchStatus()
{
	EMULE_TRY

	if(!m_bIsInitialized) return _T("");
#ifdef OLD_SOCKETS_ENABLED
	if (filelist.IsEmpty() || !theApp.serverconnect->IsConnected())
		return _T("");
#endif //OLD_SOCKETS_ENABLED

	CString sOut = _T("");
	if(lastfile)
	{
		sOut.Format(GetResString(IDS_UDPSEARCH_PROGRESS),
			m_nUDPSearchFile, filelist.GetCount(),
			m_nUDPSearchServer, theApp.serverlist->GetServerCount());
	}
	else
	{
		if(lastudpsearchtime)
			sOut.Format(GetResString(IDS_UDPSEARCH_NEXT), 
				CastSecondsToHM((UDPSERVERREASKTIME - (GetTickCount() - lastudpsearchtime))/1000));
		else
			sOut = GetResString(IDS_UDPSEARCH_NOTSTART);
	}
	return sOut;

	EMULE_CATCH
	return _T("");
}

// v:- ekmn: A4AF management
void CDownloadQueue::SetA4AFAutoFile(CPartFile* file)		
{
	EMULE_TRY
		
	m_A4AF_auto_file = file;
	if (file)
		theApp.glob_prefs->SetA4AFHash(file->GetFileHash());
	else {
		// Set null hash
		uchar null_hash[16];
		memset(null_hash, 0, sizeof(null_hash));
		theApp.glob_prefs->SetA4AFHash(null_hash);
	}

	EMULE_CATCH
}
// ^- ekmn: A4AF management

void CDownloadQueue::ResetCatParts(int cat)
{
	//DonGato: fix because of list update in SetCategory :S
	CTypedPtrList<CPtrList, CPartFile*> locallist;
	for(POSITION pos = filelist.GetHeadPosition(); pos != 0; filelist.GetNext(pos))
		locallist.AddTail(filelist.GetAt(pos));

	for(POSITION pos = locallist.GetHeadPosition(); pos != 0; locallist.GetNext(pos))
	{
		CPartFile* cur_file = locallist.GetAt(pos);
		if(cur_file->GetCategory() == cat) 
			cur_file->SetCategory(0);
		else if(cur_file->GetCategory() > cat) 
			cur_file->SetCategory(cur_file->GetCategory()-1);
	}
}

void CDownloadQueue::SetCatPrio(int cat, uint8 newprio)
{
	//DonGato: fix because of list update in SetCategory :S
	CTypedPtrList<CPtrList, CPartFile*> locallist;
	for(POSITION pos = filelist.GetHeadPosition(); pos != 0; filelist.GetNext(pos))
		locallist.AddTail(filelist.GetAt(pos));

	for(POSITION pos = locallist.GetHeadPosition(); pos != 0; locallist.GetNext(pos))
	{
		CPartFile* cur_file = locallist.GetAt(pos);
		if(cat == 0 || cur_file->GetCategory() == cat)
		{	
			if(newprio==PR_AUTO) 
			{
				cur_file->SetAutoPriority(true);
				cur_file->SetPriority(PR_HIGH);
			}
			else 
			{
				cur_file->SetAutoPriority(false);
				cur_file->SetPriority(newprio);
			}
		}
	}
}

void CDownloadQueue::SetCatStatus(int cat, int newstatus)
{
	bool reset = false;
	
	POSITION pos = filelist.GetHeadPosition();
	while(pos != 0) 
	{
		CPartFile* cur_file = filelist.GetAt(pos);

		if(!cur_file) 
			continue;

		if(cat == -1 || (cat == -2 && cur_file->GetCategory() == 0) ||
			(cat == 0 && CheckShowItemInGivenCat(cur_file,cat))
			 || (cat > 0 && cat == cur_file->GetCategory()))
		{
			switch(newstatus) 
			{
				case MP_CANCEL:	
					cur_file->DeleteFile();
					reset = true;
					break;
				case MP_PAUSE:	
					cur_file->PauseFile();
					break;
				case MP_STOP:	
					cur_file->StopFile();
					break;
				case MP_RESUME: 
					if(cur_file->GetStatus() == PS_PAUSED || cur_file->GetStatus() == PS_STOPPED)
						cur_file->ResumeFile();
					break;
			}
		}
		filelist.GetNext(pos);
		if(reset) 
		{
			reset = false;
			pos = filelist.GetHeadPosition();
		}
	}
}

void CDownloadQueue::MoveCat(uint8 from, uint8 to)
{
	//DonGato: fix because of list update in SetCategory :S
	CTypedPtrList<CPtrList, CPartFile*> locallist;
	for(POSITION pos = filelist.GetHeadPosition(); pos != 0; filelist.GetNext(pos))
		locallist.AddTail(filelist.GetAt(pos));

	if(from < to) 
		--to;

	uint8 mycat;

	POSITION pos = locallist.GetHeadPosition();
	for(POSITION pos = locallist.GetHeadPosition(); pos != 0; locallist.GetNext(pos))
	{
		CPartFile* cur_file =  locallist.GetAt(pos);

		if(!cur_file) 
			continue;

		mycat = cur_file->GetCategory();
		if(mycat >= min(from,to) && mycat <= max(from,to)) 
		{
			if(mycat == from) 
			{	
				cur_file->SetCategory(to); 
			}
			else 
			{
				if(from < to) 
					cur_file->SetCategory(mycat-1);
				else 
					cur_file->SetCategory(mycat+1);
			}
		}
	}
}

void CDownloadQueue::SetAutoCat(CPartFile* newfile)
{
	if(theApp.glob_prefs->GetCatCount()>1)
	{
		for (int ix=1;ix<theApp.glob_prefs->GetCatCount();ix++)
		{	
			int curPos = 0;
			CString catExt = theApp.glob_prefs->GetCategory(ix)->autocat;
			catExt.MakeLower();

			// No need to compare agains an empty AutoCat array
			if( catExt == "")
				continue;

			CString fullname = newfile->GetFileName();
			fullname.MakeLower();
			CString cmpExt = catExt.Tokenize("|", curPos);

			while (cmpExt != "")
			{
				// HoaX_69: Allow wildcards in autocat string
				//  thanks to: bluecow, khaos and SlugFiller
				if(cmpExt.Find(CString("*")) != -1 || cmpExt.Find(CString("?")) != -1)
				{
					// Use wildcards
					char* file = fullname.GetBuffer();
					char* spec = cmpExt.GetBuffer();
					if(PathMatchSpec(file, spec))
					{
						newfile->SetCategory(ix);
						return;
					}
				} else {
					if(fullname.Find(cmpExt) != -1)
					{
						newfile->SetCategory(ix);
						return;
					}
				}
				cmpExt = catExt.Tokenize("|",curPos);
			}
		}
	}
}
