// EmEngine.cpp: implementation of the CEmEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef NEW_SOCKETS

#include "EmWinNT.h"
#include "EmMt.h"
#include "ThreadPool.h"
#include "TaskProcessor.h"
#include "EmEngine.h"
#include "Tasks.h"
#include "../resource.h"
#include "../server.h"

bool CEmEngine::Init(HWND hWndUI)
{
	m_hWndUI = hWndUI;
	if (m_hStop = CreateEvent(NULL, TRUE, FALSE, NULL))
		if (m_stMain.Init())
			if (m_stTcp.Init())
			{
				// add relevant interfaces
				//m_stTcp.AddInterface(80, T_CLIENT_WEB);
				//m_stTcp.AddInterface(7821, T_CLIENT_PEER);
				return true;
			} else
			{
				ASSERT(FALSE);
				AddDebugLogLine(false, "Failed to initialize tcp engine");
			}
		else
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "Failed to initialize main engine");
		}
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "CreateEvent failed");
	}

	return false;
}

void CEmEngine::Uninit()
{
	if (m_hStop)
	{
		if (!SetEvent(m_hStop))
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "SetEvent failed");
		}

		m_stTcp.Uninit(false);
		m_stMain.Uninit();

		m_stTcp.Uninit(); // final

		if (!CloseHandle(m_hStop))
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "CloseHandle failed");
		}
		m_hStop = NULL;
	}
}

void CEmEngine::ConnectToServer(CServer* pServer /* = NULL */)
{
	EMULE_TRY

	CTask_Connect *pTask = new CTask_Connect(pServer);
	if(pTask)
	{
		m_stData.SetConnectionState(SERVER_CONNECTING);
		m_stMain.Push(pTask);
	}
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
	}

	EMULE_CATCH2
}

void CEmEngine::ConnectToAnyServer()
{
	ConnectToServer();
}

void CEmEngine::DisconnectFromServer(SOCKET hPrevServer /* = NULL */)
{
	if(!hPrevServer)
		hPrevServer = m_stData.GetConnectedServer();
	if(!hPrevServer)
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, _T("can't find connected server"));
		return;
	}
	CEmClient* pServer = m_stMain.Lookup(hPrevServer);
	if(!pServer)
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, _T("lookup for server failed"));
		return;
	}
	CTask_KillClient* pTask = new CTask_KillClient(pServer);
	if(pTask)
	{
		m_stMain.Push(pTask);
	}
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, _T("no memory"));
	}
}

#endif