//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "otherfunctions.h"
#include "emule.h"
#include "memcpy_amd.h"
//#include "Winsock2.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// Base chars for encode an decode functions
static byte base16Chars[17] = "0123456789ABCDEF";
static byte base32Chars[33] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";
#define BASE16_LOOKUP_MAX 23
static byte base16Lookup[BASE16_LOOKUP_MAX][2] = {
    { '0', 0x0 },
    { '1', 0x1 },
    { '2', 0x2 },
    { '3', 0x3 },
    { '4', 0x4 },
    { '5', 0x5 },
    { '6', 0x6 },
    { '7', 0x7 },
    { '8', 0x8 },
    { '9', 0x9 },
	{ ':', 0x9 },
    { ';', 0x9 },
    { '<', 0x9 },
    { '=', 0x9 },
    { '>', 0x9 },
    { '?', 0x9 },
    { '@', 0x9 },
    { 'A', 0xA },
    { 'B', 0xB },
    { 'C', 0xC },
    { 'D', 0xD },
    { 'E', 0xE },
    { 'F', 0xF }
};

CString CastItoXBytes(uint64 count)
{
	EMULE_TRY

	CString buffer;
	if (count < 1024)
	{
		if ( count > 0 )
		{
			buffer.Format(_T("%.0f %s"),(float)count,GetResString(IDS_BYTES).GetBuffer(0));
		}
		else
		{
			buffer.Format(_T("0 %s"),GetResString(IDS_BYTES).GetBuffer(0));
		}
	}
	else if (count < 1048576)
		buffer.Format(_T("%.0f %s"),(float)count/1024,GetResString(IDS_KBYTES).GetBuffer(0));
	else if (count < 1073741824)
		buffer.Format(_T("%.2f %s"),(float)count/1048576,GetResString(IDS_MBYTES).GetBuffer(0));
	else if (count < 1099511627776)
		buffer.Format(_T("%.2f %s"),(float)count/1073741824,GetResString(IDS_GBYTES).GetBuffer(0));
	else 
		buffer.Format(_T("%.3f %s"),(float)count/1099511627776,GetResString(IDS_TBYTES).GetBuffer(0));
	
	return buffer;

	EMULE_CATCH2

	return CString(_T(""));
}

CString CastItoIShort(uint64 count)
{
	EMULE_TRY
		
	CString buffer;
	if (count < 1000)
		buffer.Format(_T("%i"),count);
	else if (count < 1000000)
		buffer.Format(_T("%.0f%s"),(float)count/1000, GetResString(IDS_KILO).GetBuffer(0));
	else if (count < 1000000000)
		buffer.Format(_T("%.2f%s"),(float)count/1000000, GetResString(IDS_MEGA).GetBuffer(0));
	else if (count < 1000000000000)
		buffer.Format(_T("%.2f%s"),(float)count/1000000000, GetResString(IDS_GIGA).GetBuffer(0));
	else if (count < 1000000000000000)
		buffer.Format(_T("%.2f%s"),(float)count/1000000000000, GetResString(IDS_TERRA).GetBuffer(0));
	
	return buffer;

	EMULE_CATCH2

	return CString(_T(""));
}

CString CastSecondsToHM(sint32 count)
{
	EMULE_TRY

	CString buffer;
   if (count < 0) 
		buffer = _T("?"); 
   else if (count < 60) 
		buffer.Format(_T("%i %s"),count,GetResString(IDS_SECS).GetBuffer(0)); 
   else if (count < 3600) 
		buffer.Format(_T("%i:%s %s"),count/60,LeadingZero(count-(count/60)*60).GetBuffer(0),GetResString(IDS_MINS).GetBuffer(0));
   else if (count < 86400) 
		buffer.Format(_T("%i:%s %s"),count/3600,LeadingZero((count-(count/3600)*3600)/60).GetBuffer(0),GetResString(IDS_HOURS).GetBuffer(0));
   else 
		buffer.Format(_T("%i %s %i %s"),count/86400,GetResString(IDS_DAYS).GetBuffer(0),(count-(count/86400)*86400)/3600,GetResString(IDS_HOURS).GetBuffer(0)); 
	
	return buffer;

	EMULE_CATCH2

	return CString(_T(""));
} 


// -khaos--+++> Prettier.
CString CastSecondsToLngHM(uint64 count)
{
	CString buffer;
	if (count < 0)
		buffer = "?"; 
	else if (count < 60)
		buffer.Format("%I64d %s",count,GetResString(IDS_LONGSECS)); 
	else if (count < 3600) 
		buffer.Format("%I64d:%s %s",count/60,LeadingZero(count-(count/60)*60),GetResString(IDS_LONGMINS));
	else if (count < 86400) 
		buffer.Format("%I64d:%s %s",count/3600,LeadingZero((count-(count/3600)*3600)/60),GetResString(IDS_LONGHRS));
	else {
		__int64 cntDays = count/86400;
		__int64 cntHrs = (count-(count/86400)*86400)/3600;
		if (cntHrs)
			buffer.Format("%I64d %s %I64d:%s %s",cntDays,GetResString(IDS_DAYS),cntHrs,LeadingZero((uint32)(count-(cntDays*86400)-(cntHrs*3600))/60),GetResString(IDS_LONGHRS)); 
		else
			buffer.Format("%I64d %s %u %s",cntDays,GetResString(IDS_DAYS),(uint32)(count-(cntDays*86400)-(cntHrs*3600))/60,GetResString(IDS_LONGMINS));
	}
	return buffer;
} 
// <-----khaos-


CString FileHashToString(const uchar *hash) 
{
	CString ret;	
	for (int i = 0; i < 16; i++) 
		ret.AppendFormat(_T("%02X"), hash[i]);
	return ret;		
}

CString LeadingZero(uint32 units) {
	CString temp;
	if (units<10) temp.Format(_T("0%i"),units); else temp.Format(_T("%i"),units);
	return temp;
}

//<<--9/21/02
void ShellOpenFile(const CString& name) 
{
	EMULE_TRY
    ShellExecute(NULL, _T("open"), name, NULL, NULL, SW_SHOW);
	EMULE_CATCH
} 

CString URLEncode(CString sIn){
	EMULE_TRY
    CString sOut;
	
    const int nLen = sIn.GetLength() + 1;

    register LPBYTE pOutTmp = NULL;
    LPBYTE pOutBuf = NULL;
    register LPBYTE pInTmp = NULL;
    LPBYTE pInBuf =(LPBYTE)sIn.GetBuffer(nLen);
//    BYTE b = 0;
	
    //alloc out buffer
    pOutBuf = (LPBYTE)sOut.GetBuffer(nLen  * 3 - 2);//new BYTE [nLen  * 3];

    if(pOutBuf)
    {
        pInTmp	= pInBuf;
	pOutTmp = pOutBuf;
		
	// do encoding
	while (*pInTmp)
	{
	    if(isalnum(*pInTmp))
	        *pOutTmp++ = *pInTmp;
	    else
	        if(isspace(*pInTmp))
		    *pOutTmp++ = '+';
		else
		{
		    *pOutTmp++ = '%';
		    *pOutTmp++ = toHex(*pInTmp>>4);
		    *pOutTmp++ = toHex(*pInTmp%16);
		}
	    pInTmp++;
	}
	*pOutTmp = '\0';
	//sOut=pOutBuf;
	//delete [] pOutBuf;
	sOut.ReleaseBuffer();
    }
    sIn.ReleaseBuffer();
    return sOut;
	EMULE_CATCH
	return sIn;
}

inline BYTE toHex(const BYTE &x){
	return x > 9 ? x + 55: x + 48;
}

// Returns the Typename, examining the extention of the given filename
CString GetFiletypeByName(const CString& infile) {
	EMULE_TRY
	TCHAR e[_MAX_EXT];
	_tsplitpath(infile, NULL, NULL, NULL, e);
	CString ext(_tcslwr(e));

	if (ext==".mp1" || ext==".mp2" || ext==".mp3" || ext==".mp4"|| ext==".mpc" || ext==".mp+" || ext==".mpp" ||
		ext==".mpga" || ext==".au" || ext==".aac" || ext==".ape" || ext==".ogg" || ext==".flac" ||
		ext==".midi" || ext==".mid" || ext==".ra" || ext==".rmi" || ext==".aif" || ext==".aiff" ||
		ext==".wav" || ext==".wma" || ext==".au")
			return GetResString(IDS_SEARCH_AUDIO);

	if (ext==".bmp" || ext==".jpg" || ext==".jpeg" || ext==".gif" || ext==".png" || ext==".tga" ||
		ext==".tif" || ext==".xif" || ext==".tiff" || ext==".psd" || ext==".psp" || ext==".pcx" ||
		ext==".dcx" || ext==".pic" || ext==".pict" || ext==".pct" || ext==".wmf" || ext==".emf")
			return GetResString(IDS_SEARCH_PICS);

	if (ext==".avi" || ext==".divx" || ext==".ram" || ext==".rm" || ext==".rv" || ext==".ogm" ||
		ext==".mpg" || ext==".mpeg" || ext==".mpv" || ext==".mps" || ext==".m1v" || ext==".m2v" ||
		ext==".mpe" || ext==".mpa" || ext==".mp4" || ext==".vob" || ext==".mov" || ext==".asf" ||
		ext==".wmv" || ext==".vivo" || ext==".rmvb" || ext==".xvid")
			return GetResString(IDS_SEARCH_VIDEO);

	if (ext==".zip" || ext==".rar" || ext==".ace" || ext==".tar" || ext==".gz" || ext==".tgz" ||
		ext==".arj" || ext==".lha" || ext==".uc2")
			return GetResString(IDS_SEARCH_ARC);

	if (ext==".exe" || ext==".com")
			return GetResString(IDS_SEARCH_PRG);

	if (ext==".ccd"|| ext==".sub" || ext==".cue" || ext==".bin" || ext==".iso" || ext==".nrg" ||
		ext==".img" || ext==".bwa" || ext==".bwi" || ext==".bws" || ext==".bwt" || ext==".mds" ||
		ext==".mdf")
		return GetResString(IDS_SEARCH_CDIMG);

	if (ext==".doc" || ext==".txt" || ext==".pdf" || ext==".xls" || ext==".rtf" || ext==".nfo" || 
		ext==".xml" || ext==".htm" || ext==".html" || ext==".ppt")
			return GetResString(IDS_SEARCH_DOC);
	EMULE_CATCH
	return GetResString(IDS_SEARCH_ANY);
}

CString GetResString(UINT uStringID) {
	return GetResString(uStringID,theApp.glob_prefs->GetLanguageID());
}

CString GetResString(UINT uStringID,WORD languageID) {
	CString resString;
	resString.LoadString(::GetModuleHandle(NULL),uStringID, languageID );
	if (!resString.GetLength())
		resString.LoadString(::GetModuleHandle(NULL),uStringID, MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT) );
	return resString;
}

CString MakeStringEscaped(CString in) {
	EMULE_TRY
	in.Replace(_T("&"),_T("&&"));
	EMULE_CATCH
	return in;
}

bool CheckIsRegistrySet()
{
	EMULE_TRY
	CRegKey regkey;
	regkey.Create(HKEY_CLASSES_ROOT,_T("ed2k\\shell\\open\\command"));
	TCHAR rbuffer[500];	
	ULONG size = sizeof(rbuffer) / sizeof(rbuffer[0]);
	LONG ret;
	if ((ret = regkey.QueryStringValue(NULL, rbuffer, &size)) != ERROR_SUCCESS) {
		return false;
	}

	TCHAR mod[MAX_PATH];	
	if (::GetModuleFileName(NULL, mod, MAX_PATH) == 0) {
		return false;
	}

	CString reg(CString(_T("\"")) + mod + _T("\" \"%1\""));
		
	return (reg == rbuffer);
	EMULE_CATCH
	return false;
}

bool Ask4RegFix(bool dontAsk){
	EMULE_TRY
	// Barry - Make backup first	
	BackupReg();

	// check registry if ed2k links is assigned to emule
	if (!CheckIsRegistrySet()) {			
		if (dontAsk || (MessageBox(0,GetResString(IDS_ASSIGNED2K),GetResString(IDS_ED2KLINKFIX),MB_ICONQUESTION|MB_YESNO) == IDYES)){
			CRegKey regkey;
			
			TCHAR mod[MAX_PATH];		
			if (::GetModuleFileName(NULL, mod, MAX_PATH) == 0) {
				return false;
			}
			CString reg(CString(_T("\"")) + mod + _T("\" \"%1\""));
			
			regkey.Create(HKEY_CLASSES_ROOT,_T("ed2k\\shell\\open\\command"));
			regkey.SetStringValue(0,reg);	
			regkey.Create(HKEY_CLASSES_ROOT,_T("ed2k\\DefaultIcon") );// Added Shrink 
			regkey.SetStringValue(0,mod);
			regkey.Create(HKEY_CLASSES_ROOT,_T("ed2k") );
			regkey.SetStringValue(0,_T("URL: ed2k Protocol"));
			regkey.SetStringValue(_T("URL Protocol"),_T("") );
			regkey.Close();
		}
	}	
	EMULE_CATCH
	return false;
}

void BackupReg(void)
{
	EMULE_TRY
	// Look for pre-existing old ed2k links
	CRegKey regkey;
	regkey.Create(HKEY_CLASSES_ROOT, _T("ed2k\\shell\\open\\command"));
	
	TCHAR rbuffer[500];
	ULONG maxsize = sizeof(rbuffer) / sizeof(rbuffer[0]);
	// Is it ok to write new values
	if ((regkey.QueryStringValue(_T("OldDefault"), rbuffer, &maxsize) != ERROR_SUCCESS) || (maxsize == 0))
	{
		maxsize = sizeof(rbuffer) / sizeof(rbuffer[0]);
		regkey.QueryStringValue(0, rbuffer, &maxsize);
		regkey.SetStringValue(_T("OldDefault"), rbuffer);	
		regkey.Create(HKEY_CLASSES_ROOT, _T("ed2k\\DefaultIcon") );
		maxsize = sizeof(rbuffer) / sizeof(rbuffer[0]);	
		if (regkey.QueryStringValue(0, rbuffer, &maxsize) == ERROR_SUCCESS)
			regkey.SetStringValue(_T("OldIcon"), rbuffer);
	}
	regkey.Close();
	EMULE_CATCH
}

// Barry - Restore previous values
void RevertReg(void)
{
	EMULE_TRY
	// restore previous ed2k links before being assigned to emule
	CRegKey regkey;
	regkey.Create(HKEY_CLASSES_ROOT, _T("ed2k\\shell\\open\\command"));	
	TCHAR rbuffer[500];
	ULONG maxsize = sizeof(rbuffer) / sizeof(rbuffer[0]);

	if (regkey.QueryStringValue(_T("OldDefault"), rbuffer, &maxsize) == ERROR_SUCCESS)
	{
		regkey.SetStringValue(0, rbuffer);	
		regkey.Create(HKEY_CLASSES_ROOT, _T("ed2k\\DefaultIcon") );
		regkey.DeleteValue(_T("OldDefault"));
		maxsize = sizeof(rbuffer) / sizeof(rbuffer[0]);	
		if (regkey.QueryStringValue(_T("OldIcon"), rbuffer, &maxsize) == ERROR_SUCCESS)
		{
			regkey.SetStringValue(0, rbuffer);
			regkey.DeleteValue(_T("OldIcon"));
		}
	}
	regkey.Close();
	EMULE_CATCH
}

int GetMaxConnections() {
	EMULE_TRY
	OSVERSIONINFOEX osvi;
	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if(!GetVersionEx((OSVERSIONINFO*)&osvi)) {
		//if OSVERSIONINFOEX doesn't work, try OSVERSIONINFO
		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
		if(!GetVersionEx((OSVERSIONINFO*)&osvi))
			return -1;  //shouldn't ever happen
	}

	if(osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) // Windows NT product family
		return -1;  //no limits

	if(osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) { // Windows 95 product family

		if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0) { //old school 95
			HKEY hKey;
			DWORD dwValue;
			DWORD dwLength = sizeof(dwValue);
			LONG lResult;

			RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("System\\CurrentControlSet\\Services\\VxD\\MSTCP"),
				0, KEY_QUERY_VALUE, &hKey);
			lResult = RegQueryValueEx(hKey, TEXT("MaxConnections"), NULL, NULL,
				(LPBYTE)&dwValue, &dwLength);
			RegCloseKey(hKey);

			if(lResult != ERROR_SUCCESS || lResult < 1)
				return 100;  //the default for 95 is 100

			return dwValue;

		} else { //98 or ME
			HKEY hKey;
			TCHAR szValue[32];
			DWORD dwLength = sizeof(szValue);
			LONG lResult;

			RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("System\\CurrentControlSet\\Services\\VxD\\MSTCP"),
				0, KEY_QUERY_VALUE, &hKey);
			lResult = RegQueryValueEx(hKey, TEXT("MaxConnections"), NULL, NULL,
				(LPBYTE)szValue, &dwLength);
			RegCloseKey(hKey);

			LONG lMaxConnections;
			if(lResult != ERROR_SUCCESS || (lMaxConnections = _tstoi(szValue)) < 1)
				return 100;  //the default for 98/ME is 100

			return lMaxConnections;
		}         
	}
	EMULE_CATCH
	return -1;  //give the user the benefit of the doubt, most use NT+ anyway
}

WORD DetectWinVersion(BOOL &bNTBased)
{
	bNTBased	=	FALSE;
	EMULE_TRY
	OSVERSIONINFOEX osvi;
	BOOL bOsVersionInfoEx;

	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if(!(bOsVersionInfoEx = GetVersionEx((OSVERSIONINFO*)&osvi)))
	{
		osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		if(!GetVersionEx((OSVERSIONINFO*)&osvi)) 
		return FALSE;
	}

	switch(osvi.dwPlatformId)
	{
		case VER_PLATFORM_WIN32_NT:
			bNTBased	=	TRUE;
			if(osvi.dwMajorVersion <= 4)
				return _WINVER_NT4_;
			if(osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0)
				return _WINVER_2K_;
			if(osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1)
				return _WINVER_XP_;
			if(osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2)	// netwolf: Windows Server 2003
				return _WINVER_SE_;
			break;
      
		case VER_PLATFORM_WIN32_WINDOWS:
			if(osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
				return _WINVER_95_; 
			if(osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10)
				return _WINVER_98_; 
			if(osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90)
				return _WINVER_ME_; 
			break;
		
		default:
			break;
	}
	EMULE_CATCH
	return _WINVER_95_;		// there should'nt be anything lower than this
}

_int64 GetFreeDiskSpaceX(LPCTSTR pDirectory)
{	
	FARPROC pGetDiskFreeSpaceEx;

	pGetDiskFreeSpaceEx = GetProcAddress(GetModuleHandle(_T("kernel32.dll")), "GetDiskFreeSpaceExA");

	if(pGetDiskFreeSpaceEx)
	{
		ULARGE_INTEGER nFreeDiskSpace;
		ULARGE_INTEGER dummy;
		GetDiskFreeSpaceEx(pDirectory, &nFreeDiskSpace, &dummy, &dummy);
		return nFreeDiskSpace.QuadPart;
	}
	else 
	{
		TCHAR cDrive[16];
		TCHAR *p = _tcschr(pDirectory, '\\');
		if(p)
		{	
			#ifndef AMD
			memcpy(cDrive, pDirectory, p-pDirectory);
			#else
			memcpy_amd(cDrive, pDirectory, p-pDirectory);
			#endif
			cDrive[p-pDirectory] = '\0';
		}
		else
			_tcscpy(cDrive, pDirectory);
		DWORD dwSectPerClust, dwBytesPerSect, dwFreeClusters, dwDummy;
		GetDiskFreeSpace(cDrive, &dwSectPerClust, &dwBytesPerSect, &dwFreeClusters, &dwDummy);
		return (dwFreeClusters * dwSectPerClust * dwBytesPerSect);
	}
	
//	return 0;
}

HICON LoadImgLstIcon(LPCTSTR lpszName, int cx, int cy)
{
	static HICON lasticon = NULL;
	if(lasticon)
		DestroyIcon(lasticon);
	lasticon = (HICON)::LoadImage(AfxGetInstanceHandle(), lpszName, IMAGE_ICON, cx, cy, 0);
	return lasticon;
}

//For  Rate File // 
CString GetRateString(uint16 rate) {
	EMULE_TRY
	switch (rate)
	{
	case 0:
		return GetResString(IDS_CMT_NOTRATED);
		break;
	case 1:
		return GetResString(IDS_CMT_FAKE);
		break;
	case 2:
		return GetResString(IDS_CMT_POOR);
		break;
	case 3:
		return GetResString(IDS_CMT_GOOD);
		break;
	case 4:
		return GetResString(IDS_CMT_FAIR);
		break;
	case 5:
		return GetResString(IDS_CMT_EXCELLENT);
		break;
	}
	EMULE_CATCH
	return GetResString(IDS_CMT_NOTRATED);
} 
//end rate File//

// Returns a BASE32 encoded byte array
//
// [In]
//   buffer: Pointer to byte array
//   bufLen: Lenght of buffer array
//
// [Return]
//   CString object with BASE32 encoded byte array
CString EncodeBase32(const unsigned char* buffer, unsigned int bufLen)
{
	CString Base32Buff(_T(""));
    EMULE_TRY
	unsigned int i, index;
    unsigned char word;
//	int pos = 0;

    for(i = 0, index = 0; i < bufLen;) {

		// Is the current word going to span a byte boundary?
        if (index > 3) {
            word = (buffer[i] & (0xFF >> index));
            index = (index + 5) % 8;
            word <<= index;
            if (i < bufLen - 1)
                word |= buffer[i + 1] >> (8 - index);

            i++;
        } else {
            word = (buffer[i] >> (8 - (index + 5))) & 0x1F;
            index = (index + 5) % 8;
            if (index == 0)
               i++;
        }

		Base32Buff += (char) base32Chars[word];
    }
	EMULE_CATCH
    return Base32Buff;
}

// Returns a BASE16 encoded byte array
//
// [In]
//   buffer: Pointer to byte array
//   bufLen: Lenght of buffer array
//
// [Return]
//   CString object with BASE16 encoded byte array
CString EncodeBase16(const unsigned char* buffer, unsigned int bufLen)
{
	CString Base16Buff(_T(""));
	EMULE_TRY
	for(unsigned int i = 0; i < bufLen; i++) {
		Base16Buff += base16Chars[buffer[i] >> 4];
		Base16Buff += base16Chars[buffer[i] & 0xf];
	}
	EMULE_CATCH
    return Base16Buff;
}

// Decodes a BASE16 string into a byte array
//
// [In]
//   base16Buffer: String containing BASE16
//   base16BufLen: Lenght BASE16 coded string's length
//
// [Out]
//   buffer: byte array containing decoded string
void DecodeBase16(LPCTSTR base16Buffer, unsigned int base16BufLen, byte *buffer)
{
	EMULE_TRY
    memset(buffer, 0, DecodeLengthBase16(base16BufLen));
  
    for(unsigned int i = 0; i < base16BufLen; i++) {
		int lookup = toupper(base16Buffer[i]) - '0';

        // Check to make sure that the given word falls inside a valid range
		byte word = 0;
        
		if ( lookup < 0 || lookup >= BASE16_LOOKUP_MAX)
           word = 0xFF;
        else
           word = base16Lookup[lookup][1];

		if(i % 2 == 0) {
			buffer[i/2] = word << 4;
		} else {
			buffer[(i-1)/2] |= word;
		}
	}
	EMULE_CATCH
}

// Calculates length to decode from BASE16
//
// [In]
//   base16Length: Actual length of BASE16 string
//
// [Return]
//   New length of byte array decoded
int	DecodeLengthBase16(int base16Length)
{
	return base16Length / 2;
}

void UpdateURLMenu(CMenu &menu,int &counter)
{
	EMULE_TRY
	counter=0;
	theApp.webservices.RemoveAll();
	CString name,url,sbuffer;
	char buffer[1024];
	int lenBuf = 1024;

	FILE* readFile= _tfopen(CString(theApp.glob_prefs->GetConfigDir())+_T("webservices.dat"), _T("r"));
	if (readFile!=NULL)
	{
		while (!feof(readFile))
		{
			if (fgets(buffer,lenBuf,readFile)==0)
				break;
			sbuffer=buffer;

			// ignore comments & too short lines
			if (sbuffer.GetAt(0) == '#' || sbuffer.GetAt(0) == '/' || sbuffer.GetLength()<5)
				continue;

			if (sbuffer == "<separator>\n" || sbuffer == "<separator>") // Sp4rK patch !!
			{
				menu.AppendMenu(MF_SEPARATOR);
			}
			else
			{
				int pos=sbuffer.Find(',');
				if (pos>0)
				{
					counter++;
					menu.AppendMenu(MF_STRING,MP_WEBURL+(counter-1), sbuffer.Left(pos).Trim());
					theApp.webservices.Add(sbuffer.Right(sbuffer.GetLength()-pos-1).Trim());
				}
			}
		}
		fclose(readFile);
	}
	EMULE_CATCH
}

void RunURL(CAbstractFile* file, CString urlpattern)
{
	EMULE_TRY
	if (file!=NULL) {
	    // Convert hash to hexadecimal text and add it to the URL
	    urlpattern.Replace(_T("#hashid"), EncodeBase16(file->GetFileHash(), 16));
    
	    // Add file size to the URL
	    CString temp;
	    temp.Format(_T("%u"),file->GetFileSize());
	    urlpattern.Replace(_T("#filesize"), temp);
    
	    // add filename to the url
	    urlpattern.Replace(_T("#filename"),URLEncode(file->GetFileName()));
	}
	// Open URL
	ShellExecute(NULL, NULL, urlpattern, NULL, theApp.glob_prefs->GetAppDir(), SW_SHOWDEFAULT);
	EMULE_CATCH
}

CString GetClientVersionString(eClientTypes clientype,uint32 uVersion)
{
	CString cbuffer	=	_T("0");
	EMULE_TRY
	double dVersion	=	0.;
	double dVersion1	=	0.;
	double dVersion2	=	0.;
	uint32 uDivider		=	10;
	switch(clientype)
	{
	case SO_EMULE:
	case SO_CDONKEY:
	case SO_SHAREAZA:
	case SO_LMULE:
		cbuffer.Format(_T("0.%02X"),uVersion);
		break;
	case SO_EDONKEYHYBRID:
		//cbuffer.Format(_T("%i"),uVersion - 1000);
		// [Lord KiRon]
		// I am not sure about this but this should take care of some stange
		// v1432 ve getting , so I assume it's 43.2
		// v44.7 for sure  reports 10447 so may be here some hex math goes on but 
		// until some1 finds it - this works
		while(uVersion/uDivider !=0 )uDivider*=10;
		uVersion	=	uVersion - (uDivider/10);
		dVersion	=	(double)(uVersion);
		if (dVersion > 100. )
		{
			dVersion /= 10.;
		}
		cbuffer.Format(_T("%.1f") , dVersion);
		break;
	case SO_EDONKEY:
		cbuffer.Format(_T("%i"),uVersion);
		break;
	case SO_MLDONKEY:
		cbuffer.Format(_T("%i"),uVersion);
		break;
	case SO_OLDEMULE:
		// Lord KiRon - not needed any more because version corrected at read packet time
		/*if(uVersion == 0x2B)
		{
			uVersion = 0x22;
		}*/
		cbuffer.Format(_T("%i"),uVersion);
		break;
	default:
		break;
	};
	EMULE_CATCH
	return (cbuffer==_T("0.00")||cbuffer==_T("0.0")||cbuffer==_T("0"))?_T(" ?"):cbuffer ;
}

CString StringLimit(const CString& in,uint16 length){
	if (in.GetLength()<=length || length<10) return in;
	return (in.Left(length-8)+_T("...")+in.Right(8));
}

uint8 GetRealPrio(uint8 in) {
	switch(in) {
		case 4 : return 0;
		case 0 : return 1;
		case 1 : return 2;
		case 2 : return 3;
		case 3 : return 4;
	}
	return 0;
}

CString GetStatusDLQueueString(eDLQState State)
{
	CString stateString = GetResString(IDS_UNKNOWN);
	EMULE_TRY
	switch(State)
	{
	case DS_DOWNLOADING:
		stateString	=	GetResString(IDS_TRANSFERRING);
		break;
	case DS_ONQUEUE:
		stateString	=	GetResString(IDS_ONQUEUE);
		break;
	case DS_CONNECTED:
		stateString	=	GetResString(IDS_ASKING);
		break;
	case DS_CONNECTING:
		stateString	=	GetResString(IDS_CONNECTING);
		break;
	case DS_WAITCALLBACK:
		stateString	=	GetResString(IDS_CONNVIASERVER);
		break;
	case DS_REQHASHSET:
		stateString	=	GetResString(IDS_RECHASHSET);
		break;
	case DS_NONEEDEDPARTS:
		stateString	=	GetResString(IDS_NONEEDEDPARTS);
		break;
	case DS_TOOMANYCONNS:
		stateString	=	GetResString(IDS_TOOMANYCONNS);
		break;
	case DS_LOWTOLOWIP:
		stateString	=	GetResString(IDS_NOCONNECTLOW2LOW);
		break;
	case DS_BANNED:
		stateString	=	GetResString(IDS_BANNED);
		break;
	case DS_ERROR:
		stateString	=	GetResString(IDS_ERROR_STATE); 
		break;
	case DS_NONE:
		stateString	=	GetResString(IDS_UNKNOWN);
		break;
	};
	EMULE_CATCH
	return stateString;
}

CString GetStatusULQueueString(eULQState State)
{
	CString stateString = GetResString(IDS_UNKNOWN);
	EMULE_TRY
	switch(State)
	{
	case US_UPLOADING:
		stateString	=	GetResString(IDS_TRANSFERRING);
		break;
	case US_ONUPLOADQUEUE:
		stateString	=	GetResString(IDS_ONQUEUE);
		break;
	case US_WAITCALLBACK:
		stateString	=	GetResString(IDS_CONNVIASERVER);
		break;
	case US_CONNECTING:
		stateString	=	GetResString(IDS_CONNECTING);
		break;
	case US_PENDING:
		stateString	=	GetResString(IDS_PENDING);
		break;
	case US_LOWTOLOWIP:
		stateString	=	GetResString(IDS_NOCONNECTLOW2LOW);
		break;
	case US_BANNED:
		stateString	=	GetResString(IDS_BANNED);
		break;
	case US_ERROR:
		stateString	=	GetResString(IDS_ERROR_STATE);
		break;
	case US_NONE:
		stateString	=	GetResString(IDS_UNKNOWN);
		break;
	};
	EMULE_CATCH
	return stateString;
}

CString GetClientNameString(eClientTypes clienttype)
{
	CString nameString = GetResString(IDS_UNKNOWN);
	EMULE_TRY
	switch(clienttype)
	{
	case SO_EMULE:
		nameString	=	_T("eMule");
		break;
	case SO_CDONKEY:
		nameString	=	_T("CDonkey");
		break;
	case SO_LMULE:
		nameString	=	_T("LMule");
		break;
	case SO_SHAREAZA:
		nameString	=	_T("Shareaza");
		break;
	case SO_EDONKEYHYBRID:
		nameString	=	_T("Hybrid");
		break;
	case SO_EDONKEY:
		nameString	=	_T("eDonkey");
		break;
	case SO_MLDONKEY:
		nameString	=	GetResString(IDS_OLDMLDONKEY);
		break;
	case SO_OLDEMULE:
		nameString	=	GetResString(IDS_OLDEMULE);
		break;
	};
	EMULE_CATCH
	return 	nameString;
}

CString GetClientNameAndVersionString(CUpDownClient *client) {
	CString buffer(_T("?"));
	EMULE_TRY
	bool isPlus=(client->PlusVersion() && client->PlusVersion()<0x2600);
	switch(client->GetClientSoft())
	{
	case SO_EMULE:
        if(isPlus)	//Cax2 - second check - don't display the 38e (pre v1?) mods
		{	//Cax2 - assuming v1 v1a v1b - v1.1 v1.1a etc...
			int num=(client->PlusVersion() & 0xF0)>>4;
			int letter=(client->PlusVersion() & 0x0F);
			letter=(letter==0)?(int)' ':letter+(int)'a'-1;
			if (num>0)
				buffer.Format(_T("Plus v%u.%u%c"), client->PlusVersion()>>8,num, (char)letter);
			else 
				buffer.Format(_T("Plus v%u%c"), client->PlusVersion()>>8,(char)letter);
				//buffer.Format("plus v1 %c",(char) letter);
			break;
        }
	case SO_LMULE:  // Other Mules fall under one catagory
	case SO_SHAREAZA:
	case SO_OLDEMULE:
	case SO_CDONKEY:
        {
			if (!client->GetModVersion().IsEmpty() && !isPlus)
				buffer.Format(_T("%s v%s [%s]"),GetClientNameString(client->GetClientSoft()),
					GetClientVersionString(client->GetClientSoft(), client->GetMuleVersion()), client->GetModVersion());
			else
				buffer.Format(_T("%s v%s"),GetClientNameString(client->GetClientSoft()),
					GetClientVersionString(client->GetClientSoft(), client->GetMuleVersion()));
        }
		break;
	case SO_UNKNOWN:
		buffer.Format(_T("???"));
		break;
	default:
		buffer.Format(_T("%s v%s"),GetClientNameString(client->GetClientSoft()),
            GetClientVersionString(client->GetClientSoft(),client->GetVersion()));
		break;
	}
	EMULE_CATCH
    return buffer;
}

// START - enkeyDEV(Ottavio84) -ChangeDir-
int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData) {
	EMULE_TRY
	switch(uMsg){
	case BFFM_INITIALIZED:
		if (lpData){
			SendMessage(hwnd,BFFM_SETSELECTION,TRUE,lpData);
		}
		break;
	default:
	   break;
	}
	EMULE_CATCH
	return 0;
}

CString BrowseFolder(HWND hwndOwner, LPCTSTR pszTitle, LPCTSTR pszStartFolder) {
	CString retstring(pszStartFolder);
	EMULE_TRY
    TCHAR pszDisplayName[MAX_PATH];
	LPITEMIDLIST lpID;
	BROWSEINFO bi;
	
	bi.hwndOwner = hwndOwner;
	bi.pidlRoot = NULL;
	bi.pszDisplayName = pszDisplayName;
	bi.lpszTitle = pszTitle;
	bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT | BIF_NEWDIALOGSTYLE;
	bi.lpfn = BrowseCallbackProc;
	bi.lParam = (LPARAM)pszStartFolder;
	bi.iImage = NULL;
	lpID = SHBrowseForFolder(&bi);
	if (lpID != NULL){
		if (SHGetPathFromIDList(lpID, pszDisplayName))
			retstring.Format(_T("%s"), pszDisplayName);
		LPMALLOC pMalloc = NULL; 
		HRESULT hResult = SHGetMalloc (&pMalloc); 
		if (hResult == NOERROR && pMalloc) 
			pMalloc->Free (lpID); 
		pMalloc->Release();
	}
	EMULE_CATCH
	return retstring;
}
// END - enkeyDEV(Ottavio84) -ChangeDir-

//SyruS: cleanup centralized functions
CString URLDecode(CString inStr) {
	EMULE_TRY
	CString outStr("");
	int more = -1; // EC 11.07.03
	for (int x = 0;x < inStr.GetLength() ; ++x )
	{
		if ( inStr.GetAt(x)== '%' && x+2 < inStr.GetLength() && _istxdigit(inStr.GetAt(x+1)) && _istxdigit(inStr.GetAt(x+2)) ) {
			TCHAR hexstr[3]; hexstr[2]=0;
			int Result; // EC 11.07.03
			int sumb; // EC 11.07.03
			_tcsncpy(hexstr, inStr.Mid(x+1,2).GetBuffer(), 2);
			x = x + 2;

			// EC 11.07.03 UTF-8 Decoding
			Result = _tcstoul(hexstr, NULL, 16);
			if ((Result & 0xc0) == 0x80) // 10xxxxxx (continuation byte)
			{			
	         sumb = (sumb << 6) | (Result & 0x3f) ;	// Add 6 bits to sumb
	         if (--more == 0) outStr.AppendChar((char) sumb) ; // Add char to sbuf
            } else 
			if ((Result & 0x80) == 0x00) 		// 0xxxxxxx (yields 7 bits)
			{
	          outStr.AppendChar((char) Result) ;			// Append Character
            } else 
			if ((Result & 0xe0) == 0xc0) 		// 110xxxxx (yields 5 bits)
			{
			  sumb = Result & 0x1f;
	          more = 1;				// Expect 1 more byte
			} else 
			if ((Result & 0xf0) == 0xe0) 		// 1110xxxx (yields 4 bits)
			{
			  sumb = Result & 0x0f;
	          more = 2;				// Expect 2 more bytes
			} else 
			if ((Result & 0xf8) == 0xf0) 
			{		// 11110xxx (yields 3 bits)
			  sumb = Result & 0x07;
			  more = 3;				// Expect 3 more bytes
			} else 
			if ((Result & 0xfc) == 0xf8) 
			{		// 111110xx (yields 2 bits)
			 sumb = Result & 0x03;
			 more = 4;				// Expect 4 more bytes
			}	else /*if ((b & 0xfe) == 0xfc)*/ 	// 1111110x (yields 1 bit)
			{
			 sumb = Result & 0x01;
			 more = 5;				// Expect 5 more bytes
			}
		}
		else {
			outStr.AppendChar( inStr.GetAt(x));
		}
		// EC 11.07.03 UTF-8 Decoding Ends
	}
	return outStr;
	EMULE_CATCH
	return inStr;
}

CString LeadingCaps(CString inStr, CString exceptStr /*=""*/) {
	//there is no upper after chars defined in exceptStr (recommended: "'")
	EMULE_TRY
	if (!inStr.IsEmpty())
	{
		CString tempStr(inStr.GetAt(0));
		tempStr.MakeUpper();
		inStr.SetAt(0, tempStr.GetAt(0));

		int max = inStr.GetLength()-1;
		for (int pos=0; pos < max; pos++)
		{
			if (!IsCharAlpha(inStr.GetAt(pos)) && exceptStr.Find(inStr.GetAt(pos)) < 0)
			{
				tempStr=inStr.GetAt(pos+1);
				tempStr.MakeUpper();
				inStr.SetAt(pos+1,tempStr.GetAt(0));
			}
		}
	}
	EMULE_CATCH
	return inStr;
}

CString DeleteNonAlphaNumeric(CString inStr, CString exceptStr /*=""*/, bool onlyRep /*=true*/) {
	//with standard-params output simply won't contain any nonalphanumeric chars
	//set exceptStr to chars u don't want to be deleted (i.e. pipe as separator-char "|")
	//if onlyRep==true only repeated non-alphanumeric chars will be reduced to one occurence
	//if onlyRep==false all non-alphanumeric chars are deleted except the allowed ones in exceptStr
	EMULE_TRY
	TCHAR curChar;
	int pos = 0;
	while (pos < inStr.GetLength())
	{
		curChar = inStr.GetAt(pos);
		if (!IsCharAlphaNumeric(curChar) && exceptStr.Find(curChar) < 0
		&& (!onlyRep || (pos+1 < inStr.GetLength() && inStr.Find(curChar, pos+1) == pos+1 )))
			inStr.Delete(pos);
		else
			pos++;
	}
	EMULE_CATCH
	return inStr;
}

CString ReplaceDotsWithSpaces(CString inStr, CString exceptStr /*=""*/, CString addClear /*=""*/, bool keepExt /*=false*/) {
	//with standard-params simply all dots are replaced with spaces :)
	//dots won't be replaced between 2 chars of exceptStr (numbers for versions and dates)
	//if u want to replace additional chars with spaces set addClear (invalid & nonsense chars)
	//set keepExt=true for keeping the extension-dot
	EMULE_TRY
	int extpos = inStr.ReverseFind('.');
	int max = inStr.GetLength()-1;
	for (int pos=0; pos <= max; pos++)
	{
		if (inStr.GetAt(pos) == '.')
		{
			//replace dot at beginning and end
		   	if( pos == 0 || pos == max )
				inStr.SetAt(pos, ' ');
			//replace dot only if not between 2 chars of exceptStr
			else if( exceptStr.Find(inStr.GetAt(pos-1)) < 0 || exceptStr.Find(inStr.GetAt(pos+1)) < 0)
				inStr.SetAt(pos, ' ');
		}
		//replace chars from addClear with space, too
		else if (addClear.Find(inStr.GetAt(pos)) >= 0)
			inStr.SetAt(pos, ' ');
	}
	if (keepExt && extpos >= 0) inStr.SetAt(extpos, '.');
	EMULE_CATCH
	return inStr;
}

//TODO: ms-help://MS.VSCC/MS.MSDNVS.1031/vclib/html/_crt_Interpretation_of_Multibyte.2d.Character_Sequences.htm
CString CleanupFilename(CString inStr) {
	EMULE_TRY
	CString filename(inStr);	//process on local copy only
	CString strList(_T(""));	//list of pipe-separated tokens
	CString resToken(_T(""));	//one token out of strList
	int foundPos = 0;			//position where something is found
	int curPos = 0;				//position for tokenize() (by reference!)
	CString extension(_T(""));	//only the extension of filename with dot

	filename.SetString(URLDecode(filename.MakeLower()));

	//cut off & cleanup extension before further process
	foundPos = filename.ReverseFind('.');
	if( foundPos >= 0 && foundPos < filename.GetLength()-1 )					//one char minimum
	{
		extension.SetString(filename.Right(filename.GetLength()-foundPos));		//copy extension with dot
		extension.SetString(DeleteNonAlphaNumeric(extension, _T("."), false));	//only alphanum & dot allowed
		filename.SetString(filename.Left(foundPos));							//cut to filename without dot
		while (true)
		{	//cutoff doubled extensions in filename
			filename.TrimRight(_T(" .,"));
			curPos = filename.GetLength()-extension.GetLength();
			if (curPos < 0)
				break;
			foundPos = filename.Find(extension,curPos);
			if (foundPos == curPos)
				filename.SetString(filename.Left(curPos));
			else
				break;
		}
	}
	if (extension.IsEmpty()) extension.SetString(_T(".non"));

	//remove all between [ and ]
	if (theApp.glob_prefs->GetFilenameCleanupTags())
	{
		int foundPo2, foundPo3;
		while (true)
		{
			foundPos = filename.Find('[');
			if (foundPos < 0)
				break;
			foundPo2 = filename.Find(']',foundPos);
			if (foundPo2 > foundPos)
			{
				foundPo3 = filename.Find('[',foundPos+1);
				while (foundPo3 > 0 && foundPo3 < foundPo2)
				{	//check for nested brackets
					foundPos = foundPo3;
					foundPo3 = filename.Find('[',foundPos+1);
				}
				filename.SetString(filename.Left(foundPos) + _T(" ") + filename.Right(filename.GetLength()-foundPo2-1));
			}
			else
				break;
		}
	}

	strList.SetString(theApp.glob_prefs->GetFilenameCleanups().MakeLower());
	curPos = 0;
	resToken.SetString(strList.Tokenize(_T("|"),curPos));
	while (!resToken.IsEmpty())
	{
		filename.Replace(resToken,_T(" "));
		resToken.SetString(strList.Tokenize(_T("|"),curPos));
	}

	filename.SetString(ReplaceDotsWithSpaces(filename, _T("01234567689"), _T("_+^!$%=~@;\\\"/:*?<>|"), false)); //spaceholders

	//advertising cleanup (lowercase without dots)
	filename.Replace(_T("shared by"), _T(""));
	filename.Replace(_T("shared for"), _T(""));
	filename.Replace(_T("shared via"), _T(""));
	filename.Replace(_T("shared "), _T(""));
	filename.Replace(_T("found at"), _T(""));
	filename.Replace(_T("found via xxx"), _T(""));	//filedonkey.com special
	filename.Replace(_T("found via"), _T(""));
	filename.Replace(_T("link by"), _T(""));
	filename.Replace(_T("linked at"), _T(""));
	filename.Replace(_T("linked by"), _T(""));
	filename.Replace(_T("linked "), _T(""));
	filename.Replace(_T("powered by"), _T(""));
	filename.Replace(_T("-powered-by-"), _T(""));	//eselfilme.de special
	filename.Replace(_T("powered "), _T(""));
	filename.Replace(_T("sponsored for"), _T(""));
	filename.Replace(_T("sponsored by"), _T(""));
	filename.Replace(_T("sponsored "), _T(""));

	filename.SetString(DeleteNonAlphaNumeric(filename, _T(""), true));	//reduce doubles

	//remove spaces inside brackets
	filename.Replace(_T("( "), _T("("));
	filename.Replace(_T(" )"), _T(")"));
	filename.Replace(_T("[ "), _T("["));
	filename.Replace(_T(" ]"), _T("]"));
	filename.Replace(_T("{ "), _T("{"));
	filename.Replace(_T(" }"), _T("}"));
	//remove empty brackets
	filename.Replace(_T("()"), _T(""));
	filename.Replace(_T("[]"), _T(""));
	filename.Replace(_T("{}"), _T(""));

	filename.SetString(LeadingCaps(filename, _T("'`0123456789")));	//leadingcaps except after "'`" and numbers

	//special uppercasing by netwolf and DoubleT
	filename.Replace(_T("Cd"),_T("CD"));
	filename.Replace(_T("Vcd"),_T("VCD"));
	filename.Replace(_T("Svcd"),_T("SVCD"));
	filename.Replace(_T("Dvd"),_T("DVD"));
	filename.Replace(_T("Divx"),_T("DivX"));
	filename.Replace(_T("Xvid"),_T("XviD"));
	filename.Replace(_T("Ac3"),_T("AC3"));
	filename.Replace(_T("Mp3"),_T("MP3"));
	filename.Replace(_T("Vbr"),_T("VBR"));
	filename.Replace(_T("Cbr"),_T("CBR"));
	filename.Replace(_T("Vhs"),_T("VHS"));
	filename.Replace(_T("Tv"),_T("TV"));
	filename.Replace(_T("Fsk"),_T("FSK"));
	filename.Replace(_T("Dvb"),_T("DVB"));
	filename.Replace(_T("Emuleplus"),_T("eMule Plus"));
	filename.Replace(_T("Emule"),_T("eMule"));
	//make "v" in front of versions lowercase
	filename.Replace(_T("V1"),_T("v1"));
	filename.Replace(_T("V2"),_T("v2"));
	filename.Replace(_T("V3"),_T("v3"));
	filename.Replace(_T("V4"),_T("v4"));
	filename.Replace(_T("V5"),_T("v5"));
	filename.Replace(_T("V6"),_T("v6"));
	filename.Replace(_T("V7"),_T("v7"));
	filename.Replace(_T("V8"),_T("v8"));
	filename.Replace(_T("V9"),_T("v9"));
	filename.Replace(_T("V0"),_T("v0"));
	//roman numerals up to 20
	filename.Replace(_T("Iii"),_T("III"));
	filename.Replace(_T("Viii"),_T("VIII"));
	filename.Replace(_T("Vii"),_T("VII"));
	filename.Replace(_T("Xiii"),_T("XIII"));
	filename.Replace(_T("Xii"),_T("XII"));
	filename.Replace(_T("Xiv"),_T("XIV"));
	filename.Replace(_T("Xviii"),_T("XVIII"));
	filename.Replace(_T("Xvii"),_T("XVII"));
	filename.Replace(_T("Xvi"),_T("XVI"));
	filename.Replace(_T("Xix"),_T("XIX"));
	filename.Replace(_T("Xxx"),_T("XXX"));
	filename.Append(_T(" "));
	filename.Replace(_T("Ii "),_T("II "));
	filename.Replace(_T("Iv "),_T("IV "));
	filename.Replace(_T("Vi "),_T("VI "));
	filename.Replace(_T("Ix "),_T("IX "));
	filename.Replace(_T("Xi "),_T("XI "));
	filename.Replace(_T("Xx "),_T("XX "));

	//final steps
	filename.TrimRight(_T(" -.,#&([{"));
	filename.TrimLeft(_T(" -.,#&)]}"));
	if (filename.IsEmpty())
		return inStr;
	filename.Append(extension);
	return filename;
	EMULE_CATCH
	return inStr;
}

CString GetErrorMessage(CException* e) {
	ASSERT(e != NULL);
	EMULE_TRY
	const size_t bs = 255;
	TCHAR b[bs];
	e->GetErrorMessage(b, bs);
	return CString(b);
	EMULE_CATCH
	return CString(_T("N/A"));
}

CString DialogBrowseFile(CString Filters, CString DefaultFileName) {
        CFileDialog myFileDialog(true,NULL,LPCTSTR(DefaultFileName),
							 OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, LPCTSTR(Filters), NULL, 
							 0/*automatically use explorer style open dialog on systems which support it*/);
        myFileDialog.DoModal();
        return myFileDialog.GetPathName();
	}

#ifdef UNICODE
unsigned long inet_addr(LPCTSTR cp) {
	USES_CONVERSION;
	return inet_addr(T2A(cp));
}
#endif

CString Crypt(CString sNormal)
{
	ULONG lStringLen = sNormal.GetLength();
	BYTE *pBuf = new BYTE[lStringLen * 2 + 1];
	strcpy((char *)pBuf, sNormal);

	for (ULONG dwBufSize = lStringLen; dwBufSize--; )
	{
		((PSTR) pBuf)[2 * dwBufSize + 1]	= (((PSTR) pBuf)[dwBufSize] % 0x10) + 'A';
		((PSTR) pBuf)[2 * dwBufSize]		= (((PSTR) pBuf)[dwBufSize] / 0x10) + 'A';
	}

	pBuf[lStringLen*2] = 0;

	CString str(pBuf);
	delete[] pBuf;
	
	return str;
}

CString Decrypt(CString sCrypted)
{
	ULONG lStringLen = sCrypted.GetLength();
	lStringLen /= 2;
	BYTE *pBuf = new BYTE[lStringLen + 1];

	for (ULONG dwPos = 0; dwPos < lStringLen; dwPos++)
	{
		char chHigh = ((PCSTR) sCrypted)[dwPos * 2] - 'A';
		char chLow  = ((PCSTR) sCrypted)[dwPos * 2 + 1] - 'A';

		((PSTR) pBuf)[dwPos] = (chHigh << 4) + chLow;
	}
	
	pBuf[lStringLen] = 0;

	CString str(pBuf);
	delete[] pBuf;
	
	return str;
}

bool CheckShowItemInGivenCat(CPartFile* file, int inCategory) 
{
	// easy normal cases
	if((inCategory > 0 && inCategory == file->GetCategory()) || 
		(inCategory == 0 && theApp.glob_prefs->GetAllcatType() == 0)) 
	{
		return true;
	}

	// ToDo

	if(inCategory == 0 && theApp.glob_prefs->GetAllcatType() > 0)
	{
		switch(theApp.glob_prefs->GetAllcatType()) 
		{
			case 1 : return (file->GetCategory() == 0);
			case 2 : return (file->IsPartFile());
			case 3 : return (!file->IsPartFile());
			case 4 : return ((file->GetStatus() == PS_READY || file->GetStatus() == PS_EMPTY) && file->GetTransferingSrcCount() == 0);
			case 5 : return ((file->GetStatus() == PS_READY|| file->GetStatus() == PS_EMPTY) && file->GetTransferingSrcCount() > 0);
			case 6 : return (file->GetStatus() == PS_ERROR);
			case 7 : return (file->GetStatus() == PS_PAUSED);
			case 8 : return (file->GetStatus() == PS_STOPPED);
			case 10 : return file->IsMovie();
			case 11 : return (ED2KFT_AUDIO == GetED2KFileTypeID(file->GetFileName()));
			case 12 : return file->IsArchive();
			case 13 : return (ED2KFT_CDIMAGE == GetED2KFileTypeID(file->GetFileName()));
		}
	}

	return false;
}

CString GetCatTitle(int catid) {
	switch (catid) {
		case 0 : return GetResString(IDS_ALL);
		case 1 : return GetResString(IDS_ALLOTHERS);
		case 2 : return GetResString(IDS_STATUS_NOTCOMPLETED);
		case 3 : return GetResString(IDS_DL_TRANSFCOMPL);
		case 4 : return GetResString(IDS_WAITING);
		case 5 : return GetResString(IDS_DOWNLOADING);
		case 6 : return GetResString(IDS_ERRORLIKE);
		case 7 : return GetResString(IDS_PAUSED);
		case 8 : return GetResString(IDS_STOPPED);
		case 10 : return GetResString(IDS_VIDEO);
		case 11 : return GetResString(IDS_AUDIO);
		case 12 : return GetResString(IDS_SEARCH_ARC);
		case 13 : return GetResString(IDS_SEARCH_CDIMG);
	}
	return "?";
}

typedef struct
{
	LPCTSTR	pszInitialDir;
	LPCTSTR	pszDlgTitle;
} BROWSEINIT, *LPBROWSEINIT;

bool SelectDir(HWND hWnd, LPTSTR pszPath, LPCTSTR pszTitle, LPCTSTR pszDlgTitle)
{
	BOOL bResult = FALSE;
	CoInitialize(0);
	LPMALLOC pShlMalloc;
	if (SHGetMalloc(&pShlMalloc) == NOERROR)
	{
		BROWSEINFO BrsInfo = {0};
		BrsInfo.hwndOwner = hWnd;
		BrsInfo.lpszTitle = (pszTitle != NULL) ? pszTitle : pszDlgTitle;
		BrsInfo.ulFlags = BIF_VALIDATE | BIF_NEWDIALOGSTYLE | BIF_RETURNONLYFSDIRS | BIF_SHAREABLE | BIF_DONTGOBELOWDOMAIN;

		BROWSEINIT BrsInit = {0};
		if (pszPath != NULL || pszTitle != NULL || pszDlgTitle != NULL){
			// Need the 'BrowseCallbackProc' to set those strings
			BrsInfo.lpfn = BrowseCallbackProc;
			BrsInfo.lParam = (LPARAM)&BrsInit;
			BrsInit.pszDlgTitle = (pszDlgTitle != NULL) ? pszDlgTitle : NULL/*pszTitle*/;
			BrsInit.pszInitialDir = pszPath;
		}

		LPITEMIDLIST pidlBrowse;
		if ((pidlBrowse = SHBrowseForFolder(&BrsInfo)) != NULL){
			if (SHGetPathFromIDList(pidlBrowse, pszPath))
				bResult = TRUE;
			pShlMalloc->Free(pidlBrowse);
		}
		pShlMalloc->Release();
	}
	CoUninitialize();
	return bResult;
}

void MakeFoldername(char* path)
{
//	CString string(path);
//	if(string.GetLength() > 0) 
//	{	
//		if(string.Right(1) == '\\') 
//		{	
//			string = string.Left(string.GetLength()-1);
//		}
//	}
//	sprintf(path, "%s", string);

	CString strPath(path);
	PathCanonicalize(path, strPath);
	PathRemoveBackslash(path);
}

struct SED2KFileType
{
	LPCTSTR pszExt;
	EED2KFileType iFileType;
} _aED2KFileTypes[] = 
{
    { _T(".669"),   ED2KFT_AUDIO },
	{ _T(".aac"),	ED2KFT_AUDIO },
	{ _T(".aif"),	ED2KFT_AUDIO },
	{ _T(".aiff"),	ED2KFT_AUDIO },
    { _T(".amf"),   ED2KFT_AUDIO },
    { _T(".ams"),   ED2KFT_AUDIO },
	{ _T(".ape"),	ED2KFT_AUDIO },
	{ _T(".au"),	ED2KFT_AUDIO },
    { _T(".dbm"),   ED2KFT_AUDIO },
    { _T(".dmf"),   ED2KFT_AUDIO },
    { _T(".dsm"),   ED2KFT_AUDIO },
    { _T(".far"),   ED2KFT_AUDIO },
	{ _T(".flac"),	ED2KFT_AUDIO },
    { _T(".it"),    ED2KFT_AUDIO },
    { _T(".mdl"),   ED2KFT_AUDIO },
    { _T(".med"),   ED2KFT_AUDIO },
	{ _T(".mid"),	ED2KFT_AUDIO },
	{ _T(".midi"),	ED2KFT_AUDIO },
    { _T(".mod"),   ED2KFT_AUDIO },
    { _T(".mol"),   ED2KFT_AUDIO },
	{ _T(".mp1"),	ED2KFT_AUDIO },
	{ _T(".mp2"),	ED2KFT_AUDIO },
	{ _T(".mp3"),	ED2KFT_AUDIO },
	{ _T(".mp4"),	ED2KFT_AUDIO },
	{ _T(".mpa"),	ED2KFT_AUDIO },
	{ _T(".mpc"),	ED2KFT_AUDIO },
	{ _T(".mpp"),	ED2KFT_AUDIO },
    { _T(".mtm"),   ED2KFT_AUDIO },
    { _T(".nst"),   ED2KFT_AUDIO },
	{ _T(".ogg"),	ED2KFT_AUDIO },
    { _T(".okt"),   ED2KFT_AUDIO },
    { _T(".psm"),   ED2KFT_AUDIO },
    { _T(".ptm"),   ED2KFT_AUDIO },
	{ _T(".ra"),	ED2KFT_AUDIO },
	{ _T(".rmi"),	ED2KFT_AUDIO },
    { _T(".s3m"),   ED2KFT_AUDIO },
    { _T(".stm"),   ED2KFT_AUDIO },
    { _T(".ult"),   ED2KFT_AUDIO },
    { _T(".umx"),   ED2KFT_AUDIO },
	{ _T(".wav"),	ED2KFT_AUDIO },
	{ _T(".wma"),	ED2KFT_AUDIO },
    { _T(".wow"),   ED2KFT_AUDIO },
    { _T(".xm"),    ED2KFT_AUDIO },

	{ _T(".asf"),	ED2KFT_VIDEO },
	{ _T(".avi"),	ED2KFT_VIDEO },
	{ _T(".divx"),	ED2KFT_VIDEO },
	{ _T(".m1v"),	ED2KFT_VIDEO },
	{ _T(".m2v"),	ED2KFT_VIDEO },
	{ _T(".mkv"),	ED2KFT_VIDEO },
	{ _T(".mov"),	ED2KFT_VIDEO },
	{ _T(".mp1v"),	ED2KFT_VIDEO },
	{ _T(".mp2v"),	ED2KFT_VIDEO },
	{ _T(".mpe"),	ED2KFT_VIDEO },
	{ _T(".mpeg"),	ED2KFT_VIDEO },
	{ _T(".mpg"),	ED2KFT_VIDEO },
	{ _T(".mps"),	ED2KFT_VIDEO },
	{ _T(".mpv"),	ED2KFT_VIDEO },
	{ _T(".mpv1"),	ED2KFT_VIDEO },
	{ _T(".mpv2"),	ED2KFT_VIDEO },
	{ _T(".ogm"),	ED2KFT_VIDEO },
	{ _T(".qt"),	ED2KFT_VIDEO },
	{ _T(".ram"),	ED2KFT_VIDEO },
	{ _T(".rm"),	ED2KFT_VIDEO },
	{ _T(".rv"),	ED2KFT_VIDEO },
	{ _T(".vivo"),	ED2KFT_VIDEO },
	{ _T(".vob"),	ED2KFT_VIDEO },
	{ _T(".wmv"),	ED2KFT_VIDEO },

	{ _T(".bmp"),	ED2KFT_IMAGE },
	{ _T(".dcx"),	ED2KFT_IMAGE },
	{ _T(".emf"),	ED2KFT_IMAGE },
	{ _T(".gif"),	ED2KFT_IMAGE },
	{ _T(".ico"),	ED2KFT_IMAGE },
	{ _T(".jpeg"),	ED2KFT_IMAGE },
	{ _T(".jpg"),	ED2KFT_IMAGE },
	{ _T(".pct"),	ED2KFT_IMAGE },
	{ _T(".pcx"),	ED2KFT_IMAGE },
	{ _T(".pic"),	ED2KFT_IMAGE },
	{ _T(".pict"),	ED2KFT_IMAGE },
	{ _T(".png"),	ED2KFT_IMAGE },
	{ _T(".psd"),	ED2KFT_IMAGE },
	{ _T(".psp"),	ED2KFT_IMAGE },
	{ _T(".tga"),	ED2KFT_IMAGE },
	{ _T(".tif"),	ED2KFT_IMAGE },
	{ _T(".tiff"),	ED2KFT_IMAGE },
	{ _T(".wmf"),	ED2KFT_IMAGE },
	{ _T(".xif"),	ED2KFT_IMAGE },

	{ _T(".ace"),	ED2KFT_ARCHIVE },
	{ _T(".arj"),	ED2KFT_ARCHIVE },
	{ _T(".gz"),	ED2KFT_ARCHIVE },
	{ _T(".hqx"),	ED2KFT_ARCHIVE },
	{ _T(".lha"),	ED2KFT_ARCHIVE },
	{ _T(".rar"),	ED2KFT_ARCHIVE },
	{ _T(".sea"),	ED2KFT_ARCHIVE },
	{ _T(".sit"),	ED2KFT_ARCHIVE },
	{ _T(".tar"),	ED2KFT_ARCHIVE },
	{ _T(".tgz"),	ED2KFT_ARCHIVE },
	{ _T(".uc2"),	ED2KFT_ARCHIVE },
	{ _T(".zip"),	ED2KFT_ARCHIVE },

	{ _T(".bat"),	ED2KFT_PROGRAM },
	{ _T(".cmd"),	ED2KFT_PROGRAM },
	{ _T(".com"),	ED2KFT_PROGRAM },
	{ _T(".exe"),	ED2KFT_PROGRAM },

	{ _T(".bin"),	ED2KFT_CDIMAGE },
	{ _T(".bwa"),	ED2KFT_CDIMAGE },
	{ _T(".bwi"),	ED2KFT_CDIMAGE },
	{ _T(".bws"),	ED2KFT_CDIMAGE },
	{ _T(".bwt"),	ED2KFT_CDIMAGE },
	{ _T(".ccd"),	ED2KFT_CDIMAGE },
	{ _T(".cue"),	ED2KFT_CDIMAGE },
	{ _T(".dmg"),	ED2KFT_CDIMAGE },
	{ _T(".dmz"),	ED2KFT_CDIMAGE },
	{ _T(".img"),	ED2KFT_CDIMAGE },
	{ _T(".iso"),	ED2KFT_CDIMAGE },
	{ _T(".mdf"),	ED2KFT_CDIMAGE },
	{ _T(".mds"),	ED2KFT_CDIMAGE },
	{ _T(".nrg"),	ED2KFT_CDIMAGE },
	{ _T(".sub"),	ED2KFT_CDIMAGE },
	{ _T(".toast"), ED2KFT_CDIMAGE }

	// To be uncommented after we use the 'Doc' ed2k filetype for search expressions
//	{ _T(".txt"),   ED2KFT_DOCUMENT },
//	{ _T(".nfo"),   ED2KFT_DOCUMENT },
//	{ _T(".diz"),   ED2KFT_DOCUMENT },
//	{ _T(".doc"),   ED2KFT_DOCUMENT },
//	{ _T(".rtf"),   ED2KFT_DOCUMENT },
//	{ _T(".pdf"),   ED2KFT_DOCUMENT },	// double check this!
//	{ _T(".xls"),   ED2KFT_DOCUMENT },	// double check this!
//	{ _T(".html"),  ED2KFT_DOCUMENT },
//	{ _T(".htm"),   ED2KFT_DOCUMENT }
};

int __cdecl CompareE2DKFileType(const void* p1, const void* p2)
{
	return _tcscmp( ((const SED2KFileType*)p1)->pszExt, ((const SED2KFileType*)p2)->pszExt );
}

EED2KFileType GetED2KFileTypeID(LPCTSTR pszFileName)
{
	LPCTSTR pszExt = _tcsrchr(pszFileName, _T('.'));
	if (pszExt == NULL)
		return ED2KFT_ANY;
	CString strExt(pszExt);
	strExt.MakeLower();

	SED2KFileType ft;
	ft.pszExt = strExt;
	ft.iFileType = ED2KFT_ANY;
	const SED2KFileType* pFound = (SED2KFileType*)bsearch(&ft, _aED2KFileTypes, ELEMENT_COUNT(_aED2KFileTypes), sizeof _aED2KFileTypes[0], CompareE2DKFileType);
	if (pFound != NULL)
		return pFound->iFileType;
	return ED2KFT_ANY;
}

// Retuns the ed2k file type term which is to be used in server searches
LPCSTR GetED2KFileTypeSearchTerm(EED2KFileType iFileID)
{
	if (iFileID == ED2KFT_AUDIO)		return "Audio";
	if (iFileID == ED2KFT_VIDEO)		return "Video";
	if (iFileID == ED2KFT_IMAGE)		return "Image";
	if (iFileID == ED2KFT_ARCHIVE || iFileID == ED2KFT_PROGRAM || iFileID == ED2KFT_CDIMAGE) return "Pro";
	if (iFileID == ED2KFT_DOCUMENT)		return "Doc";
	return NULL;
}

// Returns a localized typename, examining the extention of the given filename
CString GetFiletypeByName(LPCTSTR pszFileName)
{
	EED2KFileType iFileType = GetED2KFileTypeID(pszFileName);
	switch (iFileType) {
		case ED2KFT_AUDIO :		return GetResString(IDS_SEARCH_AUDIO);
		case ED2KFT_VIDEO :		return GetResString(IDS_SEARCH_VIDEO);
		case ED2KFT_IMAGE :		return GetResString(IDS_SEARCH_PICS);
		case ED2KFT_ARCHIVE :	return GetResString(IDS_SEARCH_ARC);
		case ED2KFT_PROGRAM :	return GetResString(IDS_SEARCH_PRG);
		case ED2KFT_CDIMAGE :	return GetResString(IDS_SEARCH_CDIMG);
		default:				return GetResString(IDS_SEARCH_ANY);
	}
}


class CED2KFileTypes{
public:
	CED2KFileTypes(){
		qsort(_aED2KFileTypes, ELEMENT_COUNT(_aED2KFileTypes), sizeof _aED2KFileTypes[0], CompareE2DKFileType);
#ifdef _DEBUG
		// check for duplicate entries
		LPCTSTR pszLast = _aED2KFileTypes[0].pszExt;
		for (int i = 1; i < ELEMENT_COUNT(_aED2KFileTypes); i++){
			ASSERT( _tcscmp(pszLast, _aED2KFileTypes[i].pszExt) != 0 );
			pszLast = _aED2KFileTypes[i].pszExt;
		}
#endif
	}
};
CED2KFileTypes theED2KFileTypes; // get the list sorted *before* any code is accessing it


CHAR *stristr(const CHAR *str1, const CHAR *str2)
{
	const CHAR *cp = str1;
	const CHAR *s1;
	const CHAR *s2;

	if (!*str2)
		return (CHAR *)str1;

	while (*cp)
	{
		s1 = cp;
		s2 = str2;

		while (*s1 && *s2 && tolower(*s1) == tolower(*s2))
			s1++, s2++;

		if (!*s2)
			return (CHAR *)cp;

		cp++;
	}

	return NULL;
}

COLORREF LightenColor(COLORREF crColor, int i)
{
	int r = GetRValue(crColor);
	int g = GetGValue(crColor);
	int b = GetBValue(crColor);

	r += i;
	g += i;
	b += i;

	r = min(r, 255);
	r = max(r, 0);
	g = min(g, 255);
	g = max(g, 0);
	b = min(b, 255);
	b = max(b, 0);
	
	return RGB(r,g,b);
}

float GetPercent(uint32 Value, uint32 TotalValue)
{
	float ProcentValue = 0.0f;
		
	if (TotalValue > 0)
		ProcentValue = 100*Value/TotalValue;

	return ProcentValue;
}

int CompareDirectories(const CString& rstrDir1, const CString& rstrDir2)
{
	// use case insensitive compare as a starter
	if (rstrDir1.CompareNoCase(rstrDir2)==0)
	{	
		return 0;
	}

	// if one of the paths ends with a '\' the paths may still be equal from the file system's POV
	CString strDir1(rstrDir1);
	CString strDir2(rstrDir2);
	PathRemoveBackslash(strDir1.GetBuffer());	// remove any available backslash
	strDir1.ReleaseBuffer();
	PathRemoveBackslash(strDir2.GetBuffer());	// remove any available backslash
	strDir2.ReleaseBuffer();
	return strDir1.CompareNoCase(strDir2);		// compare again
}
