//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "server.h"
#include "opcodes.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CServer::CServer(ServerMet_Struct* in_data){	
	port = in_data->port;
	tagcount = 0;
	ip = in_data->ip;
	in_addr host;
	host.S_un.S_addr = ip;
	ipfull = inet_ntoa(host);
	files = 0;
	users = 0;
	m_ePreferences = ePR_LOW;
	ping = 0;
	failedcount = 0; 
	lastpinged = 0;
	lastpingedtime = 0;
	lastdescpingedcount = 0;
	staticservermember=0;
	maxusers=0;
	softfiles = 0;
	hardfiles = 0;
}

CServer::CServer(uint16 in_port, const CString& i_addr){	
	port = in_port;
	tagcount = 0;
	if (inet_addr(i_addr) == INADDR_NONE){
		dynip = i_addr;
		ip = 0;
	}
	else{
		ip = inet_addr(i_addr);
		dynip = "";
	}
	in_addr host;
	host.S_un.S_addr = ip;
	ipfull = inet_ntoa(host);
	files = 0;
	users = 0;
	m_ePreferences = ePR_LOW;
	ping = 0;	
	failedcount = 0; 
	lastpinged = 0;
	lastpingedtime = 0;
	lastdescpingedcount = 0;
	staticservermember=0;
	maxusers=0;
	softfiles = 0;
	hardfiles = 0;
}

// copy constructor
CServer::CServer(const CServer& pOld)
{	
	for(POSITION pos = pOld.taglist.GetHeadPosition(); pos != NULL; pOld.taglist.GetNext(pos))
	{
		CTag* pOldTag = pOld.taglist.GetAt(pos);
		taglist.AddTail(new CTag(*pOldTag));
	}
	port = pOld.port;
	ip = pOld.ip; 
	staticservermember=pOld.IsStaticMember();
	tagcount = pOld.tagcount;
	ipfull = pOld.ipfull;
	files = pOld.files;
	users = pOld.users;
	m_ePreferences = pOld.m_ePreferences;
	ping = pOld.ping;
	failedcount = pOld.failedcount; 
	lastpinged = pOld.lastpinged;
	lastpingedtime = pOld.lastpinged;
	lastdescpingedcount = pOld.lastdescpingedcount;
	maxusers = pOld.maxusers;	
	description = pOld.description;	
	listname = pOld.listname;
	dynip = pOld.dynip;	
	softfiles = pOld.softfiles;
	hardfiles = pOld.hardfiles;
	m_strVersion = pOld.m_strVersion;
}

CServer::~CServer(){		
	for(POSITION pos = taglist.GetHeadPosition(); pos != NULL; taglist.GetNext(pos))
		delete taglist.GetAt(pos);
	taglist.RemoveAll();	
}

bool CServer::AddTagFromFile(CSafeFile& servermet){
	
	CTag* tag = new CTag(servermet);
	switch(tag->GetSpecialTag()){		
					case ST_SERVERNAME:						
						listname = tag->GetStringValue();						
						delete tag;
						break;
					case ST_DESCRIPTION:						
						description = tag->GetStringValue();						
						delete tag;
						break;
					case ST_PREFERENCE:
						// Modified by Tarod [thx Hein] - import server preferences and translate it
						m_ePreferences = (eServerPriority)ed2k2eMule(tag->GetIntValue());
						delete tag;
						break;
					case ST_PING:
						ping = tag->GetIntValue();
						delete tag;
						break;
					case ST_DYNIP:						
						dynip = tag->GetStringValue();						
						delete tag;
						break;
					case ST_FAIL:
						failedcount = tag->GetIntValue();
						delete tag;
						break;
					case ST_LASTPING:
						lastpingedtime = tag->GetIntValue();
						delete tag;
						break;
					case ST_MAXUSERS:
						maxusers = tag->GetIntValue();
						delete tag;
						break;
					case ST_SOFTFILES:
						softfiles = tag->GetIntValue();
						delete tag;
						break;
					case ST_HARDFILES:
						hardfiles = tag->GetIntValue();
						delete tag;
						break;
					case ST_VERSION:
						if (tag->GetType() == 2)
							m_strVersion = tag->GetStringValue();
						delete tag;
						break;
					default:
						if (tag->GetSpecialTag()){
							tag->SetTagName(_T("Unknown"));
							AddTag(tag);
						}
						else if (tag->GetTagName() == _T("files")){
							files = tag->GetIntValue();
							delete tag;
						}
						else if (tag->GetTagName() == _T("users")){
							users = tag->GetIntValue();
							delete tag;
						}
						else
							AddTag(tag);
	}
	return true;
}

void CServer::FillWindowTags(CTreeCtrl* wnd,HTREEITEM rootitem){
	POSITION pos;
	CString buffer;
	if (!description.IsEmpty()) {
		buffer.Format(GetResString(IDS_DESC),description);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	buffer.Format(GetResString(IDS_IP)+_T(": %s"),ipfull);
	wnd->InsertItem(buffer,-1,-1,rootitem);
	buffer.Format(GetResString(IDS_PORT)+_T(": %i"),port);
	wnd->InsertItem(buffer,-1,-1,rootitem);
	if (ping) {
		buffer.Format(GetResString(IDS_PING)+_T(": %i"),ping);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	if (users) {
		buffer.Format(GetResString(IDS_USERS),users);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	if (files) {
		buffer.Format(GetResString(IDS_FILES)+_T(": %i"),files);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	CTag* cur_tag;
	for(int i = 0; i != taglist.GetCount();i++) {
		pos = taglist.FindIndex(i);
		cur_tag = taglist.GetAt(pos);
		if (cur_tag->GetSpecialTag())
			continue;
		if (cur_tag->GetType() == 2)
			buffer.Format(_T("%s: %s"),cur_tag->GetTagName(),cur_tag->GetStringValue());
		else if (cur_tag->GetType() == 3)
			buffer.Format(_T("%s: %i"),cur_tag->GetTagName(),cur_tag->GetIntValue());
		else
			continue;
		wnd->InsertItem(buffer,-1,-1,rootitem);
   }
}

CString CServer::GetAddress() const {
	if (!dynip.IsEmpty())
		return dynip;
	else
		return ipfull;
}

void CServer::SetID(uint32 newip){
	ip = newip;
	in_addr host;
	host.S_un.S_addr = ip;
	ipfull = inet_ntoa(host);
}

bool CServer::HasSameAddress(const CServer& server) const {

	if ((ip == server.ip) && ((port == server.port) || port == 0 || server.port == 0))
	{
		if (ip != 0)
			return true;
		else if (dynip == server.dynip)
			return true;
	}
	return false;
}

/**
 * convert eDonkey pref values to eMule pref values
 * The correct server preferences (as used by eDonkey) are: Low=2, Normal=0, High=1 
 * @param srvPref preference as found in server.met
 * @return preference as used in eMule
 */
uint32 CServer::ed2k2eMule(uint32 srvPref)
{
	uint32 mPref;
	switch (srvPref)
	{
	case 0:
		mPref = PR_NORMAL;
		break;
	case 1:
		mPref = PR_HIGH;
		break;
	case 2:
		mPref = PR_LOW;
		break;
	default:
		mPref = PR_LOW;
		break;
	}

	return mPref;
}

/**
 * convert eMule pref values to eDonkey pref values
 * The correct server preferences (as used by eDonkey) are: Low=2, Normal=0, High=1 
 * @param srvPref preference as found in eMule
 * @return preference as used in server.met
 */
uint32 CServer::eMule2ed2k(uint32 srvPref)
{
	uint32 dPref;
	switch (srvPref)
	{
	case PR_NORMAL:
		dPref = 0;
		break;
	case PR_HIGH:
		dPref = 1;
		break;
	case PR_LOW:
		dPref = 2;
		break;
	default:
		dPref = 2;
		break;
	}

	return dPref;
}


bool CServer::HasPublicAddress() const
{
	in_addr host;
	host.S_un.S_addr = ip;

	int b1 = host.S_un.S_un_b.s_b1;
	int b2 = host.S_un.S_un_b.s_b2;
		
	if (b1 == 172 && (b2 >= 16 && b2 <= 31))
		return false;

	if (b1 == 192 && b2 == 168)
		return false;

	if (b1 == 0 || b1 == 10 || b1 == 127 || GetPort() == 0)
		return false;

	return true;
}