//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "stdafx.h"
#include "types.h"
#ifdef OLD_SOCKETS_ENABLED
#include "listensocket.h"
#endif //OLD_SOCKETS_ENABLED
#include "partfile.h"
#include "clientcredits.h"
#include "safefile.h"
#include "BarShader.h"
#include "Loggable.h"
#include <vector>
#include <list>

// m_byChatstate
#define	MS_NONE				0
#define	MS_CHATTING			1
#define	MS_CONNECTING		2
#define	MS_UNABLETOCONNECT	3

enum ESecureIdentState{
	IS_UNAVAILABLE		= 0,
	IS_ALLREQUESTSSEND  = 0,
	IS_SIGNATURENEEDED	= 1,
	IS_KEYANDSIGNEEDED	= 2,
};
enum EInfoPacketState{
	IP_NONE				= 0,
	IP_EDONKEYPROTPACK  = 1,
	IP_EMULEPROTPACK	= 2,
	IP_BOTH				= 3,
};


//--- xrmb:hashthieves1 ---
typedef CMap <uint64, uint64, uint32, uint32> t_offensecounter;
extern t_offensecounter offensecounter;
typedef CMap <uint64, uint64, uint64, uint64> t_hashbase;
extern t_hashbase hashbase;
//--- :xrmb ---

#ifdef OLD_SOCKETS_ENABLED
class CClientReqSocket;
#endif //OLD_SOCKETS_ENABLED
class CFriend;

class CUpDownClient : public CLoggable
{
	friend class CUploadQueue;
public:
	//base
#ifdef OLD_SOCKETS_ENABLED
	CUpDownClient(CClientReqSocket* sender = 0);
#endif //OLD_SOCKETS_ENABLED
	CUpDownClient(uint16 in_port, uint32 in_userid, uint32 in_serverup, uint16 in_serverport, CPartFile* in_reqfile);
	~CUpDownClient();
	void			Disconnected();
	bool			TryToConnect(bool bIgnoreMaxCon = false);	// SLUGFILLER: SafeDisconnect
	void			ConnectionEstablished();
	uint32			GetUserID()	const			{return m_nUserID;}
	CString			GetUserName() const			{return m_pszUsername;}
	uint32			GetIP()	const				{return m_dwUserIP;}
	bool			HasLowID() const			{return m_bIsLowID;}
	CString			GetFullIP() const 			{return m_szFullUserIP;}
	uint32			GetUserPort() const			{return m_nUserPort;}
	uint32			GetTransferedUp() const		{return m_nTransferedUp;}
	uint32			GetTransferedDown() const	{return m_nTransferedDown;}
	uint32			GetServerIP() const				{return m_dwServerIP;}
	void				SetServerIP(uint32 nIP)			{m_dwServerIP = nIP;}
	uint16			GetServerPort()	const		{return m_nServerPort;}
	void				SetServerPort(uint16 nPort)		{m_nServerPort = nPort;}
	uchar*			GetUserHash()				{return (uchar*)m_achUserHash;}
	bool			HasValidHash() const		{return ((int*)m_achUserHash[0]) != 0 || ((int*)m_achUserHash[1]) != 0 ||
												        ((int*)m_achUserHash[2]) != 0 || ((int*)m_achUserHash[3]) != 0; }
	void			SetUserHash(uchar* m_achTempUserHash);
	uint32			GetVersion() const			{return m_nClientVersion;}
	int				GetHashType();
	//--- xrmb:hashthieves1 ---
	bool  hashthief;
	uint64  getUID() { return ((uint64)m_dwUserIP<<32)+((uint64)m_nUserPort<<16); }
	//--- :xrmb ---

	uint8			GetMuleVersion() const		{return m_byEmuleVersion;}
	CString			GetModVersion()				{return m_modVerString;}	//StaTiK - mods tag
	bool			ExtProtocolAvailable()		{return m_bEmuleProtocol;}
	bool			TarodVersion()				{return (m_byDataTarodVer == 1 || m_byDataTarodVer == 2);} // Added by Tarod
	uint32			PlusVersion()				{return m_dwDataPlusVers;}
	bool			IsEmuleClient()				{return m_byEmuleVersion;}
	CClientCredits* Credits()					{return credits;}
	bool			IsBanned(bool bIgnoreDS = false) const		{return (m_bBanned && (bIgnoreDS || m_nDownloadState != DS_DOWNLOADING));}
	CString			GetClientFilename() const	{return m_strClientFilename;}
	bool			SupportsUDP()				{return m_byUDPVer != 0 && m_nUDPPort != 0;}
	uint16			GetUDPPort()				{return m_nUDPPort;}
	void			SetUDPPort(uint16 nPort)	{ m_nUDPPort = nPort; }
	uint8			GetUDPVersion()				{return m_byUDPVer;}
	uint8			GetExtendedRequestsVersion(){return m_byExtendedRequestsVer;}
	bool			IsFriend()	const			{return m_Friend != NULL;}
	float			GetCompression()	{return (float)compressiongain/notcompressed*100.0f;} // Add rod show compression
	void			ResetCompressionGain() {compressiongain = 0; notcompressed=1;} // Add show compression

	void			RequestSharedFileList();
	void			ProcessSharedFileList(char* pachPacket, uint32 nSize, LPCTSTR pszDirectory = NULL);
	
	void    SetUserID(uint32 newUserID) {		//netwolf 08.05.03 made public for OP_CLIENT_CHANGE_ID //SyruS moved-up to "base"
		m_nUserID = newUserID;
        m_bIsLowID = (m_nUserID < 16777216);
    }
	void			SetUserName(const CString& pszNewName) { m_pszUsername = pszNewName;}
	bool            IsCommunity();  	// i_a                
	void            ResetValueCache(); 	// i_a                
	eClientTypes	GetClientSoft()				{return m_clientSoft;}
	void			ReGetClientSoft();
	void			ProcessHelloAnswer(char* pachPacket, uint32 nSize);
	void			ProcessHelloPacket(char* pachPacket, uint32 nSize);
	void			SendHelloAnswer();
	void			SendHelloPacket();
	void			SendMuleInfoPacket(bool bAnswer);
	void			ProcessMuleInfoPacket(char* pachPacket, uint32 nSize);
	void			ProcessMuleCommentPacket(char* pachPacket, uint32 nSize);
	bool			Compare(CUpDownClient* tocomp);
	void			SetLastSrcReqTime()			{m_dwLastSourceRequest = ::GetTickCount();}
	void			SetLastSrcAnswerTime()		{m_dwLastSourceAnswer = ::GetTickCount();}
	uint32			GetLastSrcReqTime()			{return m_dwLastSourceRequest;}
	uint32			GetLastSrcAnswerTime()		{return m_dwLastSourceAnswer;}
	uint32			GetEnteredConnectedState()	{return m_dwEnteredConnectedState;}
	bool			GetFriendSlot()				{return m_bFriendSlot;}
	void			SetFriendSlot(bool bNV)		{m_bFriendSlot = bNV;}
	void			SetCommentDirty(bool bDirty = true) {m_bCommentDirty = bDirty;}
	uint8			GetSourceExchangeVersion()	{return m_bySourceExchangeVer;}

	void			SendPublicKeyPacket();
	void			SendSignaturePacket();
	void			ProcessPublicKeyPacket(uchar* pachPacket, uint32 nSize);
	void			ProcessSignaturePacket(uchar* pachPacket, uint32 nSize);
	uint8			GetSecureIdentState()		{return m_SecureIdentState;}
	void			SendSecIdentStatePacket();
	void			ProcessSecIdentStatePacket(uchar* pachPacket, uint32 nSize);
	void			InfoPacketsReceived();
	void			ResetFileStatusInfo();

#ifdef OLD_SOCKETS_ENABLED
	CClientReqSocket*	socket;
#endif //OLD_SOCKETS_ENABLED
	CClientCredits*		credits;
	CFriend*			m_Friend;
	//upload
	uint32	compressiongain; /// Add show compression
	uint32  notcompressed; // Add show compression
	eULQState		GetUploadState()			{return m_byUploadState;}
	void			SetUploadState(eULQState news)	{m_byUploadState = news;}
	uint32			GetWaitStartTime()			{return m_dwWaitTime;}
	uint32			GetWaitTime()				{return m_dwUploadTime-m_dwWaitTime;}
	bool			IsDownloading()				{return (m_byUploadState == US_UPLOADING);}
	bool			HasBlocks()					{return !(m_BlockSend_queue.IsEmpty() && m_BlockRequests_queue.IsEmpty());}
	sint32			GetDatarate()				{return m_nUpDatarate;}	
	uint32			GetScore(bool sysvalue, bool isdownloading = false, bool onlybasevalue = false);
	float			GetRareFilePushRatio(void) ; // Added by Tarod
	float			GetSmallFilePushRatio(void) ; // Added by Tarod
	void			AddReqBlock(Requested_Block_Struct* reqblock);
	bool			CreateNextBlockPackage();
	void			SetUpStartTime(uint32 dwTime = 0);
	uint32			GetUpStartTimeDelay()		{return ::GetTickCount() - m_dwUploadTime;}
	void			SetWaitStartTime(uint32 dwTime = 0);
	void			SendHashsetPacket(char* forfileid);
	uchar			reqfileid[16];
	void			SetUploadFileID(uchar* tempreqfileid);
	uchar*			GetUploadFileID()	{return reqfileid;}
	uint32			SendBlockData(uint32 maxammount, bool& wantsMoreBandwidth);
	void			ClearUploadBlockRequests();
	void			SendRankingInfo();
	void			SendCommentInfo(CKnownFile *file);
	uint32			GetLastAskedDelay();
	void			AddRequestCount(uchar* fileid);
	bool 			IsDifferentPartBlock(bool silent=false);
#if JUMPSTART
    bool            IsForbiddenByJumpstart();
#endif // JUMPSTART
    void			UnBan();
	void			Ban(LPCTSTR line, const CString& reason,...); //katsyonak 16.06.03
	uint32			GetBanTime()				{return m_dwBanTime;}
	uint32			GetAskedCount()				{return m_cAsked;} //<<--
	void			AddAskedCount()				{m_cAsked++;}
	void			SetAskedCount( uint32 m_cInAsked)	{m_cAsked = m_cInAsked;}
	void			FlushSendBlocks();			// call this when you stop upload, or the socket might be not able to send
	void			SetLastUpRequest()			{m_dwLastUpRequest = ::GetTickCount();}
	uint32			GetLastUpRequest()			{return m_dwLastUpRequest;}
	void			UDPFileReasked();

	uint32			GetSessionUp()			{return m_nTransferedUp - m_nCurSessionUp;}
	void			ResetSessionUp()		{m_nCurSessionUp = m_nTransferedUp;} 
    void            SetLastGotULData()         { m_LastGotULData = ::GetTickCount(); };
    DWORD           GetLastGotULData()         { return m_LastGotULData; };
    uint32			GetCurrentlyUploadingPart();

	// remote client's parts
	void			ProcessUpFileStatus(char* packet,uint32 size);
	uint16			GetUpPartCount()			{return m_nUpPartCount;}
	uint16			GetAvailUpPartCount()			{return m_nAvailUpPartCount;}
	void			DrawUpStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat);
    void            CheckForUpPartsInfo();

	//eklmn: file request counter
	uint8		GetFailedFileRequests()			{return m_cFailFileRequests;}
	void			UpdateFailedFileRequests()		{m_cFailFileRequests++;}
	void			ResetFailedFileRequests()		{m_cFailFileRequests = 0;}


	//download
	uint32			GetAskedCountDown()				{return m_cDownAsked;} //<<--
	void			AddAskedCountDown()				{m_cDownAsked++;}
	void			SetAskedCountDown( uint32 m_cInDownAsked)				{m_cDownAsked = m_cInDownAsked;}
	eDLQState		GetDownloadState()			{return m_nDownloadState;}
	void			SetDownloadState(eDLQState byNewState);
	uint32			GetLastAskedTime()			{return m_dwLastAskedTime;}
	bool			IsPartAvailable(uint16 iPart)	{return	( (iPart >= m_nPartCount) || (!m_abyPartStatus) )? 0:m_abyPartStatus[iPart];} 	
	uint8*			GetPartStatus()				{return m_abyPartStatus;}
	uint32			GetDownloadDatarate()		{return m_nDownDatarate;}
	uint16			GetRemoteQueueRank()		{return m_nRemoteQueueRank;}
	uint16   		GetDifference()    			{return m_nDifferenceQueueRank;} //cendre-eau
	void			SetRemoteQueueFull( bool flag )	{m_bRemoteQueueFull = flag;}
	bool			IsRemoteQueueFull()			{return m_bRemoteQueueFull;}
	void			SetRemoteQueueRank(uint16 nr);
	void			DrawStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat);
	bool			AskForDownload();
	void			SendFileRequest();
//	void			SendFileRequestForUpload();
	void			ProcessFileInfo(char* packet,uint32 size);
	void			ProcessFileStatus(char* packet,uint32 size);
	void			ProcessHashSet(char* packet,uint32 size);
	bool			AddRequestForAnotherFile(CPartFile* file);
	void			SendBlockRequests();
	void			ProcessBlockPacket(char* packet, uint32 size, bool packed = false);
	uint32		CalculateDownloadRate();
	//void			ResetMeasurementPeriodTimer(uint32 time)	{m_dwLastTimeCalculatedDR = time;}	
	uint16		GetAvailablePartCount();
	bool			SwapToAnotherFile(CPartFile* SwapTo, bool bIgnoreNoNeeded = false);     // eklmn: bugfix(08): A4AF switching
	uint32		GetQueueWaitTime()		{return ::GetTickCount() - m_dwQueueWaitTime;} //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
	void			StartQueueWaitTimer()	{m_dwQueueWaitTime = ::GetTickCount();}	 //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
	void			UDPReaskACK(uint16 nNewQR);
	void			UDPReaskFNF();
	void			UDPReaskForDownload();
	bool			IsSourceRequestAllowed();
	uint32			GetRemoteScore(); //Maverick
	uint16			GetRemoteRatio(); //Maverick
	float			GetRemoteBaseModifier(); //Maverick
	bool			GetRemoteCommunity(){return (m_RemoteInfoAvaiable && m_RemoteCommunity);} //Maverick
	bool			GetRemoteCommunityEnabled(){return (m_RemoteInfoAvaiable && m_RemoteCommunityEnabled);} //Maverick
	void			SetRemoteScore(uint32 Score){m_RemoteScore = Score; m_RemoteInfoAvaiable=true;} //Maverick
	void			SetRemoteRatio(uint16 Ratio){m_RemoteRatio = Ratio; m_RemoteInfoAvaiable=true;} // Maverick
	void			SetRemoteBaseModifier(uint8 BaseModifier){m_RemoteBaseModifier = BaseModifier * 10; m_RemoteInfoAvaiable=true;} // Maverick
	void			SetRemoteCommunity(bool Community){m_RemoteCommunity = Community; m_RemoteInfoAvaiable=true;} //Maverick
	void			SetRemoteCommunityEnabled(bool CommunityEnabled){m_RemoteCommunityEnabled = CommunityEnabled; m_RemoteInfoAvaiable=true;} //Maverick
	bool			IsRemoteInfoAvaiable(){return m_RemoteInfoAvaiable;} //Maverick
	void			SetRemoteClientSpeed(uint16 speed){m_RemoteClientSpeed = speed;} //Maverick
	uint16			GetRemoteClientSpeed(){return m_RemoteClientSpeed;} // Maverick
	bool			IsCompleteSource()			{return m_bCompleteSource;}		// netwolf: complete sources (zegzav) 13.05.03 
	uint16			GetUpCompleteSourcesCount()	{return m_nUpCompleteSourcesCount;}		// netwolf: complete sources (zegzav) 13.05.03
	uint32			GetUpCompleteSourcesTime()	{return m_nUpCompleteSourcesTime;}
	void			SetUpCompleteSourcesCount(uint16 n)	{m_nUpCompleteSourcesCount= n; m_nUpCompleteSourcesTime= ::GetTickCount();}

	CString			GetDownloadStateAsString(void); // Lord KiRon

	CPartFile*		reqfile;
	int				sourcesslot;

private:
	bool IsStolenName() const;

public:
	bool IsLeecherType() const;
	
	uint16 GetLastDownPart() {return m_LastPartAsked;};
	void   SetLastDownPart(uint32 last_part) {m_LastPartAsked=last_part;};
		
	//returns the count of avaible parts that this user has but we not have
	uint16	GetAvailableOfNeededPartsCount(); //added by Mr.Fry

	//chat
	uint8			GetChatState()				{return m_byChatstate;}
	void			SetChatState(uint8 nNewS)	{m_byChatstate = nNewS;}

	//File Comment 
	CString         GetFileComment()         	{return m_strComment;} 
	void		   	SetFileComment(const CString& desc) 	{m_strComment = desc;}
	uint8       	GetFileRate()            	{return m_iRate;}
	void		   	SetFileRate(int8 iNewRate) 	{m_iRate=iNewRate;}

	// Barry - Process zip file as it arrives, don't need to wait until end of block
	int unzip(Pending_Block_Struct *block, BYTE *zipped, uint32 lenZipped, BYTE **unzipped, uint32 *lenUnzipped, bool recursive = false);
	void ShowDownloadingParts(std::_Bvector& parts);
	//START enkeyDEV(kei-kun) -Messaging preferences-	
	uint32			GetAwayMessageResendCount()				{return m_awayMessageResendCount;};
	void			SetAwayMessageResendCount(uint32 newVal) {m_awayMessageResendCount = newVal;};
	//END enkeyDEV(kei-kun) -Messaging preferences-
	void UpdateDisplayedInfo(boolean force=false);

    int             GetFileListRequested() { return m_iFileListRequested; }
    void            SetFileListRequested(int iFileListRequested) { m_iFileListRequested = m_iFileListRequestedSave = iFileListRequested; }

	bool isOnLAN()						{return b_IsOnLan;};
	uint16 m_LastPartAsked; //jicxicmic

	void CleanOtherRequests()	{m_OtherRequests_list.RemoveAll();m_OtherNoNeeded_list.RemoveAll();}	//Cax2 - remove sources bugfix
	CString			GetUploadFileInfo();

	// START enkeyDEV(th1) -L2HAC-
	uint32 GetL2HACTime()							{return m_L2HAC_time ? (m_L2HAC_time - L2HAC_CALLBACK_PRECEDE) : 0;}
	void SetLastL2HACExecution(uint32 m_set_to = 0)	{m_last_l2hac_exec = m_set_to ? m_set_to : ::GetTickCount();}
	uint32 GetLastL2HACExecution()					{return m_last_l2hac_exec;}
	void EnableL2HAC()								{m_l2hac_enabled = true;}
	void DisableL2HAC()								{m_l2hac_enabled = false;}
	bool IsL2HACEnabled()							{return m_l2hac_enabled;}
	// END enkeyDEV(th1) -L2HAC-

	//BEGIN Improved Tooltips
	CString			GetUploadFileInfo4Tooltips();
	//END Improved Tooltips
	CTypedPtrList<CPtrList, CPartFile*>	m_OtherRequests_list;  //eklmn: bugfix(09): hang up by start if all files are paused(???)
	CTypedPtrList<CPtrList, CPartFile*>	m_OtherNoNeeded_list;

private:
	// base
	void	Init();
	void	ProcessHelloTypePacket(CSafeMemFile& data);
	void	SendHelloTypePacket(CMemFile& data);
	float	GetClientSpeed();		// Maverick
	//bool		m_bIsBotuser;		//Cax2: removed punishment 
	bool	isfriend;				// Tarod (herbert code)
	bool    isemuleclient;   		// Tarod (herbert code) 
	bool    iscommunity;    		// DonGato 
    bool    community_cached;    	// i_a: CPU load reduction 
    bool    score_base_cached;      // i_a: CPU load reduction 
	bool	b_IsOnLan;
	uint32	m_dwUserIP;
	uint32	m_dwServerIP;
	uint32	m_nUserID;
    bool    m_bIsLowID;
	uint16	m_nUserPort;
	uint16	m_nServerPort;
	uint32	m_nClientVersion;
	sint32	m_nUpDatarate;
	//uint32	m_cSendblock;
	uint8	m_byEmuleVersion;
	uint8	m_byDataCompVer;
	uint8	m_byDataTarodVer; // Added by Tarod (just curiosity)
	uint32   m_dwDataPlusVers;
	CString m_modVerString;		 //Cax2 - mods tag
	CString m_modExtendedString;
	bool	m_bEmuleProtocol;
	CString	m_pszUsername;
	CString	m_szFullUserIP;
	char	m_achUserHash[16];
	uint16	m_nUDPPort;
	uint8	m_byUDPVer;
	uint8	m_bySourceExchangeVer;
	uint8	m_byAcceptCommentVer;
	uint8	m_byExtendedRequestsVer;
	uint8	m_cFailed;
	eClientTypes	m_clientSoft;
	uint32	m_dwLastSourceRequest;
	uint32	m_dwLastSourceAnswer;
	uint32	m_dwEnteredConnectedState;
    int     m_iFileListRequested;
	int     m_iFileListRequestedSave;
	bool	m_bFriendSlot;
	bool	m_bCommentDirty;
	bool	m_bIsHybrid;
	uint32	m_byCompatableClient;
	bool	m_bIsML;				// Moosetea, from official

	uint32	m_RemoteScore; //By Maverick
	uint16	m_RemoteRatio; //By Maverick
	uint16	m_RemoteBaseModifier; //By Maverick
	bool	m_RemoteInfoAvaiable; //By Maverick
	bool	m_RemoteCommunity; //By Maverick
	bool	m_RemoteCommunityEnabled; // By Maverick
	uint16	m_RemoteClientSpeed;
	CTypedPtrList<CPtrList, Packet*>		 m_WaitingPackets_list;
	DWORD	m_lastRefreshedDLDisplay;

	ESecureIdentState	m_SecureIdentState;
	uint8	m_byInfopacketsReceived;			// have we received the edonkeyprot and emuleprot packet already (see InfoPacketsReceived() )
	uint32	m_dwLastSignatureIP;
	uint8	m_bySupportSecIdent;


	//upload
	// -khaos--+++> Added parameters: bool bFromPF = true
	void 	CreateStandartPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock, bool bFromPF = true);
	void 	CreatePackedPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock, bool bFromPF = true);
	void 	splitAndAddPacketsToSendQueue(Packet* packet, bool bFromPF);
	// <-----khaos-
	bool		m_bBanned;
	uint32		m_nTransferedUp;
	eULQState	m_byUploadState;
	uint32		m_dwWaitTime;
	uint32		m_dwUploadTime;
	uint32		m_nMaxSendAllowed;
	uint32		m_nAvUpDatarate;
	uint32		m_cAsked; //<<--
	uint32		m_dwLastUpRequest;
	uint32		m_dwBanTime;
	int16		filepriority;	// Added by Tarod (herbert code)
	bool		m_bUsedComprUp;	//only used for interface output
	uint32		m_nCurSessionUp; // VQB add for UL slot control
	std::list<int>				m_AverageUDR_list;	//Cax2 - memory optimization
	std::list<DWORD>		m_Average_UL_tick_list;	//Cax2 - memory optimization
	CTypedPtrList<CPtrList, Packet*>		 			m_BlockSend_queue;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> 	m_BlockRequests_queue;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> 	m_DoneBlocks_list;
	CTypedPtrList<CPtrList, Requested_File_Struct*>	m_RequestedFiles_list;
    DWORD       m_lastRefreshedULDisplay;
    uint32 m_LastGotULData;

	// remote client's parts
	uint16		m_nUpPartCount;
	uint16		m_nAvailUpPartCount;
	uint16		m_nUpCompleteSourcesCount;	//netwolf: complete sources (zegzav) 13.05.03
	uint32		m_nUpCompleteSourcesTime;
	static		CBarShader s_UpStatusBar;
	public:
	uint8*		m_abyUpPartStatus;

private:
	// eklmn: file request counter
	uint8	m_cFailFileRequests;
	//download
	bool		m_bRemoteQueueFull;
	bool		usedcompressiondown; //only used for interface output
	eDLQState	m_nDownloadState;
	uint16		m_nPartCount;
	uint32		m_cDownAsked;
	uint8*		m_abyPartStatus;
	uint32		m_dwLastAskedTime;
	CString		m_strClientFilename;
	uint32		m_nTransferedDown;
	// -khaos--+++> Download Session Stats
	bool		m_bTransferredDownMini;
	uint32		m_dwDownStartTime;
	// <-----khaos-
	uint32      m_nLastBlockOffset;   //<--NEW Cax2 Packet ? from
	uint32		m_nDownDatarate;
	uint32		m_nAvDownDatarate;
	//uint16		m_cShowDR;
	uint32		m_dwLastBlockReceived;
	uint16		m_nRemoteQueueRank;
	uint16		m_nDifferenceQueueRank; //cendre-eau
	bool		m_bCompleteSource;
	bool		m_bReaskPending;
	bool		m_bUDPPending;
    uint32		m_dwQueueWaitTime; //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
    	//download speed measuremet
	std::list<int>			m_Average_DL_DR_list;		//Cax2 - memory optimization
	std::list<DWORD>		m_Average_DL_tick_list;		//Cax2 - memory optimization
	uint64				m_Client_Sum_DL_DR_over_last_N_measurements;
	uint64				m_Client_Sum_DL_tick_over_last_N_measurements;
	uint32				m_dwLastTimeCalculatedDR;
	uint32				m_TransferedInLastPeriod;
	//DWORD       m_lastCalculatedDownloadRate;

	CTypedPtrList<CPtrList, Pending_Block_Struct*>	 	m_PendingBlocks_list;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> 	m_DownloadBlocks_list;


//	uint32	sumavgddr; // enkeyDEV(Ottavio84) -Accurate speed measurement-
//	uint32	sumavgudr; // enkeyDEV(Ottavio84) -Accurate speed measurement-

	static CBarShader s_StatusBar;
	// chat
	uint8 m_byChatstate;
	CString m_strComment;
	int8 m_iRate; 
	uint32 m_awayMessageResendCount; //<<-- enkeyDEV(kei-kun) -Messaging preferences-

	// START enkeyDEV(th1) -L2HAC-
	uint32 m_L2HAC_time;
	uint32 m_last_l2hac_exec;
	bool m_l2hac_enabled;
	// END enkeyDEV(th1) -L2HAC-

	bool m_fSharedDirectories; // client supports OP_ASKSHAREDIRS opcodes
};
